<?php
/**
 * 1997-2018 Quadra Informatique
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0) that is available
 * through the world-wide-web at this URL: http://www.opensource.org/licenses/OSL-3.0
 * If you are unable to obtain it through the world-wide-web, please send an email
 * to modules@quadra-informatique.fr so we can send you a copy immediately.
 *
 * @author    Quadra Informatique <modules@quadra-informatique.fr>
 * @copyright 1997-2018 Quadra Informatique
 * @license   http://www.opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 *
 * @category    Bnp
 * @package     Bnpparibas_Axepta
 */

namespace Bnpparibas\Axepta\Block\Adminhtml\MerchantAccount;

use Magento\Backend\Block\Widget\Form\Container;

class Edit extends Container
{
    /**
     * Core registry
     *
     * @var \Magento\Framework\Registry
     */
    protected $coreRegistry = null;

    /**
     * @param \Magento\Backend\Block\Widget\Context $context
     * @param \Magento\Framework\Registry $registry
     * @param array $data
     */
    public function __construct(
        \Magento\Backend\Block\Widget\Context $context,
        \Magento\Framework\Registry $registry,
        array $data = []
    ) {
        $this->coreRegistry = $registry;
        parent::__construct($context, $data);
    }

    /**
     * Merchant Configuration Account edit block
     *
     * @return void
     */
    protected function _construct()
    {
        $this->_objectId = 'id';
        $this->_blockGroup = 'Bnpparibas_Axepta';
        $this->_controller = 'adminhtml_merchantAccount';

        parent::_construct();
    }

    /**
     * Prepare the layout.
     *
     * @return $this
     */
    protected function _prepareLayout()
    {
        if ($this->_isAllowedAction('Bnpparibas_Axepta::merchant_account_save')) {
            $this->buttonList->add(
                'saveandcontinue',
                [
                    'label' => __('Save and Continue Edit'),
                    'class' => 'save primary',
                    'data_attribute' => [
                        'mage-init' => [
                            'button' => ['event' => 'saveAndContinueEdit', 'target' => '#edit_form'],
                        ],
                    ],
                ],
                10
            );

            $this->buttonList->update('save', 'label', __('Save Merchant Account'));
            $this->buttonList->update('save', 'class', 'save');
        } else {
            $this->buttonList->remove('save');
        }
        $this->buttonList->remove('reset');
        $this->buttonList->remove('back');
        $this->buttonList->add(
            'my_back_button',
            [
                'label' => __('Back'),
                'onclick' => 'setLocation(\'' . $this->getUrl('*/paymentsetup/') . '\')',
                'class' => 'back'
            ],
            -1
        );


        $id = $this->getRequest()->getParam('id');
        if (!$id) {
            $this->buttonList->remove('save');
        }

        return parent::_prepareLayout();
    }

    /**
     * Get header with Merchant Configuration Account Login
     *
     * @return \Magento\Framework\Phrase
     */
    public function getHeaderText()
    {
        if ($this->coreRegistry->registry('axepta_merchant_account')->getId()) {
            return __("Update");
        } else {
            return __('Add');
        }
    }

    /**
     * Check permission for passed action
     *
     * @param string $resourceId
     * @return bool
     */
    protected function _isAllowedAction($resourceId)
    {
        return true;
        return $this->_authorization->isAllowed($resourceId);
    }

    /**
     * Getter of url for "Save and Continue" button
     * tab_id will be replaced by desired by JS later
     *
     * @return string
     */
    protected function _getSaveAndContinueUrl()
    {
        return $this->getUrl('axepta/*/save', ['_current' => true, 'back' => 'edit', 'active_tab' => '']);
    }
}
