<?php
/**
 * 1997-2018 Quadra Informatique.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0) that is available
 * through the world-wide-web at this URL: http://www.opensource.org/licenses/OSL-3.0
 * If you are unable to obtain it through the world-wide-web, please send an email
 * to modules@quadra-informatique.fr so we can send you a copy immediately.
 *
 * @author    Quadra Informatique <modules@quadra-informatique.fr>
 * @copyright 1997-2018 Quadra Informatique
 * @license   http://www.opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 *
 * @category    Bnp
 */

namespace Bnpparibas\Axepta\Block\Adminhtml\MerchantAccount\Edit\Tab;

/**
 * Cms page edit form main tab.
 */
class GeneralConfiguration extends \Magento\Backend\Block\Widget\Form\Generic implements \Magento\Backend\Block\Widget\Tab\TabInterface
{
    /**
     * @var \Magento\Store\Model\System\Store
     */
    protected $systemStore;

    /**
     * @var \Magento\Config\Model\Config\Source\Yesno
     */
    protected $yesno;

    protected $stores;
    protected $countries;
    protected $currencies;
    protected $displayMethod;
    protected $captureMode;
    protected $helper;
    protected $cetelemCategories;

    /**
     * @var \Bnpparibas\Axepta\Model\Api\Methods
     */
    protected $methods;

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $scopeConfig;

    protected $model;

    protected $coreRegistry;

    public function __construct(
        \Magento\Backend\Block\Template\Context $context,
        \Magento\Framework\Registry $registry,
        \Magento\Framework\Data\FormFactory $formFactory,
        \Magento\Store\Model\System\Store $systemStore,
        \Magento\Config\Model\Config\Source\Yesno $yesno,
        \Bnpparibas\Axepta\Model\MerchantAccount\Config\Source\Stores $stores,
        \Bnpparibas\Axepta\Model\MerchantAccount\Config\Source\Currencies $currencies,
        \Bnpparibas\Axepta\Model\MerchantAccount\Config\Source\Countries $countries,
        \Bnpparibas\Axepta\Model\MerchantAccount\Config\Source\DisplayMethods $displayMethod,
        \Bnpparibas\Axepta\Model\MerchantAccount\Config\Source\CaptureMethods $captureMode,
        \Bnpparibas\Axepta\Model\MerchantAccount\Config\Source\CetelemCategories $cetelemCategories,
        \Bnpparibas\Axepta\Helper\Data $helper,
        \Magento\Directory\Model\Currency $currencyModel,
        array $data = []
    ) {
        $this->systemStore = $systemStore;
        $this->stores = $stores;
        $this->yesno = $yesno;
        $this->currencies = $currencies;
        $this->displayMethod = $displayMethod;
        $this->captureMode = $captureMode;
        $this->cetelemCategories = $cetelemCategories;
        $this->countries = $countries;
        $this->helper = $helper;
        $this->scopeConfig = $context->getScopeConfig();
        $this->currenciesModel = $currencyModel;
        $this->coreRegistry = $registry;

        parent::__construct($context, $registry, $formFactory, $data);
        /* @var \Bnpparibas\Axepta\Model\MerchantAccount $model */
        $this->model = $this->coreRegistry->registry('axepta_merchant_account');
    }

    /**
     * Prepare form.
     *
     * @return $this
     *
     * @SuppressWarnings(PHPMD.ExcessiveMethodLength)
     */
    protected function _prepareForm()
    {
        /** @var \Magento\Framework\Data\Form $form */
        $form = $this->_formFactory->create();
        $values = [];
        $form->setHtmlIdPrefix('merchantaccount_');

        /************************************************
         * Fieldset General Configuration
         */
        $fieldset = $form->addFieldset(
            'base_fieldset',
            ['legend' => __('Account Activation')]
        );

        // Merchant ID
        $fieldset->addField(
            'method_name',
            'text',
            [
                'name' => 'general[method_name]',
                'label' => __('Front Office label'),
                'title' => __('Front Office label'),
                'required' => true,
                'tabindex' => 2,
                'note' => __('This field will display the payment method name for yours customers.'),
            ]
        );
        $values['method_name'] = !empty($this->model->getMethodName()) ? $this->model->getMethodName() : __('Pay by credit card');

        // Merchant ID
        $fieldset->addField(
            'merchant_id',
            'text',
            [
                'name' => 'general[merchant_id]',
                'label' => __('Merchant ID (MID)'),
                'title' => __('Merchant ID (MID)'),
                'required' => true,
                'tabindex' => 3,
                'note' => __('Be careful when updating this field, it will affect your full configuration'),
            ]
        );
        // Password
        $fieldset->addField(
            'merchant_id_password',
            'password',
            [
                'name' => 'general[merchant_id_password]',
                'label' => __('Blowfish encryption key'),
                'title' => __('Blowfish encryption key'),
                'required' => true,
                'tabindex' => 4,
                'note' => __('Be careful when updating this field, it will affect your full configuration'),
            ]
        );
        // HMAC KEY
        $fieldset->addField(
            'hmac_key',
            'text',
            [
                'name' => 'general[hmac_key]',
                'label' => __('Hmac Key'),
                'title' => __('Hmac Key'),
                'required' => true,
                'tabindex' => 3,
                'note' => __('Be careful when updating this field, it will affect your full configuration'),
            ]
        );

        // ID
        if ($this->model->getId()) {
            $fieldset->addField(
                'id',
                'hidden',
                ['name' => 'id']
            );
        }

        // Activation Key
        $fieldset->addField(
            'activation_key',
            'textarea',
            [
                'name' => 'general[activation_key]',
                'label' => __('Activation Key'),
                'title' => __('Activation Key'),
                'required' => true,
                'tabindex' => 3,
            ]
        );
        $activationKey = $this->model->getActivationKey();
        $allowedOptions = $this->helper->allowedOptions($activationKey);

        // COUNTRIES
        $this->model->setData('country_id', explode(';', $this->model->getCountryId() ?? ''));
        $fieldset->addField(
            'country_id',
            'multiselect',
            [
                'name' => 'general[country_id]',
                'label' => __('Country'),
                'title' => __('Country'),
                'required' => true,
                'values' => $this->countries->toOptionArrayCountries($this->model->getStoreId()),
                'value' => ['ALL'],
                'note' => __('You must enter and save the country to access the payment methods configuration'),
            ]
        );

        // Is Active
        $fieldset->addField(
            'is_active',
            'select',
            [
                'name' => 'general[is_active]',
                'label' => __('Status'),
                'title' => __('Status'),
                'required' => true,
                'options' => [
                    '1' => __('Enabled'),
                    '0' => __('Disabled'),
                ],
                'note' => __('Enable the payment in the front office.'),
            ]
        );

        /************************************************
        * Fieldset Store Configuration
        ***********************************************/
        $fieldset = $form->addFieldset(
            'store_fieldset',
            ['legend' => __('Configuration Store View')]
        );
        // STORE VIEW
        $fieldset->addField(
            'store_id',
            'select',
            [
                'name' => 'general[store_id]',
                'label' => __('Store'),
                'title' => __('Store'),
                'required' => true,
                'values' => $this->stores->toOptionArray(),
            ]
        );

        /************************************************
        * Fieldset Country Configuration
        ***********************************************/
        if (!empty($activationKey)) {
            $fieldset = $form->addFieldset(
                'country_fieldset',
                ['legend' => __('Account Options')]
            );

            // CURRENCY
            $fieldset->addField(
                'currency_id',
                'label',
                [
                    'name' => 'general[currency_id]',
                    'label' => __('Currency'),
                    'title' => __('Currency'),
                    'note' => __('Currency will be come after registration of your activation key'),
                ]
            );

            // Display Method
            $fieldset->addField(
                'display_method',
                'select',
                [
                    'name' => 'options[display_method]',
                    'label' => __('Display Method'),
                    'title' => __('Display Method'),
                    'required' => false,
                    'values' => $this->displayMethod->toOptionArray(),
                    'note' => __('Redirect => External redirect to the payment page').'.<br/>'.__('Iframe => Payment page rendering to the store'),
                ]
            );

            // 3DS
            $fieldset->addField(
                '3d_secure_active',
                'select',
                [
                    'name' => 'options[3d_secure_active]',
                    'label' => __('3DSecure exemption request'),
                    'title' => __('3DSecure exemption request'),
                    'required' => true,
                    'values' => $this->yesno->toOptionArray(),
                    'note' => __('Yes => The use of 3D Secure authentication protects the merchant against the cardholder dispute reason for non-payment. In case of request of exemption of authentication by the merchant, the latter loses this transfer of responsibility in the majority of the cases.')."<br/><a href='https://docs.axepta.bnpparibas/pages/viewpage.action?pageId=37198423' target='_blank'>".__('(for more details: Transfer of responsibility and 3D-Secure Matrices - Documentation Axepta BNP Paribas - Axepta)').'</a>',
                ]
            );

            $fieldset->addField(
                '3d_secure_active_amount',
                'text',
                [
                    'name' => 'options[3d_secure_active_amount]',
                    'label' => __('Amount'),
                    'title' => __('Amount'),
                    'required' => false,
                    'note' => __('Maximum amount for exemption requests'),
                ]
            );

            // ONE CLICK
            $fieldset->addField(
                'oneclick_payment_active',
                'select',
                [
                    'name' => 'options[oneclick_payment_active]',
                    'label' => __('Oneclick payment active'),
                    'title' => __('Oneclick payment active'),
                    'required' => true,
                    'values' => $this->yesno->toOptionArray(),
                    'note' => __('Enable the one-click payment in the front office.'),
                ]
            );

            // RECURRING
            $fieldset->addField(
                'recurring_payment_active',
                'select',
                [
                    'name' => 'options[recurring_payment_active]',
                    'label' => __('Recurring payment active'),
                    'title' => __('Recurring payment active'),
                    'required' => true,
                    'values' => $this->yesno->toOptionArray(),
                    'note' => __('Enable the recurring payment in the front office'),
                ]
            );

            // Capture Mode
            $fieldset->addField(
                'capture_mode',
                'select',
                [
                    'name' => 'options[capture_mode]',
                    'label' => __('Capture Mode'),
                    'title' => __('Capture Mode'),
                    'required' => false,
                    'values' => $this->captureMode->toOptionArray(),
                    'note' => __('Catch the amount at preparation. Attention: Capturing a transaction beyond the 7th day results in the loss of collateral associated with 3DSecure - if applicable, and beyond the 29th day the authorization granted by the issuer will no longer be valid. It is therefore preferable to capture transactions before these maturities'),
                ]
            );

            // Time before capture
            $fieldset->addField(
                'capture_time',
                'text',
                [
                    'name' => 'options[capture_time]',
                    'label' => __('Time before capture'),
                    'title' => __('Time before capture'),
                    'required' => false,
                    'tabindex' => 3,
                    'note' => __('Delay in hours before the capture (number between 1 and 696)'),
                ]
            );
        }

        /************************************************
        * Fieldset Cetelem Presto
        ***********************************************/
        if (!empty($activationKey)) {
            if (in_array('PRE', $this->helper->allowedOptions($activationKey))) {
                $fieldset = $form->addFieldset(
                    'cetelem_fieldset',
                    ['legend' => __('Cetelem Presto Options')]
                );

                // Cetelem Presto Category
                $fieldset->addField(
                    'cetelem_category',
                    'select',
                    [
                        'name' => 'options[cetelem_category]',
                        'label' => __('Cetelem Category'),
                        'title' => __('Cetelem Categories'),
                        'required' => true,
                        'values' => $this->cetelemCategories->toOptionArray(),
                        'note' => __('Select the most representative category of your shop.'),
                    ]
                );
            }
        }

        $this->_eventManager->dispatch('adminhtml_axepta_merchantaccount_edit_tab_main_prepare_form', ['form' => $form]);
        $form->setValues($this->model->getData());
        $form->addValues($values);
        $this->setForm($form);

        return parent::_prepareForm();
    }

    /**
     * Prepare label for tab.
     *
     * @return \Magento\Framework\Phrase
     */
    public function getTabLabel()
    {
        return __('Account Configuration');
    }

    /**
     * Prepare title for tab.
     *
     * @return \Magento\Framework\Phrase
     */
    public function getTabTitle()
    {
        return __('Account Configuration');
    }

    public function canShowTab()
    {
        return true;
    }

    public function isHidden()
    {
        return false;
    }

    /**
     * Check permission for passed action.
     *
     * @param string $resourceId
     *
     * @return bool
     */
    protected function _isAllowedAction($resourceId)
    {
        return $this->_authorization->isAllowed($resourceId);
    }
}
