<?php
/**
 * 1997-2018 Quadra Informatique
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0) that is available
 * through the world-wide-web at this URL: http://www.opensource.org/licenses/OSL-3.0
 * If you are unable to obtain it through the world-wide-web, please send an email
 * to modules@quadra-informatique.fr so we can send you a copy immediately.
 *
 * @author    Quadra Informatique <modules@quadra-informatique.fr>
 * @copyright 1997-2018 Quadra Informatique
 * @license   http://www.opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 *
 * @category    Bnp
 * @package     Bnpparibas_Axepta
 */
namespace Bnpparibas\Axepta\Block\Adminhtml\MerchantAccount\Edit\Tab;

/**
 * Cms page edit form main tab
 */
class Paygate extends \Magento\Backend\Block\Widget\Form\Generic implements \Magento\Backend\Block\Widget\Tab\TabInterface
{
    /**
     * @var \Magento\Store\Model\System\Store
     */
    protected $systemStore;

    /**
     * @var \Magento\Config\Model\Config\Source\Locale\Currency
     */
    protected $currency;

    /**
     * @var \Magento\Config\Model\Config\Source\Yesno
     */
    protected $yesno;

    protected $model;

    protected $helper;
    protected $methodAllowedCountryFactory;
    protected $merchantAccountPaygateConfigurationFactory;
    protected $methodFactory;

    protected $coreRegistry = null;


    /**
     * @param \Magento\Backend\Block\Template\Context $context
     * @param \Magento\Framework\Registry $registry
     * @param \Magento\Framework\Data\FormFactory $formFactory
     * @param \Magento\Store\Model\System\Store $systemStore
     * @param \Magento\Config\Model\Config\Source\Locale\Currency $currency
     * @param \Bnpparibas\Axepta\Model\Api\Methods $methods
     * @param array $data
     */
    public function __construct(
        \Magento\Backend\Block\Template\Context $context,
        \Magento\Framework\Registry $registry,
        \Magento\Framework\Data\FormFactory $formFactory,
        \Magento\Store\Model\System\Store $systemStore,
        \Magento\Config\Model\Config\Source\Yesno $yesno,
        \Bnpparibas\Axepta\Helper\Data $helper,
        \Bnpparibas\Axepta\Model\MethodAllowedCountryFactory $methodAllowedCountryFactory,
        \Bnpparibas\Axepta\Model\MerchantAccountPaygateConfigurationFactory $merchantAccountPaygateConfigurationFactory,
        \Bnpparibas\Axepta\Model\MethodFactory $methodFactory,
        array $data = []
    ) {
        $this->systemStore = $systemStore;
        $this->yesno = $yesno;
        $this->helper = $helper;
        $this->methodAllowedCountryFactory = $methodAllowedCountryFactory;
        $this->merchantAccountPaygateConfigurationFactory = $merchantAccountPaygateConfigurationFactory;
        $this->methodFactory = $methodFactory;
        $this->coreRegistry = $registry;

        parent::__construct($context, $registry, $formFactory, $data);
        /** @var \Bnpparibas\Axepta\Model\MerchantAccount $model */
        $this->model = $this->coreRegistry->registry('axepta_merchant_account');
    }

    /**
     * Prepare form
     *
     * @return $this
     * @SuppressWarnings(PHPMD.ExcessiveMethodLength)
     */
    protected function _prepareForm()
    {
        /** @var \Magento\Framework\Data\Form $form */
        $form = $this->_formFactory->create();

        $form->setHtmlIdPrefix('merchantaccount_');

        /** Fieldset Standard Payment * */
        $fieldset = $form->addFieldset(
            'base_fieldset',
            []
        );

        $methodList = $this->helper->getMehotdListFromConfigAccount($this->model->getData('id'));

        $paygateConfiguration = $this->merchantAccountPaygateConfigurationFactory->create();
        $merchantAccountPaygateConfigurations = $paygateConfiguration->getCollection()->addFieldToFilter('merchant_account_id', ['eq' => $this->model->getData('id')]);
        $activePaygateMethod = [];
        foreach ($merchantAccountPaygateConfigurations as $merchantAccountPaygateConfiguration) {
            if ($merchantAccountPaygateConfiguration->getIsActive() == "1") {
                $activePaygateMethod[] = $merchantAccountPaygateConfiguration->getMethodId();
            }
        }

        $activePaygateMethodValues = [];
        $allowedMethods = [];
        foreach ($methodList as $method) {
            $allowedMethods[] = $method['code'];

            $name = "paygate[{$method['id']}_active]";

            // If empty, then set default to true
            if (empty($activePaygateMethod)) {
                $activePaygateMethodValues[$name] = '1';
            } else {
                $activePaygateMethodValues[$name] = in_array($method['id'], $activePaygateMethod) ? '1' : '0';
            }

            $fieldset->addField(
                $name,
                'select',
                [
                    'name' => $name,
                    'label' => __($method['libelle']) ,
                    'title' => __($method['libelle']),
                    'required' => false,
                    'values' => $this->yesno->toOptionArray()
                ]
            );
        }
        $methods = $this->methodFactory->create()->getCollection();

        foreach ($methods as $method) {
            if (in_array($method->getCode(), $allowedMethods)) {
                continue;
            }

            $name = "paygate[{$method['id']}_active]";
            $description = __('Not available in your offer, please contact Axepta');
            $fieldset->addField($name, 'label', [
                'name' => $name,
                'label' => $method->getLibelle(),

            ])->setAfterElementHtml(<<<HTML
                <label class="admin__field-label">
                    <span>$description</span>
                </label>
HTML
            );
        }


        $this->_eventManager->dispatch('adminhtml_Axepta_merchantaccount_edit_tab_main_prepare_form', ['form' => $form]);
        $form->setValues($this->model->getData());
        $form->addValues($activePaygateMethodValues);
        $this->setForm($form);

        return parent::_prepareForm();
    }

    /**
     * Prepare label for tab
     *
     * @return \Magento\Framework\Phrase
     */
    public function getTabLabel()
    {
        return __('Paygate configuration');
    }

    /**
     * Prepare title for tab
     *
     * @return \Magento\Framework\Phrase
     */
    public function getTabTitle()
    {
        return __('Paygate configuration');
    }

    /**
     * {@inheritdoc}
     */
    public function canShowTab()
    {
        if ($this->model->getActivationKey() && $this->model->getCountryId()) {
            return true;
        }
        return false;
    }

    /**
     * {@inheritdoc}
     */
    public function isHidden()
    {
        return false;
    }

    /**
     * Check permission for passed action
     *
     * @param string $resourceId
     * @return bool
     */
    protected function _isAllowedAction($resourceId)
    {
        return $this->_authorization->isAllowed($resourceId);
    }
}
