<?php

namespace Bnpparibas\Axepta\Block\Adminhtml\System\Config\Form\Button;

use Magento\Framework\App\Config\ScopeConfigInterface;

class Download extends \Magento\Config\Block\System\Config\Form\Field
{

    const XML_PATH_LOG_KEY = 'bnpparibas_axepta/api/log_key';
    const BUTTON_TEMPLATE = 'system/config/button/download.phtml';

    protected $helper;

    protected $moduleDir;

    public function __construct(
        \Magento\Backend\Block\Template\Context $context,
        \Bnpparibas\Axepta\Helper\Data $helper,
        \Magento\Framework\Module\Dir $moduleDir,
        array $data = []
    ) {
        parent::__construct($context, $data);
        $this->helper = $helper;
        $this->moduleDir = $moduleDir;
    }

    /**
     * Set template to itself
     *
     * @return $this
     */
    protected function _prepareLayout()
    {
        parent::_prepareLayout();
        if (!$this->getTemplate()) {
            $this->setTemplate(static::BUTTON_TEMPLATE);
        }
        return $this;
    }

    /**
     * Render button
     *
     * @param  \Magento\Framework\Data\Form\Element\AbstractElement $element
     * @return string
     */
    public function render(\Magento\Framework\Data\Form\Element\AbstractElement $element)
    {
        // Remove scope label
        $element->unsScope()->unsCanUseWebsiteValue()->unsCanUseDefaultValue();
        return parent::render($element);
    }

     /**
      * Get the button and scripts contents
      *
      * @param \Magento\Framework\Data\Form\Element\AbstractElement $element
      * @return string
      */
    protected function _getElementHtml(\Magento\Framework\Data\Form\Element\AbstractElement $element)
    {
        $this->addData(
            [
                'id'        => 'log_download',
                'button_label'     => _('Download log'),
                'onclick'   => 'javascript:check(); return false;'
            ]
        );
        return $this->_toHtml();
    }

    public function downloadLogFiles()
    {
        return $this->getUrl('axepta/download/getfile');
    }

    public function ifKeyValid()
    {
        $key = $this->helper->getConfig(self::XML_PATH_LOG_KEY);

        $setupDir = $this->getModuleSetupDir();
        if (($debugKey = file_get_contents($setupDir . "/Tools/debug.key")) === false) {
            return false;
        }

        if (is_null($key) || $key == 'undefined') {
            return false;
        }

        if (!password_verify($key, $debugKey)) {
            return false;
        }

        return true;
    }

    public function getModuleSetupDir()
    {
        $directoryPath = $this->moduleDir->getDir(
            'Bnpparibas_Axepta',
            'Setup'
        );

        return $directoryPath;
    }
}
