<?php
/**
 * 1997-2019 Quadra Informatique
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0) that is available
 * through the world-wide-web at this URL: http://www.opensource.org/licenses/OSL-3.0
 * If you are unable to obtain it through the world-wide-web, please send an email
 * to modules@quadra-informatique.fr so we can send you a copy immediately.
 *
 * @author    Quadra Informatique <modules@quadra-informatique.fr>
 * @copyright 1997-2019 Quadra Informatique
 * @license   http://www.opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 *
 * @category    Bnp
 * @package     Bnpparibas_Axepta
 */

namespace Bnpparibas\Axepta\Block\Adminhtml\System\Config\Form;

class Hint extends \Magento\Backend\Block\AbstractBlock implements
    \Magento\Framework\Data\Form\Element\Renderer\RendererInterface
{

    protected $context;
    protected $storeManager;
    protected $packageInfos;

    /**
     * Constructor
     *
     * @param \Magento\Framework\App\Helper\Context              $context      Context
     * @param \Magento\Store\Model\StoreManagerInterface         $storeManager Store manager
     * @param \Magento\Framework\Module\PackageInfo              $packageInfos Package infos
     */
    public function __construct(
        \Magento\Framework\App\Helper\Context $context,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Framework\Module\PackageInfo $packageInfos
    ) {
        $this->context = $context;
        $this->storeManager = $storeManager;
        $this->packageInfos = $packageInfos;
    }

    /**
     * Remove SID from url
     *
     * @param  String $url Url with SID
     * @return String      Url without SID
     */
    public function sanitize($url)
    {
        $url = (string) $url;
        $pos = strpos($url, '?');
        return ($pos > 0) ? substr($url, 0, $pos) : $url;
    }

    /**
     * Render element html
     *
     * @param \Magento\Framework\Data\Form\Element\AbstractElement $element
     * @return string
     */
    public function render(\Magento\Framework\Data\Form\Element\AbstractElement $element)
    {
        $params = ['_secure' => true, '_store_to_url' => false];
        $version = $this->packageInfos->getVersion('Bnpparibas_Axepta');
        $html = '<p class="axepta-urls-box">' . __('Axepta BNP Paribas Connector Version: v%1', $version) . '</p>';
        return $html;
    }
}
