<?php

/**
 * 1997-2018 Quadra Informatique
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0) that is available
 * through the world-wide-web at this URL: http://www.opensource.org/licenses/OSL-3.0
 * If you are unable to obtain it through the world-wide-web, please send an email
 * to modules@quadra-informatique.fr so we can send you a copy immediately.
 *
 * @author    Quadra Informatique <modules@quadra-informatique.fr>
 * @copyright 1997-2018 Quadra Informatique
 * @license   http://www.opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 *
 * @category    Bnp
 * @package     Bnpparibas_Axepta
 */

namespace Bnpparibas\Axepta\Block\Adminhtml\Transaction;

class View extends \Magento\Backend\Block\Template
{

    /**
     * @var \Bnpparibas\Axepta\Model\TransactionFactory
     */
    protected $model;

    /**
     * @var \Magento\Framework\Registry
     */
    protected $coreRegistry = null;

    /**
     *
     * @var \Bnpparibas\Axepta\Helper\Data
     */
    protected $data;

    /**
     * @var \Bnpparibas\Axepta\Model\PaygateActionCodeFactory
     */
    protected $paygateActionCodeFactory;

    /**
     * @var \Bnpparibas\Axepta\Model\PaygateCategoryCodeFactory
     */
    protected $paygateCategoryCodeFactory;

    /**
     * @var \Bnpparibas\Axepta\Model\PaygateDetailCodeFactory
     */
    protected $paygateDetailCodeFactory;

    /**
     * @param \Magento\Backend\Block\Widget\Context $context
     * @param \Magento\Framework\Registry $registry
     * @param array $data
     */
    public function __construct(
        \Magento\Backend\Block\Widget\Context $context,
        \Magento\Framework\Registry $registry,
        \Bnpparibas\Axepta\Model\TransactionFactory $model,
        \Bnpparibas\Axepta\Model\PaygateActionCodeFactory $paygateActionCodeFactory,
        \Bnpparibas\Axepta\Model\PaygateCategoryCodeFactory $paygateCategoryCodeFactory,
        \Bnpparibas\Axepta\Model\PaygateDetailCodeFactory $paygateDetailCodeFactory,
        \Bnpparibas\Axepta\Helper\Data $dataHelper,
        array $data = []
    ) {
        $this->coreRegistry = $registry;
        $this->model = $model;

        $this->data = $dataHelper;
        $this->paygateActionCodeFactory = $paygateActionCodeFactory;
        $this->paygateCategoryCodeFactory = $paygateCategoryCodeFactory;
        $this->paygateDetailCodeFactory = $paygateDetailCodeFactory;
        parent::__construct($context, $data);
    }

    /**
     * Check permission for passed action
     *
     * @param string $resourceId
     * @return bool
     */
    protected function _isAllowedAction()
    {
        return true;
    }

    /**
     *
     * @return \Bnpparibas\Axepta\Model\TransactionFactory
     */
    public function getTransactionsDataById()
    {
        $id = $this->getRequest()->getParam('id');
        $transaction = $this->model->create()->load($id);

        $transactions = $this->model
            ->create()
            ->getCollection()
            ->addFieldToFilter('transaction_reference', ['eq' => $transaction->getTransactionReference()])
            ->addFieldToFilter('xid', ['eq' => $transaction->getXid()])
            ->addFieldToFilter('pay_id', ['eq' => $transaction->getPayId()]);

        $transactions->getSelect()->group('transaction_reference')->order('id DESC');
        return $transactions;
    }

    /**
     * return format error message
     * @param type $error_code
     */
    public function getErrorMessage($errorCode)
    {

        $actionCode = $this->getActionCode(substr($errorCode, 0, 1));
        $categoryCode = $this->getCategoryCode(substr($errorCode, 1, 3));
        $detailCode = $this->getDetailCode(substr($errorCode, -4));

        $message = [];

        if ($errorCode === '00000000') {
            $message[$actionCode->getMessage()] = [
                'description' => $actionCode->getDescription()
            ];
        } else {
            $message[$actionCode->getMessage()] = [
                'description' => $actionCode->getDescription()
            ];

            $message[$categoryCode->getMessage()] = [
                'description' => $categoryCode->getDescription()
            ];

            $message[$detailCode->getMessage()] = [
                'description' => $detailCode->getDescription()
            ];
        }

        return $message;
    }

    /**
     * get action code in db
     */
    public function getActionCode($code)
    {
        return $this->paygateActionCodeFactory->create()->load($code);
    }

    /**
     * get category code in db
     */
    public function getCategoryCode($code)
    {
        return $this->paygateCategoryCodeFactory->create()->load($code);
    }

    /**
     * get detail code in db
     */
    public function getDetailCode($code)
    {
        return $this->paygateDetailCodeFactory->create()->load($code);
    }
}
