<?php
/**
 * 1997-2018 Quadra Informatique.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0) that is available
 * through the world-wide-web at this URL: http://www.opensource.org/licenses/OSL-3.0
 * If you are unable to obtain it through the world-wide-web, please send an email
 * to modules@quadra-informatique.fr so we can send you a copy immediately.
 *
 * @author    Quadra Informatique <modules@quadra-informatique.fr>
 * @copyright 1997-2018 Quadra Informatique
 * @license   http://www.opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 *
 * @category    Bnp
 */

namespace Bnpparibas\Axepta\Block\Payment;

use Magento\Payment\Block\Info as MageInfo;

class Info extends MageInfo\Cc
{
    /** @var \Magento\Sales\Api\TransactionRepositoryInterface */
    protected $transactionRepository;

    /** @var \Magento\Framework\Stdlib\DateTime\TimezoneInterface */
    protected $timezoneInterface;

    /** @var \Magento\Framework\Pricing\PriceCurrencyInterface */
    protected $priceCurrencyInterface;

    /** @var \Bnpparibas\Axepta\Model\MerchantConfigurationAccountFactory */
    protected $_merchantConfigurationAccountFactory;

    /** @var \Bnpparibas\Axepta\Helper\Data */
    protected $_AxeptaHelperData;

    /** @var string */
    protected $_template = 'Bnpparibas_Axepta::info/default.phtml';

    protected $transactionFactory;

    public function __construct(
        \Magento\Framework\View\Element\Template\Context $context,
        \Magento\Payment\Model\Config $paymentConfig,
        \Magento\Sales\Api\TransactionRepositoryInterface $transactionRepository,
        \Magento\Framework\Pricing\PriceCurrencyInterface $priceCurrencyInterface,
        \Bnpparibas\Axepta\Model\TransactionFactory $transactionFactory,
        array $data = []
    ) {
        $this->transactionRepository = $transactionRepository;
        $this->timezoneInterface = $context->getLocaleDate();
        $this->priceCurrencyInterface = $priceCurrencyInterface;
        $this->transactionFactory = $transactionFactory;

        parent::__construct($context, $paymentConfig, $data);
    }

    /**
     * Prepare payment information.
     *
     * @param \Magento\Framework\DataObject|array|null $transport
     *
     * @return \Magento\Framework\DataObject
     */
    protected function _prepareSpecificInformation($transport = null)
    {
        $transport = MageInfo::_prepareSpecificInformation($transport);
        $infos[(string) __('Credit Card Type')] = $this->getPaymentName();

        return $transport->addData($infos);
    }

    public function getPaymentName()
    {
        $payment = $this->getInfo();
        $paymentAdditionnalInfos = $payment->getAdditionalInformation();
        if (isset($paymentAdditionnalInfos['axeptaMethodData'])) {
            if (isset($paymentAdditionnalInfos['axeptaMethodData']['code'])) {
                switch ($paymentAdditionnalInfos['axeptaMethodData']['code']) {
                    case 'VIM':
                    case 'CVM':
                    case 'AMX':
                        // Get Method
                        $transaction = $this->transactionFactory->create();
                        $transaction->load($payment->getParentId(), 'order_id');

                        return $transaction->getPaymentMeanBrand();
                        break;

                    default:
                        return $paymentAdditionnalInfos['axeptaMethodData']['libelle'];
                        break;
                }
            }
        }
    }
}
