<?php

/**
 * 1997-2019 Quadra Informatique
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0) that is available
 * through the world-wide-web at this URL: http://www.opensource.org/licenses/OSL-3.0
 * If you are unable to obtain it through the world-wide-web, please send an email
 * to modules@quadra-informatique.fr so we can send you a copy immediately.
 *
 * @author    Quadra Informatique <modules@quadra-informatique.fr>
 * @copyright 1997-2019 Quadra Informatique
 * @license   http://www.opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 *
 * @category    Bnp
 * @package     Bnpparibas_Axepta
 */

namespace Bnpparibas\Axepta\Block\Payment;

class Redirect extends \Magento\Framework\View\Element\Template
{

    /** @var \Magento\Framework\Data\FormFactory */
    protected $formFactory;

    /** @var \Magento\Sales\Model\Order */
    protected $order;
        
    protected $response;
        
    /** @var \Magento\Framework\App\Config\ScopeConfigInterface  */
    protected $scopeConfig;
    
    /** @var \Bnpparibas\Axepta\Model\MerchantAccount */
    protected $merchantAccount;
    
    /** @var \Bnpparibas\Axepta\Helper\Data */
    protected $helper;
    
    /** @var \Bnpparibas\Axepta\Model\Api\Service */
    protected $service;
    protected $isLogActive;

    /**
     * Logger
     *
     * @var LoggerInterface
     */
    protected $loggerAxepta;

    protected $resolver;

    /**
     * @param MagentoFrameworkViewElementTemplateContext $context
     * @param array                                      $data
     */
    public function __construct(
        \Magento\Framework\View\Element\Template\Context $context,
        \Magento\Framework\Data\FormFactory $formFactory,
        \Magento\Framework\App\ResponseInterface $response,
        \Bnpparibas\Axepta\Helper\Data $helper,
        \Bnpparibas\Axepta\Model\Api\Service $service,
        \Bnpparibas\Axepta\Logger\Logger $loggerAxepta,
        \Magento\Framework\Locale\Resolver $resolver,
        array $data = []
    ) {
        $this->formFactory = $formFactory;
        $this->response = $response;
        $this->scopeConfig = $context->getScopeConfig();
        $this->helper = $helper;
        $this->service = $service;
        $this->_resolver = $resolver;
        
        $this->loggerAxepta = $loggerAxepta;
        $this->isLogActive = $this->scopeConfig->getValue(
            \Bnpparibas\Axepta\Model\Api\Service::AXEPTA_LOG_ACTIVE,
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE
        );
        
        parent::__construct($context, $data);
    }

    public function setOrder(\Magento\Sales\Model\Order $order)
    {
        $this->order = $order;
    }
    
    public function setMarchantAccount(\Bnpparibas\Axepta\Model\MerchantAccount $merchantAccount)
    {
        $this->merchantAccount = $merchantAccount;
    }

    /**
     * Return html form containing current payment parameters
     * @return string
     */
    protected function _toHtml()
    {
        
        if ($this->order == null) {
            return __('Order not found!');
        }
      
        $paymentInfos = $this->order->getPayment();
        $additionalInformation = $paymentInfos->getAdditionalInformation();
        
        $action = $additionalInformation['axeptaMethodData']['apiUrl'];
        $isOneClick = $additionalInformation['axeptaMethodData']["isOneClick"];

        $customField1 = ($paymentInfos->getAdditionalInformation('CustomField1') != null) ? $paymentInfos->getAdditionalInformation('CustomField1') : null;
        $customField2 = ($paymentInfos->getAdditionalInformation('CustomField2') != null) ? $paymentInfos->getAdditionalInformation('CustomField2') : null;
        $customField3 = ($paymentInfos->getAdditionalInformation('CustomField3') != null) ? $paymentInfos->getAdditionalInformation('CustomField3') : null;
        $customField4 = ($paymentInfos->getAdditionalInformation('CustomField4') != null) ? $paymentInfos->getAdditionalInformation('CustomField4') : null;
        $customField6 = ($paymentInfos->getAdditionalInformation('CustomField6') != null) ? $paymentInfos->getAdditionalInformation('CustomField6') : null;
        $customField14 = ($paymentInfos->getAdditionalInformation('CustomField14') != null) ? $paymentInfos->getAdditionalInformation('CustomField14') : null;
        
        if ($customField1 !== null) {
            $action .= '?CustomField1='.(string) $customField1;
            $action .= '&CustomField2='.(string) $customField2;
            $action .= ($customField3 != null) ? '&CustomField3='.(string) $customField3 : "";
            $action .= '&CustomField4='.(string) $customField4;
            $action .= '&CustomField6='.(string) $customField6;

            $haystack  = $this->_resolver->getLocale();
            $langCode = strstr($haystack, '_', true);
            $action .= '&language='.$langCode;
        }

        if ($customField14 !== null) {
            $action .= '?CustomField14='.(string) $customField14;
        }
  
        if ($action == null) {
            return __('Unable to connect to Axepta BNP Paribas servers');
        } else {
            unset($additionalInformation['axeptaMethodData']);
            unset($additionalInformation['AccVerify']);
            unset($additionalInformation['method_title']);
            if ($isOneClick) {
                $cardOneClick = self::decodeParamJson($additionalInformation["card"]);
                unset($additionalInformation['card']);
            }
            if ($this->isLogActive) {
                $this->loggerAxepta->info(__METHOD__);
                $this->loggerAxepta->info(print_r($additionalInformation, true));
            }
            
            $plaintext =  $this->helper->getApiPlainText($additionalInformation);
            $length = mb_strlen($plaintext);
            $encryptedData = $this->helper->ctEncrypt($plaintext, $length, $this->merchantAccount->getMerchantIdPassword());
            
            
            $form = $this->formFactory->create();
            $form->setAction($action)
                ->setId('axepta_checkout')
                ->setName('axepta_checkout')->setMethod('POST')
                ->setUseContainer(true);
            
            $form->addField('MerchantID', 'hidden', [
                'name' => 'MerchantID',
                'value' => $this->merchantAccount->getMerchantId()
            ]);
            $form->addField('Data', 'hidden', [
                'name' => 'Data',
                'value' => $encryptedData
            ]);
            $form->addField('Len', 'hidden', [
                'name' => 'Len',
                'value' => $length
            ]);
            $form->addField('URLBack', 'hidden', [
                'name' => 'URLBack',
                'value' => $paymentInfos->getAdditionalInformation('URLBack')
            ]);

            if ($isOneClick) {
                $form->addField('number', 'hidden', [
                    'name' => 'number',
                    'value' => $cardOneClick["number"]
                ]);
                $form->addField('expiryDate', 'hidden', [
                    'name' => 'expiryDate',
                    'value' => $cardOneClick["expiryDate"]
                ]);
                $form->addField('brand', 'hidden', [
                    'name' => 'brand',
                    'value' => $cardOneClick["brand"]
                ]);
                $form->addField('cardholder', 'hidden', [
                    'name' => 'cardholder',
                    'value' => $cardOneClick["cardholderName"]
                ]);
            }
            
            $redirectText = __('You\'ll be redirected to the payment server in a few seconds...');
            $html = <<<HTML
                <html>
                    <body>
                        $redirectText
                        {$form->toHtml()}
                        <script type="text/javascript">
                            document.getElementById("axepta_checkout").submit();
                        </script>
                    </body>
                </html>
HTML;
                        
            

            $this->response->setHeader(
                'Pragma',
                'no-cache',
                true
            )->setHeader(
                'Cache-Control',
                'no-store, no-cache, must-revalidate, post-check=0, pre-check=0',
                true
            );

            return $html;
        }
    }

    /**
     * Decode param in json format
     * @param string $param
     * @return array
     */
    public static function decodeParamJson($param)
    {
        if ($param) {
            return json_decode(base64_decode($param), true);
        }
        return false;
    }
}
