<?php
/**
 * 1997-2019 Quadra Informatique
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0) that is available
 * through the world-wide-web at this URL: http://www.opensource.org/licenses/OSL-3.0
 * If you are unable to obtain it through the world-wide-web, please send an email
 * to modules@quadra-informatique.fr so we can send you a copy immediately.
 *
 * @author    Quadra Informatique <modules@quadra-informatique.fr>
 * @copyright 1997-2019 Quadra Informatique
 * @license   http://www.opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 *
 * @category    Bnp
 * @package     Bnpparibas_Axepta
 */

namespace Bnpparibas\Axepta\Controller\Adminhtml\MerchantAccount;

use Bnpparibas\Axepta\Model\ResourceModel\MerchantAccount as ResourceMerchantAccount;
use Bnpparibas\Axepta\Model\MerchantAccountFactory;
use Magento\Backend\App\Action;
use Magento\Framework\Controller\ResultFactory;

class Edit extends Action
{

    protected $resultPageFactory;
    protected $coreRegistry;
    protected $model;

    private $merchantAccountFactory;
    private $resourceMerchantAccount;

    public function __construct(
        \Magento\Backend\App\Action\Context $context,
        \Magento\Framework\View\Result\PageFactory $resultPageFactory,
        \Magento\Framework\Registry $registry,
        \Bnpparibas\Axepta\Model\MerchantAccount $model
    ) {
        $this->resultPageFactory = $resultPageFactory;
        $this->coreRegistry = $registry;
        $this->model = $model;
        parent::__construct($context);
    }

    public function execute()
    {
        $id = $this->getRequest()->getParam('id');
        $model = $this->model;

        // If you have got an id, it's edition
        if ($id) {
            $model->load($id);
            if (!$model->getId()) {
                $this->messageManager->addError(__('This merchant account does not exists.'));
                /** \Magento\Backend\Model\View\Result\Redirect $resultRedirect */
                $resultRedirect = $this->resultRedirectFactory->create();
                return $resultRedirect->setPath('*/*/');
            }
        }
        $this->coreRegistry->register('axepta_merchant_account', $model);
        $data = $this->_getSession()->getFormData(true);
        if (!empty($data)) {
            $model->setData($data);
        }
        /** @var \Magento\Backend\Model\View\Result\Page $resultPage */
        $resultPage = $this->_initAction();

        $resultPage->addBreadcrumb(
            $id ? __("'%1'/ Axepta BNP Paribas Payments configuration", $model->getMerchantId()) : __('New Axepta BNP Paribas Payments configuration'),
            $id ? __("'%1' / Axepta BNP Paribas Payments configuration", $model->getMerchantId()) : __('New Axepta BNP Paribas Payments configuration')
        );
        $resultPage->getConfig()->getTitle()->prepend(__('Merchant Accounts'));
        $resultPage->getConfig()->getTitle()
           ->prepend($model->getId() ? __($model->getMerchantId()) : __('New Merchant Account'));
        return $resultPage;
    }

    /**
     * Init actions
     *
     * @return \Magento\Backend\Model\View\Result\Page
     */
    protected function _initAction()
    {
        // load layout, set active menu and breadcrumbs
        /** @var \Magento\Backend\Model\View\Result\Page $resultPage */
        $resultPage = $this->resultPageFactory->create();
        $resultPage->setActiveMenu('Bnpparibas_Axepta::mechantaccount')
            ->addBreadcrumb(__('Bnp'), __('Bnp'))
            ->addBreadcrumb(__('Manage Merchant Accounts'), __('Manage Merchant Accounts'));
        return $resultPage;
    }
}
