<?php
/**
 * 1997-2018 Quadra Informatique
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0) that is available
 * through the world-wide-web at this URL: http://www.opensource.org/licenses/OSL-3.0
 * If you are unable to obtain it through the world-wide-web, please send an email
 * to modules@quadra-informatique.fr so we can send you a copy immediately.
 *
 * @author    Quadra Informatique <modules@quadra-informatique.fr>
 * @copyright 1997-2018 Quadra Informatique
 * @license   http://www.opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 *
 * @category    Bnp
 * @package     Bnpparibas_Axepta
 */

namespace Bnpparibas\Axepta\Controller\Adminhtml\MerchantAccount;

class Save extends \Magento\Backend\App\Action
{
    /**
     * @var \Magento\Payment\Model\Config\Source\Allspecificcountries
     */
    protected $_allSpecificCountries;

    /**
     * @var \Bnpparibas\Axepta\Model\MerchantConfigurationAccount
     */
    protected $account;
    protected $accountFactory;

    /**
     * @var \Bnpparibas\Axepta\Model\MerchantAccountPaygateConfigurationFactory
     */
    protected $accountPaygateConfigurationFactory;

    /**
     * @var \Bnpparibas\Axepta\Helper\Data
     */
    protected $helper;

    /**
     * @var \Bnpparibas\Axepta\Model\MerchantConfigurationAccountCountry
     */
    protected $_country;

    /**
     * @var \Bnpparibas\Axepta\Model\Api\Methods $methods
     */
    protected $_methods;

    protected $resourceConnection;

    /**
     * @param \Magento\Backend\App\Action $context
     * @param \Bnpparibas\Axepta\Model\MerchantConfigurationAccount $account
     */
    public function __construct(
        \Magento\Backend\App\Action\Context $context,
        \Bnpparibas\Axepta\Model\MerchantAccount $account,
        \Bnpparibas\Axepta\Model\MerchantAccountFactory $accountFactory,
        \Bnpparibas\Axepta\Model\MerchantAccountPaygateConfigurationFactory $accountPaygateConfigurationFactory,
        \Bnpparibas\Axepta\Helper\Data $helper,
        \Magento\Framework\App\ResourceConnection $resourceConnection
    ) {
        parent::__construct($context);
        $this->account = $account;
        $this->accountFactory = $accountFactory;
        $this->accountPaygateConfigurationFactory = $accountPaygateConfigurationFactory;
        $this->helper = $helper;
        $this->resourceConnection = $resourceConnection;
    }

    /**
     * {@inheritdoc}
     */
    protected function _isAllowed()
    {
        return $this->_authorization->isAllowed('Bnpparibas_Axepta::merchant_account_save');
    }

    /**
     * Save action
     *
     * @return \Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {
        
        $accountData = $this->getRequest()->getPostValue();
        $returnToEdit = false;

        if ($accountData) {

            /** @var \Bnpparibas\Axepta\Model\MerchantConfigurationAccount $account */
            $account = $this->account;

            $id = $this->getRequest()->getParam('id');
            if ($id) {
                $account->load($id);
            }
            $merchantAccount = $accountData['general'];
            $allowedOptions = $this->helper->allowedOptions($merchantAccount['activation_key']);
            $previousCountries = $account->getCountryId();
            try {
                /* GENERAL CONFIGURATION */
                $merchantAccount = $accountData['general'];
                $allowedOptions = $this->helper->allowedOptions($merchantAccount['activation_key']);
                $currency = end($allowedOptions);
                $countryIds = (isset($merchantAccount['country_id'])) ? implode(";", $merchantAccount['country_id']) : null;
                if (strpos($countryIds, 'ALL') !== false || $countryIds == 'ALL') {
                    $checkAccount = $this->accountFactory->create()->getCollection()
                                    ->addFieldToFilter('country_id', ['like' => '%ALL%'])
                                    ->addFieldToFilter('merchant_id', ['eq' => $merchantAccount['merchant_id']])
                                    ->addFieldToFilter('currency_id', ['eq' => $currency])
                                    ->addFieldToFilter('store_id', ['eq' => $merchantAccount['store_id']])
                                    ->getFirstItem();
                    if (!empty($checkAccount->getData()) && $checkAccount->getId() !== $id) {
                        throw new \Magento\Framework\Validator\Exception(__('An active configuration on all countries is already active on this store with this merchant ID'));
                    }
                }


                if (isset($merchantAccount['country_id'])) {
                    foreach ($merchantAccount['country_id'] as $countryId) {
                        $checkAccountMerchantId = $this->accountFactory->create()->getCollection()
                                ->addFieldToFilter('country_id', ['like' => "%$countryId%"])
                                ->addFieldToFilter('merchant_id', ['eq' => $merchantAccount['merchant_id']])
                                ->addFieldToFilter('currency_id', ['eq' => $currency])
                                ->addFieldToFilter('store_id', ['eq' => $merchantAccount['store_id']])
                                ->getFirstItem();

                        if (!empty($checkAccountMerchantId->getData()) && $checkAccountMerchantId->getId() !== $id) {
                            throw new \Magento\Framework\Validator\Exception(__('An active merchant ID is already active on this store on this country : %1', $countryId));
                        }
                    }
                }
                
                $options = (isset($accountData['options'])) ? $accountData['options'] : [];
                
                $account->addData([
                    'activation_key' => $merchantAccount['activation_key'],
                    'method_name' => $merchantAccount['method_name'],
                    'merchant_id' => (isset($merchantAccount['merchant_id'])) ? $merchantAccount['merchant_id'] : null,
                    'merchant_id_password' => (isset($merchantAccount['merchant_id_password'])) ? $merchantAccount['merchant_id_password'] : null,
                    'hmac_key' => (isset($merchantAccount['hmac_key'])) ? $merchantAccount['hmac_key'] : null,
                    'store_id' => (isset($merchantAccount['store_id'])) ? $merchantAccount['store_id'] : null,
                    'country_id' => $countryIds,
                    'currency_id' => (isset($currency)) ? $currency : null,
                    'capture_time' => (isset($merchantAccount['capture_time'])) ?  $merchantAccount['capture_time'] : null,
                    'is_active' => (isset($merchantAccount['is_active'])) ? $merchantAccount['is_active'] : false,
                    'display_method' => ($options !== []) ? $options['display_method'] : 'redirect',
                    'capture_mode' => ($options !== []) ? $options['capture_mode'] : 'auto',
                    'cetelem_category' => ($options !== [] && isset($options['cetelem_category'])) ? $options['cetelem_category'] : 320,
                    'capture_time' => ($options !== []) ? $options['capture_time'] :  null,
                    'oneclick_payment_active' => ($options !== [] && isset($options['oneclick_payment_active'])) ? $options['oneclick_payment_active'] : true,
                    'recurring_payment_active' => ($options !== [] && isset($options['recurring_payment_active'])) ? $options['recurring_payment_active'] : true,
                    '3d_secure_active' => ($options !== [] && isset($options['3d_secure_active']))  ? $options['3d_secure_active'] : false,
                    '3d_secure_active_amount' => ($options !== [] && isset($options['3d_secure_active_amount'])) ? $options['3d_secure_active_amount'] : null,
                ]);
                
                $account->save();
               
                $allowedOptions = $this->helper->getMehotdListFromConfigAccount($account->getId());
                
                $paygates = [];
                $arrayCountry = explode(';', $previousCountries ?? '');
                
                if ($merchantAccount['country_id'] != $arrayCountry) {
                    if (isset($accountData['paygate'])) {
                        $accountData['paygate'] = [];
                    }
                }
              
                if (!isset($accountData['paygate']) || empty($accountData['paygate'])) {
                    foreach ($allowedOptions as $key) {
                        if ($key->getId()) {
                            $accountData['paygate'][$key->getId().'_active'] = true;
                        }
                    }
                }
               
                $paygates = (isset($accountData['paygate']) && !empty($accountData['paygate'])) ?  $accountData['paygate'] : null;
               
                $connection = $this->resourceConnection->getConnection();
                $connection->delete(
                    $this->resourceConnection->getTableName('axepta_merchant_account_paygate_configuration'),
                    ['merchant_account_id = ?' => $id]
                );
                if (!empty($paygates)) {
                    foreach ($paygates as $key => $value) {
                        $accountPaygateConfiguration = $this->accountPaygateConfigurationFactory->create();
                        $methodId = explode('_', $key ?? '')[0];
                        if (!$id) {
                            $accountPaygateConfiguration->setMerchantAccountId((int)$account->getId());
                        } else {
                            $accountPaygateConfiguration->setMerchantAccountId((int)$id);
                        }
                        $accountPaygateConfiguration->setMethodId((int)$methodId);
                        $accountPaygateConfiguration->setIsActive($value);
                        $accountPaygateConfiguration->save();
                    }
                }


                $returnToEdit = (bool)$this->getRequest()->getParam('back', false);
                $this->messageManager->addSuccess(__('Merchant Account saved'));
            } catch (\Magento\Framework\Exception\LocalizedException $e) {
                $this->messageManager->addError($e->getMessage());
                $returnToEdit = true;
            } catch (\RuntimeException $e) {
                $this->messageManager->addError($e->getMessage());
                $returnToEdit = true;
            } catch (\Exception $e) {

                $this->messageManager->addException($e, __($e->getMessage()));
                $this->messageManager->addException($e, __('Something went wrong while saving the merchant account'));
                $returnToEdit = true;
            }
        }

        $resultRedirect = $this->resultRedirectFactory->create();
        if ($returnToEdit) {
            if ($account->getId()) {
                $resultRedirect->setPath(
                    'axepta/merchantaccount/edit',
                    ['id' => $account->getId(), '_current' => true]
                );
            } else {
                $resultRedirect->setPath(
                    'axepta/merchantaccount/new',
                    ['_current' => true]
                );
            }
        } else {
            $resultRedirect->setPath('axepta/paymentsetup/index');
        }

        return $resultRedirect;
    }

    protected function _getSaveAndContinueUrl()
    {
        return $this->getUrl('axepta/*/save', ['_current' => true, 'back' => 'edit', 'active_tab' => '']);
    }
}
