<?php

/**
 * 1997-2019 Quadra Informatique
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0) that is available
 * through the world-wide-web at this URL: http://www.opensource.org/licenses/OSL-3.0
 * If you are unable to obtain it through the world-wide-web, please send an email
 * to modules@quadra-informatique.fr so we can send you a copy immediately.
 *
 * @author    Quadra Informatique <modules@quadra-informatique.fr>
 * @copyright 1997-2019 Quadra Informatique
 * @license   http://www.opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 *
 * @category    Bnp
 * @package     Bnpparibas_Axepta
 */

namespace Bnpparibas\Axepta\Controller;

use Magento\Framework\Controller\ResultFactory;
use Magento\Sales\Model\Order;

abstract class Payment extends \Magento\Framework\App\Action\Action
{

    /** @var  \Magento\Framework\View\Result\Page */
    protected $resultPageFactory;

    /** @var \Magento\Sales\Model\OrderFactory */
    protected $orderFactory;

    protected $quoteRepository;

    /** @var \Psr\Log\LoggerInterface */
    protected $logger;

    /** @var \Magento\Checkout\Model\Session */
    protected $checkoutSession;

    /** @var \Magento\Framework\Message\ManagerInterface */
    protected $messageManager;

    /** @var \Magento\Framework\App\Config\ScopeConfigInterface */
    protected $scopeConfig;

    /** @var \Magento\Framework\HTTP\PhpEnvironment\RemoteAddress */
    protected $remoteAddress;

    /** @var \\Magento\Framework\App\Request\Http */
    protected $_request;

    /** @var \Bnpparibas\Axepta\Helper\Data */
    protected $helper;

    /** @var \Bnpparibas\Axepta\Model\MerchantAccountFactory */
    protected $merchantAccountFactory;

    /** @var \Bnpparibas\Axepta\Model\TransactionFactory */
    protected $transactionFactory;

    /** @var \Bnpparibas\Axepta\Model\CustomerOneclickPaymentCardFactory */
    protected $customerOneclickPaymentCardFactory;

    protected $orderCollectionFactory;

    /** @var LoggerInterface */
    protected $loggerAxepta;

    protected $transaction;

    protected $orderSender;

    protected $invoiceSender;

    protected $isLogActive;

    protected $storeManagerInterface;

    /**
     * @param \Magento\Framework\App\Action\Context $context
     * @param \Magento\Framework\View\Result\PageFactory $resultPageFactory
     * @param \Magento\Checkout\Model\Session $checkoutSession
     * @param \Magento\Sales\Model\OrderFactory $orderFactory
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Magento\Framework\HTTP\PhpEnvironment\RemoteAddress $remoteAddress
     * @param \Bnpparibas\Axepta\Helper\Data $helper
     * @param \Bnpparibas\Axepta\Model\TransactionFactory $transactionFactory
     */
    public function __construct(
        \Magento\Framework\App\Action\Context $context,
        \Magento\Framework\View\Result\PageFactory $resultPageFactory,
        \Magento\Checkout\Model\Session $checkoutSession,
        \Magento\Sales\Model\OrderFactory $orderFactory,
        \Magento\Sales\Model\ResourceModel\Order\CollectionFactory $orderCollectionFactory,
        \Magento\Quote\Api\CartRepositoryInterface $quoteRepository,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Framework\HTTP\PhpEnvironment\RemoteAddress $remoteAddress,
        \Bnpparibas\Axepta\Helper\Data $helper,
        \Bnpparibas\Axepta\Model\MerchantAccountFactory $merchantAccountFactory,
        \Bnpparibas\Axepta\Model\TransactionFactory $transactionFactory,
        \Bnpparibas\Axepta\Model\CustomerOneclickPaymentCardFactory $customerOneclickPaymentCardFactory,
        \Bnpparibas\Axepta\Logger\Logger $loggerAxepta,
        \Psr\Log\LoggerInterface $logger,
        \Magento\Framework\DB\Transaction $transaction,
        \Magento\Sales\Model\Order\Email\Sender\OrderSender $orderSender,
        \Magento\Sales\Model\Order\Email\Sender\InvoiceSender $invoiceSender,
        \Magento\Store\Model\StoreManagerInterface $storeManagerInterface
    ) {
        $this->resultPageFactory = $resultPageFactory;
        $this->messageManager = $context->getMessageManager();
        $this->orderFactory = $orderFactory;
        $this->orderCollectionFactory = $orderCollectionFactory;
        $this->quoteRepository = $quoteRepository;
        $this->checkoutSession = $checkoutSession;
        $this->logger = $logger;
        $this->helper = $helper;
        $this->merchantAccountFactory = $merchantAccountFactory;
        $this->transactionFactory = $transactionFactory;
        $this->customerOneclickPaymentCardFactory = $customerOneclickPaymentCardFactory;
        $this->scopeConfig = $scopeConfig;
        $this->remoteAddress = $remoteAddress;
        $this->isLogActive = $this->scopeConfig->getValue(
            \Bnpparibas\Axepta\Model\Api\Service::AXEPTA_LOG_ACTIVE,
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE
        );
        $this->loggerAxepta = $loggerAxepta;
        $this->transaction = $transaction;
        $this->orderSender = $orderSender;
        $this->invoiceSender = $invoiceSender;
        $this->storeManagerInterface = $storeManagerInterface;
        parent::__construct($context);
    }

    /**
     * Get Computop API url for payment with encrypted GET parameter
     *
     * @return strinng
     */
    protected function getPayment()
    {
        $order = $this->orderFactory->create()->loadByIncrementId($this->checkoutSession->getLastRealOrderId());
        $merchantAccount = $this->helper->getAllowedMerchantAccount($order->getBillingAddress()->getCountryId());
        $paymentInfos = $order->getPayment();
        $axeptaPaymentInfos = $paymentInfos->getAdditionalInformation();

        switch ($merchantAccount->getDisplayMethod()) {
            case "direct":
                break;
            case "redirect":
            case "iframe":
                $resultPage = $this->resultPageFactory->create();
                $redirectBlock = $resultPage->getLayout()->createBlock('Bnpparibas\Axepta\Block\Payment\Redirect');
                $redirectBlock->setOrder($order);
                $redirectBlock->setMarchantAccount($merchantAccount);
                $this->getResponse()->setBody($redirectBlock->toHtml());
                return $resultPage;
                break;
        }
    }

    /**
     * Get checkout session
     *
     * @return \Magento\Checkout\Model\Type\Onepage
     * @codeCoverageIgnore
     */
    public function getCheckout()
    {
        return $this->checkoutSession;
    }

    public function restoreQuoteOrder()
    {
        $session = $this->getCheckout();
        $order = null;

        if ($this->_request->getParam('id')) {
            $order = $this->orderFactory->create()->loadByAttribute('quote_id', $this->_request->getParam('id'));
            $collection = $this->orderCollectionFactory->create()
                ->addAttributeToSelect('*')
                ->addFieldToFilter('quote_id', $this->_request->getParam('id'))
                ->addFieldToFilter('state', 'new')
                ->addFieldToFilter('status', 'pending');
            foreach ($collection as $order) {
                $session->setLastRealOrderId($order->getIncrementId());
            }
        }

        if ($session->getLastRealOrderId()) {
            $order = $this->orderFactory->create()->loadByIncrementId($session->getLastRealOrderId());
            if ($order->getId()) {
                $session->restoreQuote($order);
            }
        }

        return $order;
    }

    public function getOrderByQuote()
    {
        $session = $this->getCheckout();
        $order = null;

        if ($this->_request->getParam('id')) {
            $order = $this->orderFactory->create()->loadByAttribute('quote_id', $this->_request->getParam('id'));
        }

        if ($session->getLastRealOrderId()) {
            $order = $this->orderFactory->create()->loadByIncrementId($session->getLastRealOrderId());
        }

        return $order;
    }

    /**
     * When a customer cancel payment from Axepta.
     */
    public function cancelIframeAction()
    {
        $this->getResponse()->setBody(
            '<script type="text/javascript">' .
                'window.top.location.href="' . $this->_url->getUrl('axepta/payment/cancel?id=' . (int) $this->_request->getParam('id')) . '";' .
                '</script>'
        );
        return;
    }

    /**
     * When a customer cancel payment from Axepta.
     */
    public function cancelAction()
    {
        $order = $this->getOrderByQuote();
        $paymentInformations = $order->getPayment()->getAdditionalInformation();
        $merchantAccount = $this->merchantAccountFactory->create()
            ->getCollection()
            ->addFieldToFilter('merchant_id', $paymentInformations['MerchantID'])->getFirstItem();

        $this->storeManagerInterface->setCurrentStore($merchantAccount->getStoreId());

        $this->restoreQuoteOrder();

        if ($order) {
            $order->addStatusToHistory(
                $order->getStatus(),
                // keep order status/state cancel
                __('The customer cancelled the payment')
            )->cancel()->save();
        }

        return $this->resultRedirectFactory->create()->setPath('checkout/cart');
    }

    /**
     * when Axepta returns via ipn
     * cannot have any output here
     * validate IPN data
     * if data is valid need to update the database that the user has
     */
    public function ipnAction()
    {
        $this->getResponse()->setBody("OK");
        if (!$this->getRequest()->isPost()) {
            return $this->resultRedirectFactory->create()->setPath('');
        }

        $returnData = [];
        $encryptedData = $this->_request->getParam('Data');

        try {
            $quote = $this->quoteRepository->get($this->_request->getParam('quote_id'));
        } catch (\Exception $e) {
            if ($this->isLogActive) {
                $this->loggerAxepta->info($e);
            }
            $this->logger->critical($e);
        }

        try {
            $order = $this->orderFactory->create()->loadByIncrementId($quote->getReservedOrderId());
        } catch (\Exception $e) {
            if ($this->isLogActive) {
                $this->loggerAxepta->info($e);
            }
            $this->logger->critical($e);
        }

        $order->getPayment()->setLastTransId($this->_request->getParam('quote_id'));

        $paymentInformations = $order->getPayment()->getAdditionalInformation();
        $axeptaMethodData = $paymentInformations['axeptaMethodData'];
        $merchantAccount = $this->merchantAccountFactory->create()
            ->getCollection()
            ->addFieldToFilter('merchant_id', $paymentInformations['MerchantID'])->getFirstItem();
        if (empty($merchantAccount->getData())) {
            if ($this->isLogActive) {
                $this->loggerAxepta->info('Error - Merchant Account not found');
            }
            $this->getResponse()->setBody("Error - Merchant Account not found");
            return;
        }

        $len = $this->_request->getParam('Len');

        try {
            $decryptedData = $this->helper->ctDecrypt($encryptedData, $len, $merchantAccount->getMerchantIdPassword());
            parse_str($decryptedData, $returnData);
        } catch (\Exception $e) {
            if ($this->isLogActive) {
                $this->loggerAxepta->info($e);
            }
            $this->logger->critical($e);
        }

        $paymentType = __('PAYMENT');

        if (isset($returnData['card'])) {
            $card = self::decodeParamJson($returnData['card']);
        } else {
            $card = [];
        }

        if ($axeptaMethodData['oneClickRegister'] === "1" && isset($card['number']) && null !== $order->getCustomerId()) {
            $oneclickCards = $this->customerOneclickPaymentCardFactory->create();
            $oneclickCard = $oneclickCards->getCollection()
                ->addFieldToFilter('customer_id', ['eq' => $order->getCustomerId()])
                ->addFieldToFilter('pcnr', ['eq' => $card['number']])
                ->getFirstItem();

            $paymentType = __('ONECLICK');
            $oneclickCardSave = $this->customerOneclickPaymentCardFactory->create();
            if ($oneclickCard->getId() === null) {
                $oneclickCardSave->setData([
                    'method_id' => $axeptaMethodData['methodId'],
                    'customer_id' => $order->getCustomerId(),
                    'pcnr' => $card['number'],
                    'ccexpiry' => $card['expiryDate'],
                    'ccbrand' => $card['brand'],
                    'holder_name' => $card['cardholderName']
                ]);
                $oneclickCardSave->save();
            }
        } elseif (isset($axeptaMethodData['isRecurringPayment']) && $axeptaMethodData['isRecurringPayment'] === "1") {
            $paymentType = __('RECURRING PAYMENT - FIRST PAYMENT');
        }

        $transaction = $this->transactionFactory->create();
        $transaction->setData([
            'order_id' => $order->getId(),
            'ref_nr' => (isset($returnData['refnr'])) ? $returnData['refnr'] : null,
            'merchant_id' => (isset($returnData['mid'])) ? $returnData['mid'] : null,
            'transaction_reference' => (isset($returnData['mid'])) ? $returnData['TransID'] : null,
            'transaction_date' => date('Y-m-d H:i:s'),
            'pay_id' => (isset($returnData['PayID'])) ? $returnData['PayID'] : null,
            'bid' => (isset($returnData['billingagreementid'])) ? $returnData['billingagreementid'] : null,
            'xid' => (isset($returnData['XID'])) ? $returnData['XID'] : null,
            'transaction_type' => $paymentType,
            'payment_mean_brand' => (isset($card['brand'])) ? $card['brand'] : null,
            'payment_mean_type' => (isset($returnData['Type'])) ? $returnData['Type'] : null,
            'response_code' => (isset($returnData['Code'])) ? $returnData['Code'] : null,
            'pcnr' => (isset($card['number'])) ? $card['number'] : null,
            'ccexpiry' => (isset($card['expiryDate'])) ? $card['expiryDate'] : null,
            'status' => (isset($returnData['Status'])) ? $returnData['Status'] : null,
            'description' => (isset($returnData['Description'])) ? $returnData['Description'] : null,
            'amount' => $order->getGrandTotal(),
            'raw_data' => $decryptedData,
            'scheme_reference_id' => (isset($returnData['schemeReferenceID'])) ? $returnData['schemeReferenceID'] : null,
            'holder_name' => (isset($card['cardholderName'])) ? $card['cardholderName'] : null
        ]);
        $transaction->save();

        $order->getPayment()->setTransactionId($order->getPayment()->getId());
        $order->getPayment()->setLastTransId($order->getPayment()->getId());
        $order->getPayment()->setTransactionAdditionalInfo(
            \Magento\Sales\Model\Order\Payment\Transaction::RAW_DETAILS,
            $returnData
        );


        if ($order->hasInvoices()) {
            $total = 0;
            foreach ($order->getInvoiceCollection() as $invoice) {
                $total += $invoice->getGrandTotal();
            }
            if ($total >= $order->getGrandTotal()) {
                if ($this->isLogActive) {
                    $this->loggerAxepta->info('Axepta IPN Flux');
                    $this->loggerAxepta->info('The total order and invoice are the same for order #' . $order->getId());
                }
                $this->getResponse()->setBody("Error - The total order and invoices are the same");
                return;
            }
        }

        if (isset($axeptaMethodData['isRecurringPayment']) && $axeptaMethodData['isRecurringPayment'] === '1') {
            $this->helper->setCustomerRecurringPayment($order);
        }

        $statusCode = (string) substr($returnData['Code'], 0, 1);
        switch ($statusCode) {
            case "0":
                $newOrderStatus = (strtoupper($returnData['Description'] === "PENDING")) ? Order::STATE_PENDING_PAYMENT : Order::STATE_PROCESSING;
                $order->setState(\Magento\Sales\Model\Order::STATE_PROCESSING)
                    ->setStatus($newOrderStatus);
                $order->addStatusHistoryComment(__('Capture mode: %1', $paymentInformations['captureMode']))
                    ->setIsCustomerNotified(false);

                if (!$order->getEmailSent() && $order->getCanSendNewEmailFlag() && $newOrderStatus === Order::STATE_PROCESSING && $paymentInformations['captureMode'] != "manual") {
                    $invoice = $order->prepareInvoice();
                    $invoice->register()->capture();
                    $this->transaction->addObject($invoice)->addObject($invoice->getOrder())->save();
                    $order->addStatusHistoryComment(__('Invoice #%1 created', $invoice->getIncrementId()))
                        ->setIsCustomerNotified(true);
                    try {
                        $this->orderSender->send($order);
                    } catch (\Exception $e) {
                        $this->logger->critical($e);
                    }
                    try {
                        $this->invoiceSender->send($invoice);
                    } catch (\Exception $e) {
                        $this->loggerAxepta->info($e);
                        $this->logger->critical($e);
                    }
                }

                $order->save();
                break;
            case "2":
            case "4":
                $cancelOrder = true;

                // Timeout code check list
                $timeoutCodeList = ['0051', '0056', '0368', '0931', '2206', '9040', '110A', '0402', '4005'];
                // Timeout
                if (in_array(substr($returnData['Code'], -4), $timeoutCodeList)) {
                    $checkTransaction = $this->transactionFactory->create()
                        ->getCollection()
                        ->addFieldToFilter('order_id', ['eq' => $order->getId()])
                        ->getFirstItem();
                    // With existing transaction
                    $successfullTransactionStatus = ['OK', 'AUTHORIZED'];
                    if (
                        strtoupper($checkTransaction->getTransactionType()) === 'PAIEMENT'
                        && $checkTransaction->getResponseCode() == '00000000'
                        && in_array(strtoupper($checkTransaction->getStatus()), $successfullTransactionStatus)
                        && strtoupper($checkTransaction->getDescription()) === 'SUCCESS'
                    ) {
                        $cancelOrder = false;
                        if ($this->isLogActive) {
                            $this->loggerAxepta->info('Timeout response, order has already a successfull transaction, nothing to do');
                        }
                    } elseif ($this->isLogActive) {
                        $this->loggerAxepta->info('Timeout response, order has no successfull transaction, order w\'ll be canceled');
                    }
                }

                if ($cancelOrder) {
                    $returnDetails = $this->helper->getStatusCodeDetails($returnData['Code']);
                    $this->checkoutSession->setErrorMessage($returnDetails['action'] . " - " . $returnDetails['category'] . " - " . $returnDetails['detail']);
                    $order->setState(Order::STATE_CANCELED)->setStatus(Order::STATE_CANCELED)->save();
                }

                break;
            case "6":
                $order->setState(Order::STATE_PENDING_PAYMENT)->setStatus(Order::STATE_PENDING_PAYMENT)->save();
                break;
        }

        $this->getResponse()->setBody("OK");
    }

    /**
     * Check axepta response
     *
     * @return type
     */
    public function apiReturnAction()
    {
        $result = $this->resultRedirectFactory->create();
        if ($this->checkoutSession->getLastRealOrderId()) {
            try {
                $order = $this->orderFactory->create()->loadByIncrementId($this->checkoutSession->getLastRealOrderId());
            } catch (\Exception $e) {
                if ($this->isLogActive) {
                    $this->loggerAxepta->info($e);
                }
                $this->logger->critical($e);
            }
            $paymentInformations = $order->getPayment()->getAdditionalInformation();
            $axeptaMethodData = $paymentInformations['axeptaMethodData'];
            $merchantAccount = $this->merchantAccountFactory->create()
                ->getCollection()
                ->addFieldToFilter('merchant_id', $paymentInformations['MerchantID'])->getFirstItem();
        } else {
            $merchantAccount = $this->merchantAccountFactory->create()
                ->getCollection()
                ->addFieldToFilter('id', $this->_request->getParam('id_merchant'))->getFirstItem();
        }

        if (empty($merchantAccount->getData())) {
            if ($this->isLogActive) {
                $this->loggerAxepta->info('Error while retrieving merchant account data');
            }
            $this->checkoutSession->setErrorMessage(__('Error while retrieving merchant account data'));
            $result->setPath('checkout/onepage/failure');
            return $result;
        }

        $returnData = [];
        $encryptedData = $this->_request->getParam('Data');
        $len = $this->_request->getParam('Len');
        try {
            $decryptedData = $this->helper->ctDecrypt($encryptedData, $len, $merchantAccount->getMerchantIdPassword());
            parse_str($decryptedData, $returnData);
        } catch (\Exception $e) {
            if ($this->isLogActive) {
                $this->loggerAxepta->info($e);
            }
            $this->logger->critical($e);
        }

        if (!$this->helper->apiResponseVerifyHmac($returnData, $merchantAccount)) {
            $this->checkoutSession->setErrorMessage(__('Error while verify data, returned hmac seem to be corrupted'));
            $order = $this->restoreQuoteOrder();
            if ($order) {
                $order->addStatusToHistory(
                    $order->getStatus(),
                    __('The payment seem to be corrupted')
                )->cancel()->save();
            }
            $result->setPath('checkout/onepage/failure');
            return $result;
        }

        $statusCode = (string) substr($returnData['Code'], 0, 1);

        switch ($statusCode) {
            case "0":
                $path = 'checkout/onepage/success';
                break;
            case "2":
            case "4":
                $returnDetails = $this->helper->getStatusCodeDetails($returnData['Code']);
                $this->restoreQuoteOrder();
                if ($this->isLogActive) {
                    $this->loggerAxepta->info($returnDetails['action'] . " - " . $returnDetails['category'] . " - " . $returnDetails['detail']);
                }
                $this->checkoutSession->setErrorMessage($returnDetails['action'] . " - " . $returnDetails['category'] . " - " . $returnDetails['detail']);
                $path = 'checkout/onepage/failure';
                break;
            case "6":
            default:
                $path = 'checkout/cart';
                break;
        }

        switch (substr($returnData['Code'], -4)) {
            case '0053':
            case '0073':
            case '0723':
            case '0946':
                $path = 'checkout/cart';
                $order = $this->restoreQuoteOrder();
                if ($order) {
                    $order->addStatusToHistory(
                        $order->getStatus(),
                        // keep order status/state cancel
                        __('The customer cancelled the payment')
                    )->cancel()->save();
                }
                break;
        }

        switch ($merchantAccount->getDisplayMethod()) {
            case "iframe":
                if (isset($axeptaMethodData['code'])) {
                    if (in_array($axeptaMethodData['code'], ['FC3', 'FC4', 'PRE'])) {
                        $result->setPath($path);
                        return $result;
                    }
                }
                $this->getResponse()->setBody(
                    '<script type="text/javascript">' .
                        'window.top.location.href="' . $this->_url->getUrl($path, ['_secure' => true]) . '";' .
                        '</script>'
                );
                return;
            default:
                $this->storeManagerInterface->setCurrentStore($merchantAccount->getStoreId());
                $result->setPath($path);
                return $result;
        }
    }

    /**
     * Decode param in json format
     * @param string $param
     * @return array
     */
    public static function decodeParamJson($param)
    {
        if ($param) {
            return json_decode(base64_decode($param), true);
        }
        return false;
    }
}
