<?php

/**
 * 1997-2019 Quadra Informatique
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0) that is available
 * through the world-wide-web at this URL: http://www.opensource.org/licenses/OSL-3.0
 * If you are unable to obtain it through the world-wide-web, please send an email
 * to modules@quadra-informatique.fr so we can send you a copy immediately.
 *
 * @author    Quadra Informatique <modules@quadra-informatique.fr>
 * @copyright 1997-2019 Quadra Informatique
 * @license   http://www.opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 *
 * @category    Bnp
 * @package     Bnpparibas_Axepta
 */

namespace Bnpparibas\Axepta\Cron;

use Magento\Sales\Model\Order;

class RecurringPayment
{

    protected $storeManager;
    protected $quoteManagement;
    protected $quoteFactory;
    protected $orderFactory;
    protected $product;
    protected $customerRepositoryInterface;
    protected $registry;
    protected $customerRecurringPaymentFactory;
    protected $transaction;
    protected $service;
    protected $helper;
    protected $transactionFactory;
    protected $transactionDb;

    /**
     * Logger
     *
     * @var LoggerInterface
     */
    protected $loggerAxepta;
    protected $merchantAccountFactory;

    /**
     * @var \Magento\Framework\App\ProductMetadataInterface
     */
    protected $productMetadata;

    /**
     * @var \Magento\Framework\Module\ResourceInterface
     */
    protected $moduleResource;

    public function __construct(
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Quote\Model\QuoteFactory $quoteFactory,
        \Magento\Quote\Model\QuoteManagement $quoteManagement,
        \Magento\Sales\Model\OrderFactory $orderFactory,
        \Magento\Catalog\Model\Product $product,
        \Magento\Customer\Api\CustomerRepositoryInterface $customerRepositoryInterface,
        \Magento\Framework\Registry $registry,
        \Bnpparibas\Axepta\Model\CustomerRecurringPaymentFactory $customerRecurringPaymentFactory,
        \Bnpparibas\Axepta\Model\Transaction $transation,
        \Bnpparibas\Axepta\Model\Api\Service $service,
        \Bnpparibas\Axepta\Helper\Data $helper,
        \Bnpparibas\Axepta\Model\TransactionFactory $transactionFactory,
        \Bnpparibas\Axepta\Logger\Logger $loggerAxepta,
        \Bnpparibas\Axepta\Model\MerchantAccountFactory $merchantAccountFactory,
        \Magento\Framework\App\ProductMetadataInterface $productMetadata,
        \Magento\Framework\Module\ResourceInterface $moduleResource,
        \Magento\Framework\DB\Transaction $transactionDb
    ) {
        $this->storeManager = $storeManager;
        $this->quoteFactory = $quoteFactory;
        $this->quoteManagement = $quoteManagement;
        $this->orderFactory = $orderFactory;
        $this->product = $product;
        $this->customerRepositoryInterface = $customerRepositoryInterface;
        $this->registry = $registry;
        $this->customerRecurringPaymentFactory = $customerRecurringPaymentFactory;
        $this->transaction = $transation;
        $this->service = $service;
        $this->helper = $helper;
        $this->transactionFactory = $transactionFactory;
        $this->loggerAxepta = $loggerAxepta;
        $this->merchantAccountFactory = $merchantAccountFactory;
        $this->productMetadata = $productMetadata;
        $this->moduleResource = $moduleResource;
        $this->transactionDb = $transactionDb;
    }

    public function execute()
    {

        $this->loggerAxepta->info("=========== CRON RECURRING AXEPTA START ===========");
        $nowDate = date('Y-m-d H:i:s');
        $this->loggerAxepta->info(__METHOD__);

        $scheduleRecurringPaymentList = $this->customerRecurringPaymentFactory->create()->getCollection()
            ->addFieldToFilter('status', ['eq' => '1'])
            ->addFieldToFilter('next_schedule', ['lt' => $nowDate]);

        $this->loggerAxepta->info("Nombre de commande à traiter : " . count($scheduleRecurringPaymentList));
        foreach ($scheduleRecurringPaymentList as $schedule) {
            $this->loggerAxepta->info("Schedule start : " . $schedule->getId());
            if (!empty($schedule->getData())) {
                $this->loggerAxepta->info("Schedule data ok ");
                $this->loggerAxepta->info("Calcul des données schedule");
                $lastSchedule = substr($schedule->getLastSchedule(), 0, 10);
                $today = date("Y-m-d");
                $diff = abs(strtotime($lastSchedule) - strtotime($today));
                $years = floor($diff / (365 * 60 * 60 * 24));
                $months = floor(($diff - $years * 365 * 60 * 60 * 24) / (30 * 60 * 60 * 24));
                $days = floor($diff / (60 * 60 * 24));
                $occurence = intval($schedule->getOccurenceInterval());
                $nbFail = ($schedule->getPeriodicity() == 'd') ?
                    floor(intval($days) / $occurence) :
                    floor(intval($months) / $occurence);

                $productPrice = $schedule->getCurrentSpecificPrice();
                if ($nbFail > 0) {
                    $productPrice *= $nbFail;
                }

                $time = strtotime(date($schedule->getLastSchedule()));
                $lastOccurence = $occurence * $nbFail;
                $nextOccurence = $occurence * ($nbFail + 1);

                $schedule->setCurrentOccurence(intval($schedule->getCurrentOccurence()) + intval($nbFail));
                $schedule->setLastSchedule(($schedule->getPeriodicity() == 'd') ?
                        date("Y-m-d h:i:s", strtotime("+$lastOccurence day", $time)) :
                        date("Y-m-d h:i:s", strtotime("+$lastOccurence month", $time)));

                $schedule->setNextSchedule(($schedule->getPeriodicity() == 'd') ?
                        date("Y-m-d h:i:s", strtotime("+$nextOccurence day", $time)) :
                        date("Y-m-d h:i:s", strtotime("+$nextOccurence month", $time)));
                $this->loggerAxepta->info("Calcul des données schedule - OK");


                $this->loggerAxepta->info("Récupération de la commande d'origine");
                $firstOrder = $this->orderFactory->create()->load($schedule->getOrderId());
                $storeId = $firstOrder->getStoreId();
                $store = $this->storeManager->getStore($storeId);
                $this->loggerAxepta->info("Récupération du customer d'origine");
                $customer = $this->customerRepositoryInterface->getById($schedule->getCustomerId());
                $customer->setWebsiteId($storeId);

                $this->loggerAxepta->info("Création de l'objet quote");
                $quote = $this->quoteFactory->create();
                $quote->setStore($store);
                $quote->assignCustomer($customer);

                // Création du produit avec le prix de l'abonnement
                $this->loggerAxepta->info("Ajout du produit à la quote");
                $product = $this->product->load($schedule->getProductId());
                $product->setPrice($productPrice);
                $quote->addProduct($product, 1);

                $this->loggerAxepta->info("Ajout des adresses à la quote");
                $quote->getBillingAddress()->addData($firstOrder->getBillingAddress()->getData());
                $quote->getShippingAddress()->addData($firstOrder->getShippingAddress()->getData());
                $shippingAddress = $quote->getShippingAddress();
                $shippingAddress->setCollectShippingRates(true)
                    ->collectShippingRates()
                    ->setShippingMethod($firstOrder->getShippingMethod());


                $this->loggerAxepta->info("Ajout de la méthode de paiement à la quote");
                $quote->setPaymentMethod('axepta_recurring');
                $firstOrderPaymentAxeptaMethodData = $firstOrder->getPayment()->getAdditionalInformation('axeptaMethodData');
                $merchantId = $firstOrder->getPayment()->getAdditionalInformation('MerchantID');
                $firstOrderPaymentAxeptaMethodData['operation'] = 'recurring';
                $firstOrderPaymentAxeptaMethodData['isRecurringPayment'] = '0';
                $firstOrderPaymentAxeptaMethodData['method'] = 'axepta_recurring';
                $firstOrderPaymentAxeptaMethodData['recurringSchedule'] = true;
                $firstOrderPaymentAxeptaMethodData['MerchantID'] = $merchantId;
                $firstOrderPaymentAxeptaMethodData['RecurringCurrencyCode'] = $firstOrder->getOrderCurrencyCode();

                $this->loggerAxepta->info("Sauvegarde de la quote");
                $quote->save();
                $this->loggerAxepta->info("Sauvegarde de la quote - OK");
                $this->registry->register('axepta_recurring_quote', $quote);
                $this->loggerAxepta->info("Load transaction");
                $this->transaction->load($schedule->getTransactionId());
                $this->loggerAxepta->info("Load transaction - OK");
                $card = [
                    'expiryDate' => $this->transaction->getCcexpiry(),
                    'cardholderName' => $this->transaction->getHolderName() ?? $quote->getCustomer()->getFirstName . ' ' . $quote->getCustomer()->getLastName,
                    'number' => $this->transaction->getPcnr(),
                    'brand' => $this->transaction->getPaymentMeanBrand(),
                ];
                $firstOrderPaymentAxeptaMethodData['card'] = $card;
                $quote->getPayment()->importData($firstOrderPaymentAxeptaMethodData);

                $this->loggerAxepta->info("Check transaction si paypal ou pas");
                if ($this->transaction->getBid() !== null) {
                    $quote->getPayment()->setAdditionalInformation('BillingAgreementID', $this->transaction->getBid());
                } else {
                    // Add card for 3DSV2
                    $quote->getPayment()->setAdditionalInformation('card', self::encodeParamJson($card));
                }

                // Add credentialOnFile for 3DSV2
                $credentialOnFile = [
                    'type' => ['unscheduled' => 'MIT'],
                    'initialPayment' => false,
                    'useCase' => 'ucof'
                ];
                $quote->getPayment()->setAdditionalInformation('credentialOnFile', self::encodeParamJson($credentialOnFile));

                $this->loggerAxepta->info("Récupération des informations de paiement supplémentaire");
                $additionalInformation = $quote->getPayment()->getAdditionalInformation();
                $merchantIdPassword = $additionalInformation['merchantIdPassword'];
                $quote->getPayment()->setAdditionalInformation('MerchantID', $merchantId);

                if (empty($schedule->getSchemeReferenceID())) {
                    switch ($this->transaction->getPaymentMeanBrand()) {
                        case 'Cartes Bancaires':
                            $quote->getPayment()->setAdditionalInformation('schemeReferenceID', '9999999999999');
                            break;
                        case 'VISA':
                            $quote->getPayment()->setAdditionalInformation('schemeReferenceID', '887001863998888');
                            break;
                        case 'MasterCard':
                            $quote->getPayment()->setAdditionalInformation('schemeReferenceID', '1231_MCC999999');
                            break;
                    }
                } else {
                    $quote->getPayment()->setAdditionalInformation('schemeReferenceID', $schedule->getSchemeReferenceId());
                }

                $etild = "MAGT2_" . $this->getMagentoVersion() . "_WE_" . $this->getModuleVersion();
                $quote->getPayment()->setAdditionalInformation('EtiId', $etild);

                $this->loggerAxepta->info("Récupération des informations de paiement supplémentaire");
                $additionalInformation = $quote->getPayment()->getAdditionalInformation();

                $merchantAccounts = $this->merchantAccountFactory->create();
                $merchantAccount = $merchantAccounts->getCollection()->addFieldToFilter('merchant_id', ['eq' => $merchantId])->getFirstItem();
                $merchantIdPassword = $merchantAccount->getMerchantIdPassword();
                $action = $additionalInformation['axeptaMethodData']['apiUrl'];
                unset($additionalInformation['axeptaMethodData']);
                unset($additionalInformation['merchantIdPassword']);

                $this->loggerAxepta->info("Encrypt des paramétres");
                $plaintext =  $this->helper->getApiPlainText($additionalInformation);

                $length = mb_strlen($plaintext);
                $encryptedData = $this->helper->ctEncrypt($plaintext, $length, $merchantIdPassword);

                $curlPostFields = [
                    'MerchantId' => $merchantId,
                    'Data' => $encryptedData,
                    'Len' => $length
                ];
                $curlResponseData = [];

                $this->loggerAxepta->info("Appel CURL API Axepta");
                $curlResponse = $this->service->axeptaApiCurlCall($action, $curlPostFields);
                $this->loggerAxepta->info("Appel CURL API Axepta - OK");

                $datas = [];
                $decryptedData = $this->helper->ctDecrypt($curlResponseData['Data'], $curlResponseData['Len'], $merchantIdPassword);

                $quote->collectTotals()->save();
                $order = $this->quoteManagement->submit($quote);
                $order->setEmailSent(0);
                if (isset($datas['card'])) {
                    $card = self::decodeParamJson($datas['card']);
                }
                if ($order->getEntityId()) {
                    $this->loggerAxepta->info("Commande recurring créée : " . $order->getEntityId());
                    $transaction = $this->transactionFactory->create();
                    $transaction->setData([
                        'order_id' => $order->getEntityId(),
                        'ref_nr' => (isset($datas['refnr'])) ? $datas['refnr'] : null,
                        'merchant_id' => (isset($datas['mid'])) ? $datas['mid'] : null,
                        'transaction_reference' => (isset($datas['mid'])) ? $datas['TransID'] : null,
                        'transaction_date' => date('Y-m-d H:i:s'),
                        'pay_id' => (isset($datas['PayID'])) ? $datas['PayID'] : null,
                        'bid' => (isset($datas['billingagreementid'])) ? $datas['billingagreementid'] : null,
                        'xid' => (isset($datas['XID'])) ? $datas['XID'] : null,
                        'transaction_type' => __('RECURRING PAYMENT'),
                        'payment_mean_brand' => (isset($card['brand'])) ? $card['brand'] : null,
                        'payment_mean_type' => (isset($datas['Type'])) ? $datas['Type'] : null,
                        'response_code' => (isset($datas['Code'])) ? $datas['Code'] : null,
                        'pcnr' => (isset($card['number'])) ? $card['number'] : null,
                        'ccexpiry' => (isset($card['expiryDate'])) ? $card['expiryDate'] : null,
                        'status' => (isset($datas['Status'])) ? $datas['Status'] : null,
                        'description' => (isset($datas['Description'])) ? $datas['Description'] : null,
                        'amount' => $order->getGrandTotal(),
                        'raw_data' => $decryptedData,
                        'scheme_reference_id' => (isset($datas['schemeReferenceID'])) ? $datas['schemeReferenceID'] : null,
                        'holder_name' => (isset($card['cardholderName'])) ? $card['cardholderName'] : null
                    ]);

                    $transaction->save();

                    $order->getPayment()->setTransactionId($order->getPayment()->getId());
                    $order->getPayment()->setLastTransId($order->getPayment()->getId());
                    $order->getPayment()->setTransactionAdditionalInfo(
                            \Magento\Sales\Model\Order\Payment\Transaction::RAW_DETAILS, $datas
                    );

                    $schedule->save();
                    $this->loggerAxepta->info("Transaction sauvegardée " . $transaction->getId());
                    $this->loggerAxepta->info("Schedule sauvegardée " . $schedule->getId());

                    if ($datas['Status'] != "OK") {
                        $schedule->setStatus(2);
                    } else {
                        $newOrderStatus = Order::STATE_PROCESSING;
                        $invoice = $order->prepareInvoice();
                        $invoice->register()->capture();
                        $this->transactionDb->addObject($invoice)->addObject($invoice->getOrder())->save();
                        $order->setState($newOrderStatus)
                            ->setStatus($newOrderStatus)
                            ->addStatusHistoryComment(__('Invoice #%1 created', $invoice->getIncrementId()))
                            ->setIsCustomerNotified(true);
                        $order->save();
                    }
                } else {
                    $this->loggerAxepta->info("Erreur lors de la création de la commande recurring");
                }
            } else {
                $this->loggerAxepta->info("Schedule data problem ");
            }
        }

        $this->loggerAxepta->info("=========== CRON RECURRING AXEPTA FIN ===========");
    }

    /**
     * Encode param in json format
     * @param array $param
     * @return string
     */
    public static function encodeParamJson($param)
    {
        if (is_array($param)) {
            return base64_encode(json_encode($param));
        }
        return false;
    }

    /**
     * Decode param in json format
     * @param string $param
     * @return array
     */
    public static function decodeParamJson($param)
    {
        if ($param) {
            return json_decode(base64_decode($param), true);
        }
        return false;
    }

    public function getMagentoVersion()
    {
        return $this->productMetadata->getVersion();
    }

    public function getModuleVersion()
    {
        return $this->moduleResource->getDbVersion('Bnpparibas_Axepta');
    }
}
