<?php

/**
 * 1997-2019 Quadra Informatique.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0) that is available
 * through the world-wide-web at this URL: http://www.opensource.org/licenses/OSL-3.0
 * If you are unable to obtain it through the world-wide-web, please send an email
 * to modules@quadra-informatique.fr so we can send you a copy immediately.
 *
 * @author    Quadra Informatique <modules@quadra-informatique.fr>
 * @copyright 1997-2019 Bnp
 * @license   http://www.opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 *
 * @category    Bnp
 */

namespace Bnpparibas\Axepta\Helper;

use Magento\Framework\App\Filesystem\DirectoryList;
use Magento\Framework\App\RequestInterface;

class Data extends \Magento\Framework\App\Helper\AbstractHelper
{
    /**
     * @var \Magento\Framework\Module\Dir\Reader
     */
    protected $moduleReader;

    /**
     * @var \Magento\Checkout\Model\Session
     */
    protected $checkoutSession;

    /**
     * @var \Magento\Customer\Model\Session
     */
    protected $customerSession;

    /**
     * @var \Bnpparibas\Axepta\Model\MerchantAccountFactory
     */
    protected $merchantAccountFactory;

    /**
     * @var \Bnpparibas\Axepta\Model\MethodFactory
     */
    protected $methodFactory;

    /**
     * @var \Bnpparibas\Axepta\Model\MethodOperationFactory
     */
    protected $methodOperationFactory;

    /**
     * @var \Bnpparibas\Axepta\Model\CustomerOneclickPaymentCardFactory
     */
    protected $customerOneclickPaymentCardFactory;

    /**
     * @var \Bnpparibas\Axepta\Model\ParameterFactory
     */
    protected $parameterFactory;

    /**
     * @var \Bnpparibas\Axepta\Model\MethodAllowedCountryFactory
     */
    protected $methodAllowedCountryFactory;

    /**
     * @var \Bnpparibas\Axepta\Model\MerchantAccountPaygateConfigurationFactory
     */
    protected $merchantAccountPaygateConfigurationFactory;

    /**
     * @var \Bnpparibas\Axepta\Model\RecurringPaymentFactory
     */
    protected $recurringPaymentFactory;

    /**
     * @var \Bnpparibas\Axepta\Model\CustomerRecurringPaymentFactory
     */
    protected $customerRecurringPaymentFactory;

    /**
     * @var \Bnpparibas\Axepta\Model\TransactionFactory
     */
    protected $transactionFactory;

    /**
     * @var \Magento\Framework\Registry
     */
    protected $registry;

    /**
     * @var \Bnpparibas\Axepta\Model\Api\Blowfish
     */
    protected $blowfish;

    /**
     * @var \Bnpparibas\Axepta\Model\PaygateActionCodeFactory
     */
    protected $paygateActionCodeFactory;

    /**
     * @var \Bnpparibas\Axepta\Model\PaygateCategoryCodeFactory
     */
    protected $paygateCategoryCodeFactory;

    /**
     * @var \Bnpparibas\Axepta\Model\PaygateDetailCodeFactory
     */
    protected $paygateDetailCodeFactory;

    /**
     * @var \Magento\Quote\Api\CartRepositoryInterface
     */
    protected $quoteRepository;

    /**
     * @var \Magento\Directory\Model\Country
     */
    protected $countryRepository;

    /**
     * @var \Magento\Framework\App\ProductMetadataInterface
     */
    protected $productMetadata;

    /**
     * @var \Magento\Framework\Module\ResourceInterface
     */
    protected $moduleResource;

    protected $store;

    protected $currencyFactory;

    protected $resolver;

    protected $scopeConfigInterface;

    protected $filesystem;

    protected $mediaDirectory;

    protected $fileUploaderFactory;

    protected $httpFactory;

    protected $storeManager;

    protected $ioFile;

    protected $imageFactory;

    protected $scopeConfig;

    protected $request;

    /**
     * @array Replaced characters
     */
    public const CUSTOMFIELD_FORBIDDEN_CHAR = ['&', '\'', '>', '<', '%'];

    public function __construct(
        \Magento\Framework\App\Helper\Context $context,
        \Magento\MediaStorage\Model\File\UploaderFactory $fileUploaderFactory,
        \Magento\Framework\Filesystem $filesystem,
        \Magento\Framework\HTTP\Adapter\FileTransferFactory $httpFactory,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Framework\Filesystem\Io\File $ioFile,
        \Magento\Framework\Image\Factory $imageFactory,
        \Magento\Framework\Module\Dir\Reader $moduleReader,
        \Magento\Checkout\Model\Session $checkoutSession,
        \Magento\Customer\Model\Session $customerSession,
        \Magento\Framework\Registry $registry,
        \Bnpparibas\Axepta\Model\Api\Blowfish $blowfish,
        \Magento\Quote\Api\CartRepositoryInterface $quoteRepository,
        \Magento\Directory\Model\Country $countryRepository,
        \Magento\Framework\App\ProductMetadataInterface $productMetadata,
        \Magento\Framework\Module\ResourceInterface $moduleResource,
        \Bnpparibas\Axepta\Model\MerchantAccountFactory $merchantAccountFactory,
        \Bnpparibas\Axepta\Model\MethodFactory $methodFactory,
        \Bnpparibas\Axepta\Model\MethodOperationFactory $methodOperationFactory,
        \Bnpparibas\Axepta\Model\CustomerOneclickPaymentCardFactory $customerOneclickPaymentCardFactory,
        \Bnpparibas\Axepta\Model\ParameterFactory $parameterFactory,
        \Bnpparibas\Axepta\Model\MethodAllowedCountryFactory $methodAllowedCountryFactory,
        \Bnpparibas\Axepta\Model\MerchantAccountPaygateConfigurationFactory $merchantAccountPaygateConfigurationFactory,
        \Bnpparibas\Axepta\Model\RecurringPaymentFactory $recurringPaymentFactory,
        \Bnpparibas\Axepta\Model\CustomerRecurringPaymentFactory $customerRecurringPaymentFactory,
        \Bnpparibas\Axepta\Model\TransactionFactory $transactionFactory,
        \Bnpparibas\Axepta\Model\PaygateActionCodeFactory $paygateActionCodeFactory,
        \Bnpparibas\Axepta\Model\PaygateCategoryCodeFactory $paygateCategoryCodeFactory,
        \Bnpparibas\Axepta\Model\PaygateDetailCodeFactory $paygateDetailCodeFactory,
        \Magento\Store\Api\Data\StoreInterface $store,
        \Magento\Directory\Model\CurrencyFactory $currencyFactory,
        \Magento\Framework\Locale\Resolver $resolver,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfigInterface,
        RequestInterface $request,
    ) {
        $this->filesystem = $filesystem;
        $this->mediaDirectory = $filesystem->getDirectoryWrite(DirectoryList::MEDIA);
        $this->fileUploaderFactory = $fileUploaderFactory;
        $this->httpFactory = $httpFactory;
        $this->storeManager = $storeManager;
        $this->ioFile = $ioFile;
        $this->imageFactory = $imageFactory;
        $this->scopeConfig = $context->getScopeConfig();
        $this->request = $request;

        $this->moduleReader = $moduleReader;
        $this->checkoutSession = $checkoutSession;
        $this->customerSession = $customerSession;
        $this->registry = $registry;
        $this->blowfish = $blowfish;

        $this->merchantAccountFactory = $merchantAccountFactory;
        $this->methodFactory = $methodFactory;
        $this->methodOperationFactory = $methodOperationFactory;
        $this->customerOneclickPaymentCardFactory = $customerOneclickPaymentCardFactory;
        $this->parameterFactory = $parameterFactory;
        $this->methodAllowedCountryFactory = $methodAllowedCountryFactory;
        $this->merchantAccountPaygateConfigurationFactory = $merchantAccountPaygateConfigurationFactory;
        $this->recurringPaymentFactory = $recurringPaymentFactory;
        $this->customerRecurringPaymentFactory = $customerRecurringPaymentFactory;
        $this->transactionFactory = $transactionFactory;
        $this->paygateActionCodeFactory = $paygateActionCodeFactory;
        $this->paygateCategoryCodeFactory = $paygateCategoryCodeFactory;
        $this->paygateDetailCodeFactory = $paygateDetailCodeFactory;

        $this->quoteRepository = $quoteRepository;
        $this->countryRepository = $countryRepository;
        $this->productMetadata = $productMetadata;
        $this->moduleResource = $moduleResource;

        $this->store = $store;
        $this->currencyFactory = $currencyFactory;
        $this->resolver = $resolver;
        $this->scopeConfigInterface = $scopeConfigInterface;

        parent::__construct($context);
    }

    /**
     * Decrypt Activation key.
     */
    public function decryptActivationKey($key)
    {
        if (!defined('\Magento\Framework\Module\Dir::MODULE_SETUP_DIR')) {
            // Magento 2.1
            $viewDir = $this->moduleReader->getModuleDir(
                \Magento\Framework\Module\Dir::MODULE_ETC_DIR,
                'Bnpparibas_Axepta'
            );
            $publicKeyRes = openssl_pkey_get_public(file_get_contents($viewDir.'/../Setup/Tools/rsa.pub'));
        } else {
            // Magento 2.2+
            $viewDir = $this->moduleReader->getModuleDir(
                \Magento\Framework\Module\Dir::MODULE_SETUP_DIR,
                'Bnpparibas_Axepta'
            );
            $publicKeyRes = openssl_pkey_get_public(file_get_contents($viewDir.'/Tools/rsa.pub'));
        }

        $datas = explode("\n", $key ?? '');
        $data = trim($datas[0]);

        $signature = trim(substr($key, strpos($key, "\n") + 1));
        $signatureDecode = base64_decode($signature);
        if (function_exists('openssl_verify')) {
            $result = openssl_verify($data, $signatureDecode, $publicKeyRes);
            if ($result == 1) {
                return true;
            } elseif ($result == 0) {
                return false;
            }
        }

        return false;
    }

    public function getApiPlainText($array)
    {
        $plaintext = implode('&', array_map(
            function ($k, $v) {
                return "$k=$v";
            },
            array_keys($array),
            array_values($array)
        ));

        return $plaintext;
    }

    /**
     * Get options list from the activation key.
     *
     * @return array
     */
    public function allowedOptions($key)
    {
        $options = explode(';', $key ?? '');
        array_pop($options);

        return $options;
    }

    /**
     * Get the quote Id (B2B - Entreprise)
     */
    public function getNegotiableQuoteIdFromUrl()
    {
        $pathInfo = $this->request->getPathInfo();
        $pathSegments = explode('/', trim($pathInfo, '/'));

        $index = array_search('negotiable-carts', $pathSegments);
        $negotiableQuoteId = $index !== false ? $pathSegments[$index + 1] : null;
        if ($negotiableQuoteId === null) {
            return null;
        }
        return $negotiableQuoteId;
    }

    /**
     * Get allowed merchant account from billing address.
     *
     * @return \Bnpparibas\Axepta\Model\MerchantAccount
     */
    public function getAllowedMerchantAccount($isoCountry = '', $fromSession = true)
    {
        if (!$fromSession) {
            $billingAddress = $this->registry->registry('axepta_recurring_quote')->getBillingAddress();
            $isoCountry = $billingAddress->getCountry();
        } elseif ($isoCountry === '') {
            $billingAddress = $this->checkoutSession->getQuote()->getBillingAddress();
            $isoCountry = $billingAddress->getCountry();
        }

        if ($isoCountry === null) {
            $negotiableQuoteId = $this->getNegotiableQuoteIdFromUrl();
            if ($negotiableQuoteId === null) {
                return false;
            }
            $quote = $this->quoteRepository->get($negotiableQuoteId);
            $billingAddress = $quote->getBillingAddress();
            $isoCountry = $billingAddress->getCountry();

            if ($isoCountry === null) {
                return false;
            }
        }

        $storeId = $this->storeManager->getStore()->getId();
        $currency = $this->storeManager->getStore()->getCurrentCurrency()->getCode();
        $merchantAccounts = $this->merchantAccountFactory->create();
        $merchantAccount = $merchantAccounts->getCollection()
            ->addFieldToFilter('country_id', ['like' => "%$isoCountry%"])
            ->addFieldToFilter('store_id', ['eq' => $storeId])
            ->addFieldToFilter('currency_id', ['eq' => $currency])
            ->addFieldToFilter('is_active', ['eq' => 1])
            ->getFirstItem();

        if (empty($merchantAccount->getData())) {
            $merchantAccount = $merchantAccounts->getCollection()
                ->addFieldToFilter('country_id', ['like' => '%ALL%'])
                ->addFieldToFilter('store_id', ['eq' => $storeId])
                ->addFieldToFilter('currency_id', ['eq' => $currency])
                ->addFieldToFilter('is_active', ['eq' => 1])
                ->getFirstItem();
        }

        return $merchantAccount;
    }

    /**
     * Get allowed methods list from the activation key and DB.
     *
     * @return array
     */
    public function getAllowedMethods(\Bnpparibas\Axepta\Model\MerchantAccount $merchantAccount)
    {
        $assignMethods = [];

        if ($this->decryptActivationKey($merchantAccount->getActivationKey())) {
            $merchantAccountPaygateConfigurations = $this->merchantAccountPaygateConfigurationFactory->create();
            $merchantAccountPaygateConfiguration = $merchantAccountPaygateConfigurations->getCollection()
                ->addFieldToFilter('merchant_account_id', ['eq' => $merchantAccount->getId()])
                ->addFieldToFilter('is_active', ['eq' => '1']);

            $methods = $this->methodFactory->create();
            foreach ($merchantAccountPaygateConfiguration as $config) {
                $method = $methods->load($config->getMethodId());
                $assignMethods[] = $method->getData();
            }
        }

        $reccurring = false;
        $products = $this->checkoutSession->getQuote()->getAllVisibleItems();
        foreach ($products as $product) {
            if ($this->isReccuringProduct($product)) {
                $reccurring = true;
            }
        }

        if ($reccurring) {
            foreach ($assignMethods as $key => $method) {
                $methodOperation = $this->methodOperationFactory->create()
                    ->getCollection()
                    ->addFieldToFilter('method_id', ['eq' => $method['id']])
                    ->addFieldToFilter('code', ['eq' => \Bnpparibas\Axepta\Model\RecurringPayment::IS_RECURRING_PRODUCT_TYPE_WORDING])
                    ->getFirstItem();

                if (empty($methodOperation->getData())) {
                    unset($assignMethods[$key]);
                }
            }
        }

        return $assignMethods;
    }

    public function getDataFromCollection($collection)
    {
        $return = [];
        foreach ($collection as $item) {
            $return[] = $item->getData();
        }

        return $return;
    }

    public function getMethodParamsByOperation($methodId, $operationCode)
    {
        $return = [];
        $parameters = $this->parameterFactory->create();
        $methodOperations = $this->methodOperationFactory->create();
        $methodOperationList = $methodOperations->getCollection()
            ->addFieldToFilter('method_id', ['eq' => $methodId])
            ->addFieldToFilter('code', ['eq' => $operationCode]);
        foreach ($methodOperationList as $methodOperation) {
            $methodParameters = $parameters->getCollection()->addFieldToFilter('parameter_set_id', ['eq' => $methodOperation->getParameterSetId()]);
            $return = [
                'url' => $methodOperation->getUrl(),
                'testUrl' => $methodOperation->getTestUrl(),
                'parameter' => $methodParameters,
            ];
        }

        return $return;
    }

    public function getUrlByTrigramAndOperation($trigram, $operationCode)
    {
        $methodFactory = $this->methodFactory->create();
        $method = $methodFactory->getCollection()->addFieldToFilter('code', ['eq' => $trigram])->getFirstItem();

        $methodOperationFactory = $this->methodOperationFactory->create();
        $methodOperation = $methodOperationFactory->getCollection()
            ->addFieldToFilter('method_id', ['eq' => $method->getId()])
            ->addFieldToFilter('code', ['eq' => $operationCode])
            ->getFirstItem();

        return $methodOperation->getUrl();
    }

    public function getMethodsParams($methodId)
    {
        $return = [];
        $parameters = $this->parameterFactory->create();
        $methodOperations = $this->methodOperationFactory->create();
        $methodOperationList = $methodOperations->getCollection()->addFieldToFilter('method_id', ['eq' => $methodId]);
        foreach ($methodOperationList as $methodOperation) {
            $methodParameters = $parameters->getCollection()->addFieldToFilter('parameter_set_id', ['eq' => $methodOperation->getParameterSetId()]);
            $return[$methodOperation->getCode()] = [
                'url' => $methodOperation->getUrl(),
                'testUrl' => $methodOperation->getTestUrl(),
                'parameter' => $methodParameters,
            ];
        }

        return $return;
    }

    public function getFormatMethodOperationParams(\Magento\Framework\DataObject $data, &$paymentInfos)
    {
        $additionalData = $data->getAdditionalData();
        if (empty($additionalData)) {
            return;
        }

        if (!isset($additionalData['id']) && empty($additionalData['id'])) {
            return;
        }

        $CCNr = $CCBrand = $CCExpiry = null;
        $operation = (isset($additionalData['operation'])) ? $additionalData['operation'] : 'payment';

        $RTF = null;
        $card = null;
        switch ($operation) {
            case 'oneclick':
                $methodId = $additionalData['method_id'];
                $oneclickCard = $this->getOneclickCard($additionalData['id']);
                $CCNr = $oneclickCard->getPcnr();
                $CCBrand = $oneclickCard->getCcbrand();
                $CCExpiry = $oneclickCard->getCcexpiry();
                $holderName = $oneclickCard->getHolderName();
                $credentialOnFileType = ['unscheduled' => 'CIT'];
                $credentialOnFileInitPayment = false;
                $credentialOnFileUseCase = 'cof';
                $card = $oneclickCard;
                break;
            case 'payment':
                $methodId = $additionalData['id'];
                break;
            case 'recurring':
                $credentialOnFileType = ['unscheduled' => 'CIT'];
                $credentialOnFileInitPayment = true;
                $methodId = $additionalData['id'];
                $card = $additionalData['card'];
                $credentialOnFileUseCase = 'ucof';
                break;
        }

        $paramsList = $this->getMethodParamsByOperation($methodId, $operation);
        $additionalData['apiUrl'] = $paramsList['url'];
        $additionalData['methodId'] = $methodId;
        $paymentInfos->setAdditionalInformation('axeptaMethodData', $additionalData);

        $merchantAccount = $this->getAllowedMerchantAccount();
        $store = $this->store;
        $storeManager = $this->storeManager;
        if (isset($additionalData['recurringSchedule']) && $additionalData['recurringSchedule']) {
            $quote = $this->registry->registry('axepta_recurring_quote');
            $merchantId = $additionalData['MerchantID'];
            $merchantAccounts = $this->merchantAccountFactory->create();
            $merchantAccount = $merchantAccounts->getCollection()->addFieldToFilter('merchant_id', ['eq' => $merchantId])->getFirstItem();
            $paymentInfos->setAdditionalInformation('merchantIdPassword', $merchantAccount->getMerchantIdPassword());
            $this->registry->unregister('axepta_recurring_quote');
            $currency = $additionalData['RecurringCurrencyCode'];
        } else {
            $quote = $this->checkoutSession->getQuote();

            // Check for the B2B Quote
            $negotiableQuoteId = $this->getNegotiableQuoteIdFromUrl();
            if (!empty($negotiableQuoteId)) {
                $quote = $this->quoteRepository->get($negotiableQuoteId);
            }

            $merchantAccount = $this->getAllowedMerchantAccount();
            $currency = $quote->getData('quote_currency_code');
        }

        if (null !== $quote->getShippingAddress()->getCountryId()) {
            $country = $quote->getShippingAddress()->getCountryId();
        } else {
            $country = $quote->getBillingAddress()->getCountryId();
        }

        $secureBaseUrl = $storeManager->getStore()->getBaseUrl(\Magento\Framework\UrlInterface::URL_TYPE_WEB, true);

        $paymentInfos->setAdditionalInformation('captureMode', $merchantAccount->getCaptureMode());
        if ($merchantAccount->getCaptureMode() == 'timed') {
            $capture = (int) $merchantAccount->getCaptureTime();
            date_default_timezone_set('Europe/Paris');
            $captureDate = date('Y-m-d H:i:s', strtotime("+{$capture} hours"));
            $paymentInfos->setAdditionalInformation('captureTimed', $captureDate);
        } elseif ($merchantAccount->getCaptureMode() == 'manual') {
            $capture = 'MANUAL';
        } else {
            $capture = 'AUTO';
        }

        $currencyQuote = $this->currencyFactory->create()->load($currency);
        $currencySymbol = $currencyQuote->getCurrencySymbol();

        $getLocale = $this->resolver;
        $haystack = $getLocale->getLocale();
        $langCode = strstr($haystack, '_', true);

        // Initial One Click Payment
        $oneClickRegister = (isset($additionalData['oneClickRegister'])) ? $additionalData['oneClickRegister'] : '0';

        // Billing address
        $billingAddress = '';
        if (null !== $quote->getBillingAddress()) {
            $billingAddress = [
                'city' => $quote->getBillingAddress()->getCity(),
                'country' => [
                    'countryA3' => $this->getIsoCountryCode($quote->getBillingAddress()->getCountryId()),
                ],
                'addressLine1' => [
                    'street' => substr($quote->getBillingAddress()->getStreet()[0], 0, 32),
                ],
                'postalCode' => $quote->getBillingAddress()->getPostCode(),
            ];
        }

        // Bill to customer
        $billToCustomer = '';
        if (null !== $quote->getBillingAddress()) {
            $billToCustomer = [
                'consumer' => [
                    'firstName' => $quote->getBillingAddress()->getFirstName(),
                    'lastName' => $quote->getBillingAddress()->getLastName(),
                ],
                'email' => $quote->getBillingAddress()->getEmail(),
            ];
        }

        foreach ($paramsList['parameter'] as $parameter) {
            $value = '';
            switch ($parameter->getName()) {
                case 'MerchantID':
                    $value = $merchantAccount->getMerchantId();
                    break;
                case 'RefNr':
                    if ($quote->getReservedOrderId()) {
                        $value = $quote->getReservedOrderId();
                    } else {
                        $quote->reserveOrderId();
                        $this->quoteRepository->save($quote);
                        $value = $quote->getReservedOrderId();
                    }
                    break;
                case 'TransID':
                    $value = $quote->getId();
                    break;
                case 'Amount':
                    $value = $quote->getGrandTotal() * 100;
                    if (isset($additionalData['code']) && $additionalData['code'] == 'PAL' && ($currency == 'HUF' || $currency == 'TWD')) {
                        $value = $quote->getGrandTotal() / 100;
                    }
                    break;
                case 'CustomField1':
                    if ($merchantAccount->getDisplayMethod() === 'redirect') {
                        $value = number_format($quote->getGrandTotal(), 2).$currencySymbol;
                    } else {
                        $value = null;
                    }
                    break;
                case 'CustomField2':
                    if ($merchantAccount->getDisplayMethod() === 'redirect') {
                        if ($quote->getReservedOrderId()) {
                            $value = $quote->getReservedOrderId();
                        } else {
                            $quote->reserveOrderId();
                            $this->quoteRepository->save($quote);
                            $value = $quote->getReservedOrderId();
                        }
                    } else {
                        $value = null;
                    }
                    break;
                case 'CustomField3':
                    if ($merchantAccount->getDisplayMethod() === 'redirect') {
                        $storeLogoPath = $this->scopeConfigInterface->getValue('design/header/logo_src', \Magento\Store\Model\ScopeInterface::SCOPE_STORE, $store->getId());
                        if (isset($storeLogoPath)) {
                            $folderName = \Magento\Config\Model\Config\Backend\Image\Logo::UPLOAD_DIR;
                            $value = $secureBaseUrl.'pub/media/'.$folderName.'/'.$storeLogoPath;
                        } else {
                            $value = null;
                        }
                    } else {
                        $value = null;
                    }
                    break;
                case 'CustomField4':
                    if ($merchantAccount->getDisplayMethod() === 'redirect') {
                        $cart_contents = __('Total number of items:').' '.$quote->getItemsQty().'||';
                        foreach ($quote->getAllVisibleItems() as $item) {
                            $cart_contents .= $item->getQty().' x '.str_replace(self::CUSTOMFIELD_FORBIDDEN_CHAR, '', $item->getName()).'|';
                        }

                        $value = $cart_contents;
                    } else {
                        $value = null;
                    }
                    break;
                case 'CustomField6':
                    if ($merchantAccount->getDisplayMethod() === 'redirect') {
                        if (null !== $quote->getShippingAddress()) {
                            $shipping_address = $quote->getShippingAddress()->getLastName().' '.$quote->getShippingAddress()->getFirstName().'|'.substr($quote->getShippingAddress()->getStreet()[0], 0, 32).'|'.$quote->getShippingAddress()->getPostCode().' '.$quote->getShippingAddress()->getCity();
                        } else {
                            $shipping_address = $quote->getBillingAddress()->getLastName().' '.$quote->getBillingAddress()->getFirstName().'|'.substr($quote->getBillingAddress()->getStreet()[0], 0, 32).'|'.$quote->getBillingAddress()->getPostCode().' '.$quote->getBillingAddress()->getCity();
                        }

                        $value = str_replace(self::CUSTOMFIELD_FORBIDDEN_CHAR, '', $shipping_address);
                    } else {
                        $value = null;
                    }
                    break;
                case 'CustomField14':
                    if ($merchantAccount->getDisplayMethod() === 'iframe') {
                        $value = 'iframe';
                    } else {
                        $value = null;
                    }
                    break;
                case 'Amount3D':
                    $value = $quote->getGrandTotal() * 100;
                    break;
                case 'Currency':
                    $value = $currency;
                    break;
                case 'MAC':
                    $payId = '';

                    $value = $this->ctHMAC(
                        $merchantAccount->getMerchantId(),
                        $quote->getGrandTotal() * 100,
                        $currency,
                        $merchantAccount->getHmacKey(),
                        $payId,
                        $quote->getId()
                    );
                    break;
                case 'URLSuccess':
                    $value = "{$secureBaseUrl}axepta/payment/success";
                    break;
                case 'URLFailure':
                    $value = "{$secureBaseUrl}axepta/payment/failure";
                    break;
                case 'Response':
                    $value = 'encrypt';
                    break;
                case 'URLNotify':
                    $value = "{$secureBaseUrl}axepta/payment/ipn";
                    break;
                case 'MsgVer':
                    $value = '2.0';
                    break;
                case 'credentialOnFile':
                    if (is_array($credentialOnFileType)) {
                        $value = [
                            'type' => $credentialOnFileType,
                            'initialPayment' => $credentialOnFileInitPayment,
                            'useCase' => $credentialOnFileUseCase,
                        ];
                        $value = self::encodeParamJson($value);
                    }
                    break;
                case 'threeDSPolicy':
                    $value = self::encodeParamJson($this->getThreeDSPolicy($currency, $merchantAccount->get3dSecureActive(), $quote->getGrandTotal(), 30, (float) $merchantAccount->get3dSecureActiveAmount(), $operation));
                    break;
                case 'card':
                    if (isset($card) && is_object($card)) {
                        $value = [
                            'expiryDate' => $CCExpiry,
                            'cardholderName' => $holderName ?? $quote->getCustomer()->getFirstName.' '.$quote->getCustomer()->getLastName,
                            'number' => $CCNr,
                            'brand' => $CCBrand,
                        ];
                        $value = self::encodeParamJson($value);
                    } elseif (isset($card) && $operation == 'recurring') {
                        $value = self::encodeParamJson($card);
                    }
                    break;
                case 'browserInfo':
                    $value = [
                        'acceptHeaders' => $_SERVER['HTTP_ACCEPT'],
                        'ipAddress' => $_SERVER['REMOTE_ADDR'],
                        'javaScriptEnabled' => true,
                        'language' => substr($_SERVER['HTTP_ACCEPT_LANGUAGE'], 0, 5),
                        'userAgent' => $_SERVER['HTTP_USER_AGENT'],
                        'colorDepth' => 24,
                        'screenHeight' => 723,
                        'screenWidth' => 1536,
                        'timeZoneOffset' => '300',
                    ];
                    $value = self::encodeParamJson($value);
                    break;
                case 'UserData':
                    $value = null;
                    break;
                case 'Capture':
                    $value = $capture;
                    break;
                case 'OrderDesc':
                    if ($merchantAccount->getMerchantId() == 'BNP_DEMO_AXEPTA') {
                        $value = 'Test:0000';
                    } else {
                        $value = $value = 'Quote N° '.$quote->getId();
                    }
                    break;
                case 'ReqID':
                    $value = strtoupper(uniqid($quote->getId()));
                    break;
                case 'Custom':
                    $value = 'quote_id='.$quote->getId().'|id_merchant='.$merchantAccount->getId();
                    break;
                case 'AccVerify':
                    $value = 'No';
                    break;
                    // I = Initial payment for the new subscription
                case 'RTF':
                    $value = $RTF;
                    break;
                case 'Language':
                    $value = $langCode;
                    break;
                case 'URLBack':
                    if ($merchantAccount->getDisplayMethod() === 'iframe') {
                        $value = "{$secureBaseUrl}axepta/payment/cancelIframe?id=".$quote->getId();
                    } else {
                        $value = "{$secureBaseUrl}axepta/payment/cancel";
                    }
                    break;
                case 'CCNr':
                    $value = $CCNr;
                    break;
                case 'CCExpiry':
                    $value = $CCExpiry;
                    break;
                case 'CCBrand':
                    $value = $CCBrand;
                    break;
                case 'CCCVC':
                    $value = null;
                    break;
                case 'AddrCountryCode':
                case 'bdCountryCode':
                    $value = $country;
                    break;
                case 'AccOwner':
                    $value = $quote->getBillingAddress()->getLastName();
                    break;
                case 'FirstName':
                case 'bdFirstName':
                    $value = $quote->getBillingAddress()->getFirstName();
                    break;
                case 'LastName':
                case 'bdLastName':
                    $value = $quote->getBillingAddress()->getLastName();
                    break;
                case 'AddrStreet':
                case 'bdStreet':
                    $value = substr($quote->getBillingAddress()->getStreet()[0], 0, 32);
                    break;
                case 'AddrStreet2':
                    $value = (sizeof($quote->getBillingAddress()->getStreet()) > 1) ? substr($quote->getBillingAddress()->getStreet()[1], 0, 32) : '';
                    break;
                case 'AddrCity':
                case 'bdCity':
                    $value = $quote->getBillingAddress()->getCity();
                    break;
                case 'GoodsCategory':
                    $value = $merchantAccount->getCetelemCategory();
                    break;
                case 'AddrState':
                    $value = $quote->getBillingAddress()->getRegion();
                    break;
                case 'AddrZip':
                case 'bdZip':
                case 'AddrZIP':
                    $value = $quote->getBillingAddress()->getPostCode();
                    break;
                case 'UseBillingData':
                    $value = 'yes';
                    break;
                case 'PayType':
                    if ($additionalData['code'] === 'FC3') {
                        $value = 1;
                    } else {
                        $value = 2;
                    }
                    break;
                case 'Salutation':
                    $value = 'MME';
                    break;
                case 'UI':
                    $value = 'hermes';
                    break;
                case 'BuyerEMail':
                case 'Email':
                    $value = $quote->getBillingAddress()->getEmail();
                    break;
                case 'Phone':
                    $value = self::formatPhoneNumber($quote->getBillingAddress()->getTelephone());
                    break;
                case 'CustomerID':
                    $value = $quote->getCustomer()->getId();
                    break;
                case 'BillingAgreementID':
                    $value = null;
                    break;

                case 'SocialSecurityNumber':
                    $value = $additionalData['socialSecurityNumber'];
                    break;
                case 'ChDesc':
                case 'Template':
                case 'PayID':
                case 'Textfeld1':
                case 'Textfeld2':
                case 'TermURL':
                case 'UserAgent':
                case 'schemeReferenceID':
                    $value = null;
                    break;
                case 'billingAddress':
                    $value = self::encodeParamJson($billingAddress);
                    break;
                case 'shippingAddress':
                    if (null !== $quote->getShippingAddress()->getCity()) {
                        $shippingAddress = [
                            'city' => $quote->getShippingAddress()->getCity(),
                            'country' => [
                                'countryA3' => $this->getIsoCountryCode($quote->getShippingAddress()->getCountryId()),
                            ],
                            'addressLine1' => [
                                'street' => substr($quote->getShippingAddress()->getStreet()[0], 0, 32),
                            ],
                            'postalCode' => $quote->getShippingAddress()->getPostCode(),
                        ];
                        $value = self::encodeParamJson($shippingAddress);
                    } else {
                        $value = self::encodeParamJson($billingAddress);
                    }
                    break;
                case 'billToCustomer':
                    $value = self::encodeParamJson($billToCustomer);
                    break;
                case 'shipToCustomer':
                    if (null !== $quote->getShippingAddress()->getCity()) {
                        $shipToCustomer = [
                            'consumer' => [
                                'firstName' => $quote->getShippingAddress()->getFirstName(),
                                'lastName' => $quote->getShippingAddress()->getLastName(),
                            ],
                            'email' => $quote->getBillingAddress()->getEmail(),
                        ];
                        $value = self::encodeParamJson($shipToCustomer);
                    } else {
                        $value = self::encodeParamJson($billToCustomer);
                    }
                    break;
                case 'TxType':
                    $value = 'Auth';
                    break;
                case 'Account':
                    $account = [
                        'payment' => [
                            'klarna' => [
                                'accountName' => '0',
                            ],
                        ],
                    ];
                    $value = self::encodeParamJson($account);
                    break;
                case 'ArticleList':
                    $products = $quote->getAllVisibleItems();
                    $products_line = [];
                    $total_products_amount = 0;

                    foreach ($products as $product) {
                        $prod = [
                            'name' => $product->getName(),
                            'quantity' => $product->getQty(),
                            'unit_price' => $product->getPriceInclTax() * 100,
                            'total_amount' => $product->getRowTotalInclTax() * 100,
                            'tax_rate' => $product->getTaxPercent(),
                            'total_tax_amount' => $product->getTaxAmount() * 100,
                        ];
                        $products_line[] = $prod;
                        $total_products_amount = $total_products_amount + $product->getRowTotalInclTax();
                    }

                    $products_line[] = [
                        'name' => __('Shipping, ancillary costs and discounts'),
                        'quantity' => 1,
                        'unit_price' => ($quote->getGrandTotal() - $total_products_amount) * 100,
                        'total_amount' => ($quote->getGrandTotal() - $total_products_amount) * 100,
                        'tax_rate' => 0,
                        'total_tax_amount' => 0,
                    ];

                    $products = [
                        'order_lines' => $products_line,
                    ];

                    $value = self::encodeParamJson($products);
                    break;
                case 'TaxAmount':
                    if (null !== $quote->getShippingAddress()) {
                        $value = $quote->getShippingAddress()->getTaxAmount();
                    } else {
                        $value = $quote->getBillingAddress()->getTaxAmount();
                    }
                    break;
            }

            if ($parameter->getRequired() === 'M' && $value === '') {
                throw new \Magento\Framework\Validator\Exception(__('Payment parameter ['.$parameter->getName().'] error'));
            }

            if ($parameter->getRequired() === 'O' && $value === null) {
                continue;
            }

            if ($value !== '') {
                $type = preg_replace('/[^a-zA-Z]/', '', $parameter->getFormat());
                $length = preg_replace('/[^0-9]/', '', $parameter->getFormat());

                if (is_array($value)) {
                    throw new \Magento\Framework\Validator\Exception(__('Param array pb'));
                }

                $value = $this->checkRegexpData($type, (string) $value, $length);
                $paymentInfos->setAdditionalInformation($parameter->getName(), $value);
            }
        }

        if ($oneClickRegister === '1') {
            $credentialOnFile = [
                'type' => ['unscheduled' => 'CIT'],
                'initialPayment' => true,
                'useCase' => 'cof',
            ];
            $credentialOnFile = self::encodeParamJson($credentialOnFile);

            $threeDSPolicy = [
                'challengePreference' => 'mandateChallenge',
            ];
            $threeDSPolicy = self::encodeParamJson($threeDSPolicy);

            $paymentInfos->setAdditionalInformation('credentialOnFile', $credentialOnFile);
            $paymentInfos->setAdditionalInformation('threeDSPolicy', $threeDSPolicy);
        }

        $etild = 'MAGT2_'.$this->getMagentoVersion().'_WE_'.$this->getModuleVersion();
        $paymentInfos->setAdditionalInformation('EtiId', $etild);

        $expirationTime = $this->scopeConfigInterface->getValue('payment/axepta/expiration_time', \Magento\Store\Model\ScopeInterface::SCOPE_STORE, $store->getId());
        if (null !== $expirationTime) {
            $date_utc = new \DateTime('now', new \DateTimeZone('UTC'));
            $date_utc->add(new \DateInterval('PT'.$expirationTime.'M'));
            $stamp = $date_utc->format('Y-m-d H:i:s');
            $paymentInfos->setAdditionalInformation('expirationTime', str_replace(' ', 'T', $stamp));
        }
    }

    /**
     * Retire les caractères non autorisé pour chaque type de champs.
     *
     * @param string $type
     * @param string $string
     *
     * @return type
     */
    public static function checkRegexpData($type, $string, $length = 0)
    {
        $returnStr = (is_string($string)) ? str_replace(["'", '’'], "'", $string) : '';
        switch ($type) {
            // Indique que les valeurs alphabétiques sont acceptées
            //  - Alphabétique [aA-zZ]
            case 'A':
                $returnStr = preg_replace('/[^a-zA-Z]/', '', $returnStr);
                break;
                // Indique que les valeurs numériques sont acceptées
                //  - Numerique [0-9]
            case 'N':
                $returnStr = preg_replace('/[^0-9]/', '', $returnStr);
                break;
                // Indique que les valeurs alphanumérique sont acceptées
                //  - Alphabétique [aA-zZ]
                //  - Numerique [0-9]
            case 'AN':
                $returnStr = preg_replace('/[^0-9a-zA-Z]/', '', $returnStr);
                break;
                // Les caractères suivants sont acceptés :
                //  - Alphabétique [aA-zZ]
                //  - Numerique [0-9]
                //  - Spécial _ . + - @
                //  - espace,
            case 'AS':
                $returnStr = preg_replace("/[^0-9a-zA-Z_+.\-@, ]/", '', $returnStr);
                break;
                // Tout caractère est accepté.
            case 'ANS':
            case 'JSON':
                break;
                // ------------------------------------------------------------------
        }

        $skipFormat = ['JSON', 'ENUM'];
        if ($length != 0 && !in_array($type, $skipFormat)) {
            if ($type.$length == 'AN12') {
                $returnStr = strrev(mb_substr(strrev($returnStr), 0, $length));
            } else {
                $returnStr = mb_substr($returnStr, 0, $length);
            }
        }

        return $returnStr;
    }

    public function getOneclickCard($cardId)
    {
        $cardFactory = $this->customerOneclickPaymentCardFactory->create();
        $card = $cardFactory->getCollection()->addFieldToFilter('id', ['eq' => $cardId])->getFirstItem();

        return $card;
    }

    public function getCustomerOneclickCardList()
    {
        $idCustomer = $this->customerSession->getId();
        $cardList = [];
        if ($idCustomer !== '') {
            $cardFactory = $this->customerOneclickPaymentCardFactory->create();
            $cardList = $cardFactory->getCollection()->addFieldToFilter('customer_id', ['eq' => $idCustomer]);
        }

        return $cardList;
    }

    public function isRecurringPaymentIsAllow($merchantAccount)
    {
        $keyOptions = $this->allowedOptions($merchantAccount->getActivationKey());

        return $merchantAccount->getRecurringPaymentActive() == '1';
    }

    public function ctHMACResponse($data, $merchantAccount)
    {
        $merchantId = $merchantAccount->getMerchantId() ?? '';
        $hmackey = $merchantAccount->getHmacKey() ?? '';
        $payId = $data['PayID'] ?? '';
        $transId = $data['TransID'] ?? '';
        $status = $data['Status'] ?? '';
        $code = $data['Code'] ?? '';

        return hash_hmac('sha256', "$payId*$transId*$merchantId*$status*$code", $hmackey);
    }

    public function apiResponseVerifyHmac($data, $merchantAccount)
    {
        $verifyHmac = $this->ctHMACResponse($data, $merchantAccount);

        return strtolower($data['MAC']) === strtolower($verifyHmac);
    }

    public function getMehotdListFromConfigAccount($merchantAccountId)
    {
        $merchantAccount = $this->merchantAccountFactory->create()->load($merchantAccountId);
        $methods = [];
        if (!empty($merchantAccount)) {
            $keyOptions = $this->allowedOptions($merchantAccount->getActivationKey());
            if (strpos($merchantAccount->getCountryId(), 'ALL') !== false) {
                $methodAllowedCountries = $this->methodAllowedCountryFactory->create()
                    ->getCollection();
            } else {
                $countryList = explode(';', $merchantAccount->getCountryId() ?? '');
                $countryList[] = 'ALL';
                $methodAllowedCountries = $this->methodAllowedCountryFactory->create()
                    ->getCollection()
                    ->addFieldToFilter('country', ['in' => $countryList])
                    ->addFieldToFilter('currency', ['in' => [$merchantAccount->getCurrencyId(), 'ALL']]);
            }
            $methodIds = [];
            foreach ($methodAllowedCountries as $methodAllowedCountry) {
                $methodIds[] = $methodAllowedCountry->getMethodId();
            }

            $methods = $this->methodFactory->create()
                ->getCollection()
                ->addFieldToFilter('id', ['in' => $methodIds])
                ->addFieldToFilter('code', ['in' => $keyOptions]);
        }

        return $methods;
    }

    public function isReccuringProduct($product)
    {
        $recurringPayment = $this->recurringPaymentFactory->create()
            ->getCollection()
            ->addFieldToFilter('product_id', ['eq' => $product->getProductId()])
            ->addFieldToFilter('type', ['eq' => \Bnpparibas\Axepta\Model\RecurringPayment::IS_RECURRING_PRODUCT_TYPE_VALUE])
            ->getFirstItem();

        $recurringDataProduct = $recurringPayment->getData();

        return !empty($recurringDataProduct);
    }

    public function getCustomerRecurringPayementList()
    {
        $idCustomer = $this->customerSession->getId();
        $customerRecurringPayments = $this->customerRecurringPaymentFactory->create()
            ->getCollection()
            ->addFieldToFilter('customer_id', ['eq' => $idCustomer]);

        return $customerRecurringPayments;
    }

    public function setCustomerRecurringPayment(\Magento\Sales\Model\Order $order)
    {
        $productList = $order->getAllVisibleItems();
        foreach ($productList as $product) {
            if (!$this->isReccuringProduct($product)) {
                continue;
            }

            $recurringPayment = $this->recurringPaymentFactory->create()
                ->getCollection()
                ->addFieldToFilter('product_id', ['eq' => $product->getProductId()])
                ->getFirstItem();

            $recurringDataProduct = $recurringPayment->getData();
            if (!empty($recurringDataProduct)) {
                $transaction = $this->transactionFactory->create()
                    ->getCollection()
                    ->addFieldToFilter('order_id', ['eq' => $order->getId()])
                    ->getFirstItem();

                $time = strtotime(date('Y/m/d h:i:s'));
                $occurence = $recurringDataProduct['occurence_interval'];
                $nextSchedule = ($recurringDataProduct['periodicity'] == 'd') ?
                    date('Y-m-d h:i:s', strtotime("+$occurence day", $time)) : date('Y-m-d h:i:s', strtotime("+$occurence month", $time));

                $customerRecurringPayment = $this->customerRecurringPaymentFactory->create();
                $customerRecurringPayment->setData([
                    'order_id' => $order->getId(),
                    'customer_id' => $order->getCustomerId(),
                    'product_id' => $product->getProductId(),
                    'transaction_id' => $transaction->getId(),
                    'status' => (substr($transaction->getResponseCode(), 0, 1) === '0') ? 1 : 2,
                    'periodicity' => $recurringDataProduct['periodicity'],
                    'occurence_interval' => $occurence,
                    'current_occurence' => 0,
                    'date_add' => date('Y-m-d h:i:s'),
                    'last_schedule' => date('Y-m-d h:i:s'),
                    'next_schedule' => $nextSchedule,
                    'current_specific_price' => $recurringDataProduct['recurring_amount'],
                    'scheme_reference_id' => ($transaction->getSchemeReferenceId() !== null) ? $transaction->getSchemeReferenceId() : null,
                ]);
                $customerRecurringPayment->save();
            }
        }
    }

    public function getStatusCodeDetails($statusCode)
    {
        $actionCode = $this->paygateActionCodeFactory->create()->load(substr($statusCode, 0, 1));
        $categoryCode = $this->paygateCategoryCodeFactory->create()->load(substr($statusCode, 1, 3));
        $detailCode = $this->paygateDetailCodeFactory->create()->load(substr($statusCode, -4));

        return [
            'action' => $actionCode->getDescription(),
            'category' => $categoryCode->getDescription(),
            'detail' => $detailCode->getDescription(),
        ];
    }

    public function ctHMAC($MerchantID, $Amount, $Currency, $HmacPassword, $PayId = '', $TransID = '')
    {
        return hash_hmac('sha256', "$PayId*$TransID*$MerchantID*$Amount*$Currency", $HmacPassword);
    }

    public function ctEncrypt($plaintext, $len, $password)
    {
        if (mb_strlen($password) <= 0) {
            $password = ' ';
        }

        if (mb_strlen($plaintext) != $len) {
            throw new \Magento\Framework\Validator\Exception(__('Length mismatch. The parameter len differs from actual length.'));

            return false;
        }

        $plaintext = $this->blowfish->expand($plaintext);
        $this->blowfish->bf_set_key($password);

        return bin2hex($this->blowfish->encrypt($plaintext));
    }

    public function ctDecrypt($cipher, $len, $password)
    {
        if (mb_strlen($password) <= 0) {
            $password = ' ';
        }
        // converts hex to bin
        $cipher = pack('H'.strlen($cipher), $cipher);
        if ($len > strlen($cipher)) {
            throw new \Magento\Framework\Validator\Exception(__('Length mismatch. The parameter len is too large.'));

            return false;
        }
        $this->blowfish->bf_set_key($password);

        return mb_substr($this->blowfish->decrypt($cipher), 0, $len);
    }

    public function ctSplit($value)
    {
        $array = [];
        for ($i = 0; $i < sizeof($value); ++$i) {
            $explode = explode('=', $value[$i] ?? '');
            $array[$explode[0]] = $explode[1];
        }

        return $array;
    }

    public static function formatPhoneNumber($phone)
    {
        if (preg_match("#(^\+[0-9]{2}|^\+[0-9]{2}\(0\)|^\(\+[0-9]{2}\)\(0\)|^00[0-9]{2}|^0)([0-9]{9}$|[0-9\-\s]{10}$)#", $phone)) {
            if (stripos($phone, '+33') !== false) {
                $phone = str_replace('+33', '0', $phone);
            }
        } else {
            $phone = '0606060606';
        }

        return $phone;
    }

    /**
     * Encode param in json format.
     *
     * @param array $param
     *
     * @return string
     */
    public static function encodeParamJson($param)
    {
        if (is_array($param)) {
            return base64_encode(json_encode($param));
        }

        return false;
    }

    public function getIsoCountryCode($countryId)
    {
        $country = $this->countryRepository->loadByCode($countryId);

        return $country->getData('iso3_code');
    }

    public function getMagentoVersion()
    {
        return $this->productMetadata->getVersion();
    }

    public function getModuleVersion()
    {
        return $this->moduleResource->getDbVersion('Bnpparibas_Axepta');
    }

    public function getThreeDSPolicy($currency, $is_exemption_active, $amount, $frictionless_value, $secure_exemption_amount, $operation)
    {
        if ($operation == 'recurring') {
            return ['challengePreference' => 'mandateChallenge'];
        }

        if ($operation == 'oneclick') {
            if (!$is_exemption_active || $amount > $secure_exemption_amount) {
                return ['challengePreference' => 'noPreference'];
            }

            if ($currency == 'EUR' && $amount <= $frictionless_value) {
                return ['threeDSExemption' => ['exemptionReason' => 'lowValue']];
            }

            return ['challengePreference' => 'noChallenge'];
        }

        if (!$is_exemption_active || $amount > $secure_exemption_amount) {
            return ['challengePreference' => 'noPreference'];
        }

        if ($currency == 'EUR' && $amount <= $frictionless_value) {
            return ['threeDSExemption' => ['exemptionReason' => 'lowValue']];
        }

        return ['challengePreference' => 'noChallenge'];
    }

    public function getConfig($config_path)
    {
        return $this->scopeConfig->getValue(
            $config_path,
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE
        );
    }
}
