<?php

/**
 * 1997-2019 Quadra Informatique
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0) that is available
 * through the world-wide-web at this URL: http://www.opensource.org/licenses/OSL-3.0
 * If you are unable to obtain it through the world-wide-web, please send an email
 * to modules@quadra-informatique.fr so we can send you a copy immediately.
 *
 * @author    Quadra Informatique <modules@quadra-informatique.fr>
 * @copyright 1997-2019 Quadra Informatique
 * @license   http://www.opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 *
 * @category    Bnp
 * @package     Bnpparibas_Axepta
 */

namespace Bnpparibas\Axepta\Model\Api;

class Xml extends \Magento\Framework\Model\AbstractModel
{

    const XML_PATH_ADMIN_AXEPTA_VERSION = 'bnpparibas_axepta/api/version';
    const XML_PATH_ADMIN_AXEPTA_URL_XML = 'bnpparibas_axepta/api/url_xml';
    const XML_PATH_ADMIN_AXEPTA_URL_XML_DICTIONNARY = 'bnpparibas_axepta/api/url_xml_dictionnary';

    protected $axeptaXmlCurrency;
    protected $axeptaXmlCountry;
    protected $axeptaXmlPaygateActionCode;
    protected $axeptaXmlPaygateCategoryCode;
    protected $axeptaXmlPaygateDetailCode;
    protected $axeptaXmlMethod;
    protected $axeptaXmlMethodLang;
    protected $axeptaXmlMethodOperation;
    protected $axeptaXmlMethodAllowedCountry;
    protected $axeptaXmlParameter;
    protected $axeptaXmlPosition;
    protected $filePath;
    protected $filePathDictionnary;
    protected $resource;
    protected $resourceConfig;
    protected $scopeConfig;
    protected $connection;
    protected $messageManager;
    protected $directory_list;

    /**
     * @return void
     */
    public function __construct(
        \Magento\Framework\Message\ManagerInterface $messageManager,
        \Magento\Framework\App\ResourceConnection $resource,
        \Magento\Config\Model\ResourceModel\Config $resourceConfig,
        \Magento\Framework\App\Filesystem\DirectoryList $directory_list,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
    ) {
        $this->messageManager = $messageManager;
        $this->scopeConfig = $scopeConfig;
        $this->resource = $resource;
        $this->resourceConfig = $resourceConfig;

        $this->connection = $resource->getConnection();

        $this->directory_list = $directory_list;
        $this->filePath = $this->directory_list->getPath('app').'/code/Bnpparibas/Axepta/Tools/computop_methods.xml';
        $this->filePathDictionnary = $this->directory_list->getPath('app').'/code/Bnpparibas/Axepta/Tools/computop_dictionnary.xml';

        $this->axeptaXmlCurrency = $resource->getTableName('axepta_xml_currency');
        $this->axeptaXmlCountry = $resource->getTableName('axepta_xml_country');
        $this->axeptaXmlPaygateActionCode = $resource->getTableName('axepta_xml_paygate_action_code');
        $this->axeptaXmlPaygateCategoryCode = $resource->getTableName('axepta_xml_paygate_category_code');
        $this->axeptaXmlPaygateDetailCode = $resource->getTableName('axepta_xml_paygate_detail_code');

        $this->axeptaXmlMethod = $resource->getTableName('axepta_xml_method');
        $this->axeptaXmlMethodLang = $resource->getTableName('axepta_xml_method_lang');
        $this->axeptaXmlMethodOperation = $resource->getTableName('axepta_xml_method_operation');
        $this->axeptaXmlMethodAllowedCountry = $resource->getTableName('axepta_xml_method_allowed_country');

        $this->axeptaXmlParameter = $resource->getTableName('axepta_xml_parameter');

        $this->axeptaXmlPosition = $resource->getTableName('axepta_xml_paygate_position');
    }


    /**
     * Load XML file from configured path
     * @return SimpleXMLElement loaded XML tree root
     */
    protected function loadXml()
    {
        return simplexml_load_file($this->filePath);
    }

    /**
     * Load XML file from configured path
     * @return SimpleXMLElement loaded XML tree root
     */
    protected function loadXmlDictionnary()
    {
        return simplexml_load_file($this->filePathDictionnary);
    }

    /**
     * Imports XML file in Axepta tables
     * @return boolean true if success
     */
    public function importXml()
    {
        $xmlMethods = $this->loadXml();
        $xmlDictionnary = $this->loadXmlDictionnary();

        if (!$xmlMethods) {
            $this->messageManager->addError(__('The XML providing the list of available payment methods is not reachable. Please check the URL in settings.'));
            return false;
        }

        $this->resetTables();
        // Update XML version in module config
        $this->resourceConfig->saveConfig(
            Xml::XML_PATH_ADMIN_AXEPTA_VERSION,
            $xmlMethods->version,
            \Magento\Framework\App\Config\ScopeConfigInterface::SCOPE_TYPE_DEFAULT,
            0
        );

        $this->connection->query('SET FOREIGN_KEY_CHECKS=0;');
        try {

            // Import dictionnary
            if (true) {
                // Import paygate_action_codes
                $paygateActionCodes = $xmlDictionnary->paygate_action_codes;
                foreach ($paygateActionCodes->action_codes->children() as $code) {

                    $query = 'INSERT INTO `' . $this->axeptaXmlPaygateActionCode . '`'
                            . '(`code`,`message`,`description`)'
                            . 'VALUES (:code, :message, :description)';
                    $data = [
                        'code' => (string) $code->code,
                        'message' => (string) $code->message,
                        'description' => (string) $code->description,
                    ];
                    $this->connection->query($query, $data);
                }

                foreach ($paygateActionCodes->categories->children() as $ctg) {

                    $query = 'INSERT INTO `' . $this->axeptaXmlPaygateCategoryCode . '`'
                            . '(`code`,`message`,`description`)'
                            . 'VALUES (:code, :message, :description)';
                    $data = [
                        'code' => (string) $ctg->code,
                        'message' => (string) $ctg->message,
                        'description' => (string) $ctg->description,
                    ];
                    $this->connection->query($query, $data);
                }

                foreach ($paygateActionCodes->details_codes->children() as $detail) {
                    $query = 'INSERT INTO `' . $this->axeptaXmlPaygateDetailCode . '`'
                            . '(`code`,`message`,`description`)'
                            . 'VALUES (:code, :message, :description)';
                    $data = [
                        'code' => (string) $detail->code,
                        'message' => (string) $detail->message,
                        'description' => (string) $detail->description,
                    ];
                    $this->connection->query($query, $data);
                }
            }

            // Import methods
            if (true) {
                foreach ($xmlMethods->methods->children() as $method) {

                    $query = 'INSERT INTO `' . $this->axeptaXmlMethod . '`'
                            . '(`code`, `libelle`)'
                            . 'VALUES (:code, :libelle)';

                    $data = [
                        'code' => (string) $method->code->attributes()['trigram'],
                        'libelle' => (string) $method->code
                    ];

                    $this->connection->query($query, $data);
                    $methodId = $this->connection->lastInsertId();

                    foreach ($method->labels->children() as $label) {
                        $query = 'INSERT INTO `' . $this->axeptaXmlMethodLang . '`'
                                . '(`method_id`, `iso`, `label`)'
                                . 'VALUES (:methodId, :iso, :label)';
                        $data = [
                            'methodId' => $methodId,
                            'iso' => (string) $label->attributes()['language'],
                            'label' => (string) $label
                        ];
                        $this->connection->query($query, $data);
                    }


                    foreach ($method->operations->children() as $operation) {
                        $query = 'INSERT INTO `' . $this->axeptaXmlMethodOperation . '`'
                                . '(`method_id`, `code`, `parameter_set_id`, `url`, `test_url`)'
                                . 'VALUES (:methodId, :code, :parameterSetId, :url, :testUrl)';
                        $data = [
                            'methodId' => $methodId,
                            'code' => (string) $operation->attributes()['code'],
                            'parameterSetId' => (string) $operation->attributes()['parameter_set'],
                            'url' => (string) $operation->action_urls->url,
                            'testUrl' => (string) $operation->action_urls->test_url,
                        ];
                        $this->connection->query($query, $data);
                    }

                    foreach ($method->allow_countries->children() as $country) {
                        $query = 'INSERT INTO `' . $this->axeptaXmlMethodAllowedCountry . '`'
                                . '(`method_id`, `currency`, `country`)'
                                . 'VALUES (:methodId, :currency, :country)';
                        $data = [
                            'methodId' => $methodId,
                            'currency' => (string) $country->attributes()['currency'],
                            'country' => (string) $country,
                        ];
                        $this->connection->query($query, $data);
                    }
                }

                foreach ($xmlMethods->parameter_sets->children() as $parameterSet) {

                    $parameterSetId = (string) $parameterSet->attributes()['id'];
                    foreach ($parameterSet->parameters->children() as $parameter) {
                        $query = 'INSERT INTO `' . $this->axeptaXmlParameter . '`'
                                . '(`parameter_set_id`, `name`, `format`, `required`)'
                                . 'VALUES (:parameterSetId, :name, :format, :required)';
                        $data = [
                            'parameterSetId' => $parameterSetId,
                            'name' => (string) $parameter->code,
                            'format' => (string) $parameter->format,
                            'required' => (string) $parameter->required
                        ];
                        $this->connection->query($query, $data);
                    }
                }

                foreach ($xmlMethods->positions->simple->countries->children() as $country) {
                    $iso = $country->attributes()['iso'];
                    foreach ($country->children() as $paygate) {
                        $query = 'INSERT INTO `' . $this->axeptaXmlPosition . '`'
                        . '(`position`, `iso`, `trigram`)'
                        . 'VALUES (:position, :iso, :trigram)';
                        $data = [
                            'position' => (string) $paygate->attributes()['pos'],
                            'iso' => (string) $iso,
                            'trigram' => (string) $paygate->attributes()['trigram']
                        ];
                        $this->connection->query($query, $data);
                    }
                }
            }

            return true;
        } finally {
            $this->connection->query('SET FOREIGN_KEY_CHECKS=1;');
        }

        return false;
    }

    public function resetTables()
    {
        $this->connection->query('SET FOREIGN_KEY_CHECKS=0;');

        $result =
                $this->connection->query('TRUNCATE `' . $this->axeptaXmlPaygateActionCode . '`;') &&
                $this->connection->query('TRUNCATE `' . $this->axeptaXmlPaygateCategoryCode . '`;') &&
                $this->connection->query('TRUNCATE `' . $this->axeptaXmlPaygateDetailCode . '`;') &&
                $this->connection->query('TRUNCATE `' . $this->axeptaXmlMethod . '`;') &&
                $this->connection->query('TRUNCATE `' . $this->axeptaXmlMethodLang . '`;') &&
                $this->connection->query('TRUNCATE `' . $this->axeptaXmlMethodOperation . '`;') &&
                $this->connection->query('TRUNCATE `' . $this->axeptaXmlMethodAllowedCountry . '`;') &&
                $this->connection->query('TRUNCATE `' . $this->axeptaXmlParameter . '`;') &&
                $this->connection->query('TRUNCATE `' . $this->axeptaXmlPosition . '`;');

        $this->connection->query('SET FOREIGN_KEY_CHECKS=1;');

        return $result;
    }

    /**
     * Check if XML is a valid update
     * @param string penging xml import version
     * @return string false if can upgrade, error message if not
     */
    public function checkVersion($importVersion)
    {
        // Get stored version
        $currentVersion = $this->scopeConfig->getValue(
            Xml::XML_PATH_ADMIN_AXEPTA_VERSION,
            \Magento\Framework\App\Config\ScopeConfigInterface::SCOPE_TYPE_DEFAULT
        );

        // First import
        if ($currentVersion == '') {
            return false;
        }

        list($x1, $y1, $z1) = explode('.', $currentVersion ?? '');
        list($x2, $y2, $z2) = explode('.', $importVersion ?? '');

        $error = false;

        if ($x1 === $x2) {
            if ($y1 === $y2 && $z1 === $z2) {
                $error = __("Xml already imported in version %1", $currentVersion);
            } elseif ($y1 > $y2 || $y1 === $y2 && $z1 > $z2) {
                $error = __('XML version %1 is older than the one you are currently using %2', $importVersion, $currentVersion);
            }
        } elseif ($x1 < $x2) {
            $error = __('The XML upgrade from version %1.Y.Z to version %2.Y.Z require an updated Axepta BNP Paribas module', $x1, $x2);
        } else {
            $error = __('XML version %1 is older than the one you are currently using %2', $importVersion, $currentVersion);
        }

        return $error;
    }
}
