<?php
/**
 * 1997-2019 Quadra Informatique.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0) that is available
 * through the world-wide-web at this URL: http://www.opensource.org/licenses/OSL-3.0
 * If you are unable to obtain it through the world-wide-web, please send an email
 * to modules@quadra-informatique.fr so we can send you a copy immediately.
 *
 * @author    Quadra Informatique <modules@quadra-informatique.fr>
 * @copyright 1997-2019 Quadra Informatique
 * @license   http://www.opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 *
 * @category    Bnp
 */

namespace Bnpparibas\Axepta\Model\MerchantAccount\Config\Source;

class Countries
{
    private $country;
    private $allowedCountryModel;
    private $storeManager;

    public function __construct(
        \Magento\Directory\Model\Country $country,
        \Magento\Directory\Model\AllowedCountries $allowedCountryModel,
        \Magento\Store\Model\StoreManagerInterface $storeManager
    ) {
        $this->country = $country;
        $this->allowedCountryModel = $allowedCountryModel;
        $this->storeManager = $storeManager;
    }

    /**
     * Get available options.
     *
     * @codeCoverageIgnore
     *
     * @return array
     */
    public function toOptionArrayCountries($storeId)
    {
        $selectCountries = [];

        $allowedCountries = $this->allowedCountryModel->getAllowedCountries('websites', [(int) $this->storeManager->getStore($storeId)->getWebsiteId()]);
        $countries = $this->country->getCollection()->addFieldToFilter('country_id', ['in' => array_keys($allowedCountries)]);

        array_push($selectCountries, ['value' => 'ALL', 'label' => __('ALL').' - '.__('Allow all countries')]);
        foreach ($countries as $country) {
            array_push($selectCountries, ['value' => $country->getId(), 'label' => $country->getData('iso3_code').' - '.$country->getName()]);
        }

        return $selectCountries;
    }
}
