<?php

/**
 * 1997-2016 Quadra Informatique
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0) that is available
 * through the world-wide-web at this URL: http://www.opensource.org/licenses/OSL-3.0
 * If you are unable to obtain it through the world-wide-web, please send an email
 * to modules@quadra-informatique.fr so we can send you a copy immediately.
 *
 * @author    Quadra Informatique <modules@quadra-informatique.fr>
 * @copyright 1997-2016 Quadra Informatique
 * @license   http://www.opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 *
 * @category    Bnp
 * @package     Bnpparibas_Axepta
 */

namespace Bnpparibas\Axepta\Model\Method;

class Axepta extends \Magento\Payment\Model\Method\AbstractMethod
{

    const METHOD_CODE = 'axepta';
    const OPERATION_TYPE_PAYMENT = 'payment';
    const OPERATION_TYPE_AUTH = 'authorization';
    const OPERATION_TYPE_CAPTURE = 'capture';
    const OPERATION_TYPE_REFUND = 'refund';
    const OPERATION_TYPE_ONECLICK = 'oneclick';
    const URL_INQUIRE_WITH_PAYID = 'https://paymentpage.axepta.bnpparibas/inquire.aspx';
    const URL_CAPTURE = 'https://paymentpage.axepta.bnpparibas/capture.aspx';

    protected $checkoutSession;
    protected $_currentOperationType = "";
    protected $_isGateway = true;
    protected $_canCapture = true;
    protected $_canCapturePartial = true;
    protected $_canRefund = true;
    protected $_canRefundInvoicePartial = true;

    /**
     * Payment code
     *
     * @var string
     */
    protected $_code = self::METHOD_CODE;

    /** @var \Psr\Log\LoggerInterface */
    protected $logger;

    /** @var \Bnpparibas\Axepta\Helper\Data */
    protected $helper;

    /** @var \Magento\Framework\UrlInterface */
    protected $urlBuilder;

    protected $_infoBlockType = 'Bnpparibas\Axepta\Block\Payment\Info';

    protected $transactionFactory;

    protected $dataBnp;

    protected $merchantAccountFactory;

    protected $curl;

    protected $methodFactory;

    protected $messageManager;

    protected $backendUrlManager;

    protected $notifierPool;

    public function __construct(
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \Magento\Framework\Api\ExtensionAttributesFactory $extensionFactory,
        \Magento\Framework\Api\AttributeValueFactory $customAttributeFactory,
        \Magento\Payment\Helper\Data $paymentData,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Payment\Model\Method\Logger $logger,
        \Magento\Checkout\Model\Session $checkoutSession,
        \Bnpparibas\Axepta\Helper\Data $axeptaHelper,
        \Magento\Framework\UrlInterface $urlBuilder,
        \Bnpparibas\Axepta\Model\TransactionFactory $transactionFactory,
        \Bnpparibas\Axepta\Helper\Data $dataBnp,
        \Bnpparibas\Axepta\Model\MerchantAccountFactory $merchantAccountFactory,
        \Magento\Framework\HTTP\Client\Curl $curl,
        \Bnpparibas\Axepta\Model\MethodFactory $methodFactory,
        \Magento\Framework\Message\ManagerInterface $messageManager,
        \Magento\Backend\Model\Url $backendUrlManager,
        \Magento\Framework\Notification\NotifierInterface $notifierPool,
        \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
        \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null,
        array $data = []
    ) {
        $this->checkoutSession = $checkoutSession;
        $this->helper = $axeptaHelper;
        $this->urlBuilder = $urlBuilder;
        $this->logger = $logger;
        $this->transactionFactory = $transactionFactory;
        $this->dataBnp = $dataBnp;
        $this->merchantAccountFactory = $merchantAccountFactory;
        $this->curl = $curl;
        $this->methodFactory = $methodFactory;
        $this->messageManager = $messageManager;
        $this->backendUrlManager  = $backendUrlManager;
        $this->notifierPool = $notifierPool;

        parent::__construct(
            $context,
            $registry,
            $extensionFactory,
            $customAttributeFactory,
            $paymentData,
            $scopeConfig,
            $logger,
            $resource,
            $resourceCollection,
            $data
        );
    }

    /**
     * @return string payment action
     */
    public function getConfigPaymentAction()
    {
        return $this->getInfoInstance()->getAdditionalInformation('action');
    }

    public function authorize(\Magento\Payment\Model\InfoInterface $payment, $amount)
    {
        parent::authorize($payment, $amount);
        if ($this->getConfigPaymentAction() == self::ACTION_AUTHORIZE) {
            $payment->setIsTransactionPending(1);
        }
        return $this;
    }

    /**
     * Check refund availability
     *
     * @return bool
     */
    public function canRefund()
    {
        return true;
    }

    /**
     * Check partial refund availability for invoice
     *
     * @return bool
     */
    public function canRefundPartialPerInvoice()
    {
        return true;
    }

    public function capture(\Magento\Payment\Model\InfoInterface $payment, $amount)
    {
        $paymentInformations = $payment->getAdditionalInformation();

        if (array_key_exists('captureMode', $paymentInformations)) {
            if ($paymentInformations['captureMode'] === 'manual') {
                $orderId = $payment->getOrder()->getId();

                $transactions = $this->transactionFactory->create();
                $transaction = $transactions->getCollection()->addFieldToFilter('order_id', ['eq' => $orderId])->getFirstItem();

                $merchantAccounts = $this->merchantAccountFactory->create();
                $merchantAccount = $merchantAccounts->getCollection()->addFieldToFilter('merchant_id', ['eq' => $transaction->getMerchantId()])->getFirstItem();

                $dataInquire = [
                    'MerchantID' => $transaction->getMerchantId(),
                    'PayID' => $transaction->getPayId(),
                    'TransID' => $transaction->getTransactionReference()
                ];

                $this->curl->post(self::URL_INQUIRE_WITH_PAYID, $dataInquire);
                $responseInquire = $this->curl->getBody();

                $content = explode('&', $responseInquire ?? '');
                
                $content = $this->dataBnp->ctSplit($content);

                $content = $this->dataBnp->ctDecrypt($content['Data'], $content['Len'], $merchantAccount->getMerchantIdPassword());
                
                $a = explode('&', $content ?? '');
                
                $data = $this->dataBnp->ctSplit($a);

                if ($data['LastStatus'] !== 'Failed' && $data['AmountCap'] < $amount * 100) {
                    $data = [
                        'MerchantID' => $transaction->getMerchantId(),
                        'PayID' => $transaction->getPayId(),
                        'TransID' => $transaction->getTransactionReference(),
                        'MAC' => $this->dataBnp->ctHMAC(
                            $transaction->getMerchantId(),
                            $amount * 100,
                            $payment->getOrder()->getOrderCurrencyCode(),
                            $merchantAccount->getHmacKey(),
                            $transaction->getPayId(),
                            $transaction->getTransactionReference()
                        ),
                        'Amount' => $amount * 100,
                        'Currency' => $payment->getOrder()->getOrderCurrencyCode(),
                        'RefNr' => $transaction->getRefNr()
                    ];
                    
                    $this->curl->post(self::URL_CAPTURE, $data);
                    $this->curl->getBody();

                    $payment->setTransactionId($payment->getId());
                    $payment->setLastTransId($payment->getId());
                    $payment->setTransactionAdditionalInfo(
                            \Magento\Sales\Model\Order\Payment\Transaction::RAW_DETAILS, (array)$transaction->getData()
                    );
                } else {
                    $order = $payment->getOrder();
                    $order->addStatusHistoryComment(__('The order has already been captured from Axepta BNP Paribas. - Invoice created'))
                    ->setIsCustomerNotified(false);
                    $this->messageManager->addWarning(__("The order has already been captured from Axepta BNP Paribas."));
                    $this->addNotification($order);
                }
            }
        }
        parent::capture($payment, $amount);
    }

    /**
     * Assign data to info model instance
     *
     * @param array|\Magento\Framework\DataObject $data
     * @return $this
     * @throws \Magento\Framework\Exception\LocalizedException
     * @api
     */
    public function assignData(\Magento\Framework\DataObject $data, $recurring = false)
    {
        parent::assignData($data);
        $paymentInfos = $this->getInfoInstance();

        if ($recurring) {
            $additionalData = $data->getAdditionalData();
            $additionalData['operation'] = 'payment';
            $data->setAdditionalData($additionalData);
        } else {
            // Security check
            if ($data['method'] != self::METHOD_CODE && $recurring === false) {
                throw new \Exception(__('Axepta BNP Paribas Error %1', '1'));
            }
        }
        $this->helper->getFormatMethodOperationParams($data, $paymentInfos);

        return $this;
    }

    /**
     * Check whether payment method can be used
     *
     * @param \Magento\Quote\Api\Data\CartInterface|null $quote
     * @return bool
     * @deprecated 100.2.0
     */
    public function isAvailable(\Magento\Quote\Api\Data\CartInterface $quote = null)
    {
        $products = $quote->getAllVisibleItems();
        foreach ($products as $product) {
            if ($this->helper->isReccuringProduct($product)) {
                return false;
            }
        }
        return true;
    }

    public function refund(\Magento\Payment\Model\InfoInterface $payment, $amount)
    {
        parent::refund($payment, $amount);

        $orderId = $payment->getOrder()->getId();

        $transactions = $this->transactionFactory->create();
        $transaction = $transactions->getCollection()->addFieldToFilter('order_id', ['eq' => $orderId])->getFirstItem();

        $method = $payment->getMethod();

        if ($method != 'axepta') {
            return true;
        }

        $array = [
            'MerchantID' => $transaction->getMerchantId(),
            'TransID' => $transaction->getTransactionReference(),
            'PayID' => $transaction->getPayId()
        ];
        

        $plaintext = $this->dataBnp->getApiPlainText($array);

        $len = mb_strlen($plaintext);

        $merchantAccounts = $this->merchantAccountFactory->create();
        $merchantAccount = $merchantAccounts->getCollection()->addFieldToFilter('merchant_id', ['eq' => $transaction->getMerchantId()])->getFirstItem();

        $data = [
            'Data' => $this->dataBnp->ctEncrypt($plaintext, $len, $merchantAccount->getMerchantIdPassword()),
            'MerchantID' => $transaction->getMerchantId(),
            'Len' => $len
        ];
        
        $this->curl->post(self::URL_INQUIRE_WITH_PAYID, $data);

        $response = $this->curl->getBody();
        
        $content = explode('&', $response ?? '');
        
        $content = $this->dataBnp->ctSplit($content);

        $content = $this->dataBnp->ctDecrypt($content['Data'], $content['Len'], $merchantAccount->getMerchantIdPassword());
        
        $a = explode('&', $content ?? '');
        
        $data = $this->dataBnp->ctSplit($a);

        if ($data['Code'] != '00000000') {
            throw new \Magento\Framework\Validator\Exception(__('You cannot refund this order.'));
        }

        $operation = 'refund';

        if ((int)$data['AmountCap'] == 0) {
            if ((int)$data['AmountAuth'] == $amount * 100) {
                $operation = 'cancellation';
            }
        }

        $date_transaction = new \DateTime($transaction->getTransaction_date());
        $date_now = new \DateTime('now');
        $diff = $date_transaction->diff($date_now);

        $nb_months = $diff->m;

        if ($nb_months > 11) {
            throw new \Magento\Framework\Validator\Exception(__('This order is too old to be refunded.'));
        }

        $additionalInformation = $payment->getAdditionalInformation()['axeptaMethodData'];

        if (isset($additionalInformation['code'])) {
            $trigram = $additionalInformation['code'];
        } else {
            // oneclick
            $trigram = $this->methodFactory->create()->load($additionalInformation['method_id'])->getCode();
        }

        $url = $this->dataBnp->getUrlByTrigramAndOperation($trigram, $operation);

        if (empty($url)) {
            $operation = 'refund';
            $url = $this->dataBnp->getUrlByTrigramAndOperation($trigram, $operation);
        }

        $currency = $payment->getOrder()->getOrderCurrencyCode();

        $data = $this->getEncodeParams($transaction, $merchantAccount, $amount * 100, $currency);
        
        $this->curl->post($url, $data);
        $response = $this->curl->getBody();

        if ($response === false) {
            throw new \Magento\Framework\Validator\Exception(__('There are a problem with Axepta BNP Paribas Refund.'));
        }

        $response = $this->curl->getBody();

        $content = explode('&', $response ?? '');
        $content = $this->dataBnp->ctSplit($content);

        $decryptedData = $this->dataBnp->ctDecrypt($content['Data'], $content['Len'], $merchantAccount->getMerchantIdPassword());

        $a = explode('&', $decryptedData ?? '');
        
        $data = $this->dataBnp->ctSplit($a);

        if ($data['Code'] != '00000000') {
            throw new \Magento\Framework\Validator\Exception(__('This order cannot be refunded.'));
        }

        $transactionFactory = $this->transactionFactory->create();
        $transactionFactory->setData([
            'order_id' => $transaction->getOrderId(),
            'merchant_id' => $data['mid'],
            'transaction_reference' => $data['TransID'],
            'transaction_date' => date('Y-m-d H:i:s'),
            'pay_id' => $data['PayID'],
            'bid' => null,
            'transaction_type' => __($operation),
            'xid' => $data['XID'],
            'payment_mean_brand' => null,
            'payment_mean_type' => null,
            'response_code' => $data['Code'],
            'pcnr' => null,
            'ccexpiry' => null,
            'status' => $data['Status'],
            'description' => $data['Description'],
            'amount' => '-' . $amount,
            'raw_data' => $decryptedData
        ]);
        $transactionFactory->save();
    }

    public function getEncodeParams($transaction, $merchantAccount, $amount, $currency)
    {
        $array = [
            'MerchantID' => $transaction->getMerchantId(),
            'PayID' => $transaction->getPayId(),
            'TransID' => $transaction->getTransactionReference(),
            'Amount' => $amount,
            'Currency' => $currency,
            'MAC' => $this->dataBnp->ctHMAC(
                $transaction->getMerchantId(),
                $amount,
                $currency,
                $merchantAccount->getHmacKey(),
                $transaction->getPayId(),
                $transaction->getTransactionReference()
            )
        ];

        $plaintext = $this->dataBnp->getApiPlainText($array);

        $len = mb_strlen($plaintext);

        return [
            'Data' => $this->dataBnp->ctEncrypt($plaintext, $len, $merchantAccount->getMerchantIdPassword()),
            'MerchantID' => $transaction->getMerchantId(),
            'Len' => $len
        ];
    }

    public function addNotification($order)
    {
        $notificationUrl = $this->backendUrlManager->getUrl('sales/order/view', ['order_id' => $order->getId()]);
        $orderIncrmentId = $order->getIncrementId();

        $this->notifierPool->addCritical(
            'Axepta order already captured',
            'Order #' . $orderIncrmentId .' has already been captured from Axepta.',
            $notificationUrl
        );
    }
}
