<?php

namespace Bnpparibas\Axepta\Observer;

use Magento\Framework\Event\ObserverInterface;

class SaveRecurringProductOptions implements ObserverInterface
{

    protected $context;
    protected $request;
    protected $recurringPaymentFactory;

    /**
     * @param \Magento\Sales\Model\OrderFactory $orderModel
     * @param \Magento\Sales\Model\Order\Email\Sender\OrderSender $orderSender
     * @param \Magento\Checkout\Model\Session $checkoutSession
     *
     * @codeCoverageIgnore
     */
    public function __construct(
        \Magento\Backend\Block\Widget\Context $context,
        \Bnpparibas\Axepta\Model\RecurringPaymentFactory $recurringPaymentFactory
    ) {
        $this->context = $context;
        $this->request = $context->getRequest();
        $this->recurringPaymentFactory = $recurringPaymentFactory;
    }

    /**
     * @param \Magento\Framework\Event\Observer $observer
     * @return void
     */
    public function execute(\Magento\Framework\Event\Observer $observer)
    {
        $params = $this->request->getParams();
        $product = $observer->getEvent()->getProduct();
        $productId = $product->getId();
        $customFieldData = $params['axepta_recurring_data'];
        
        $recurringPayments = $this->recurringPaymentFactory->create();
        $recurringPayment =  $recurringPayments->getCollection()->addFieldToFilter('product_id', ['eq' => $productId])->getFirstItem();

        $recurringPaymentSave = $this->recurringPaymentFactory->create();
        $dataToSet = [
            'type' => $customFieldData['axepta_type'],
            'periodicity' => $customFieldData['periodicity'],
            'occurence_interval' => (int) $customFieldData['occurence_interval'],
            'recurring_amount' => (float) $customFieldData['recurring_amount'],
        ];


        if ($recurringPayment !== null && $recurringPayment->getProductId() !== null) {
            $recurringPaymentSave = $recurringPayment;
        } else {
            $dataToSet['product_id'] = $productId;
        }
        $recurringPaymentSave->addData($dataToSet);
        $recurringPaymentSave->save();
    }
}
