<?php

/**
 * 1997-2019 Quadra Informatique
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0) that is available
 * through the world-wide-web at this URL: http://www.opensource.org/licenses/OSL-3.0
 * If you are unable to obtain it through the world-wide-web, please send an email
 * to modules@quadra-informatique.fr so we can send you a copy immediately.
 *
 * @author    Quadra Informatique <modules@quadra-informatique.fr>
 * @copyright 1997-2019 Quadra Informatique
 * @license   http://www.opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 *
 * @category    Bnp
 * @package     Bnpparibas_Axepta
 */

namespace Bnpparibas\Axepta\Setup;

use Magento\Framework\Setup\InstallSchemaInterface;
use Magento\Framework\Setup\ModuleContextInterface;
use Magento\Framework\Setup\SchemaSetupInterface;

class InstallSchema implements InstallSchemaInterface
{

    protected $xml;

    public function __construct(\Bnpparibas\Axepta\Model\Api\Xml $xml)
    {
        $this->xml = $xml;
    }

    public function install(SchemaSetupInterface $setup, ModuleContextInterface $context)
    {
        
        $installer = $setup;
        $installer->startSetup();
        /*****************************************
         * CREATE TABLE DICTIONNARY
         *************************************** */
        $installer->getConnection()->createTable($installer->getConnection()
                        ->newTable($installer->getTable('axepta_xml_paygate_action_code'))
                        ->addColumn('code', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 11, ['nullable' => false])
                        ->addColumn('message', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 255, ['default' => null])
                        ->addColumn('description', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 255, ['default' => null]));

        
        $installer->getConnection()->createTable($installer->getConnection()
                        ->newTable($installer->getTable('axepta_xml_paygate_category_code'))
                        ->addColumn('code', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 11, ['nullable' => false])
                        ->addColumn('message', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 255, ['default' => null])
                        ->addColumn('description', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 255, ['default' => null]));


        $installer->getConnection()->createTable($installer->getConnection()
                        ->newTable($installer->getTable('axepta_xml_paygate_detail_code'))
                        ->addColumn('code', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 11, ['nullable' => false])
                        ->addColumn('message', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 255, ['default' => null])
                        ->addColumn('description', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 255, ['default' => null]));

        /*****************************************
         * CREATE TABLE METHODS
         ****************************************/
        $installer->getConnection()->createTable($installer->getConnection()
                        ->newTable($installer->getTable('axepta_xml_parameter'))
                        ->addColumn('id', \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER, 11, ['identity' => true, 'nullable' => false, 'primary' => true])
                        ->addColumn('parameter_set_id', \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER, 11, ['nullable' => false])
                        ->addColumn('name', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 255, ['default' => null])
                        ->addColumn('format', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 255, ['default' => null])
                        ->addColumn('required', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 255, ['default' => null]));


        $installer->getConnection()->createTable($installer->getConnection()
                        ->newTable($installer->getTable('axepta_xml_method'))
                        ->addColumn('id', \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER, 11, ['identity' => true, 'nullable' => false, 'primary' => true])
                        ->addColumn('code', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 255, ['default' => null])
                        ->addColumn('libelle', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 255, ['default' => null]));


        $installer->getConnection()->createTable($installer->getConnection()
                        ->newTable($installer->getTable('axepta_xml_method_lang'))
                        ->addColumn('id', \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER, 11, ['identity' => true, 'nullable' => false, 'primary' => true])
                        ->addColumn('method_id', \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER, 11, ['nullable' => false])
                        ->addColumn('iso', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 255, ['default' => null])
                        ->addColumn('label', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 255, ['default' => null])
                        ->addForeignKey($installer->getFkName('axepta_xml_method_lang', 'method_id', 'axepta_xml_method', 'id'), 'method_id', $installer->getTable('axepta_xml_method'), 'id', \Magento\Framework\DB\Ddl\Table::ACTION_CASCADE));


        $installer->getConnection()->createTable($installer->getConnection()
                        ->newTable($installer->getTable('axepta_xml_method_operation'))
                        ->addColumn('id', \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER, 11, ['identity' => true, 'nullable' => false, 'primary' => true])
                        ->addColumn('method_id', \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER, 11, ['nullable' => false])
                        ->addColumn('code', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 255, ['default' => null])
                        ->addColumn('parameter_set_id', \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER, 11, ['nullable' => false])
                        ->addColumn('url', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 255, ['default' => null])
                        ->addColumn('test_url', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 255, ['default' => null])
                        ->addForeignKey($installer->getFkName('axepta_xml_method_operation_method_id', 'method_id', 'axepta_xml_method', 'id'), 'method_id', $installer->getTable('axepta_xml_method'), 'id', \Magento\Framework\DB\Ddl\Table::ACTION_CASCADE)
                        ->addForeignKey($installer->getFkName('axepta_xml_method_operation_parameter_set_id', 'parameter_set_id', 'axepta_xml_parameter', 'parameter_set_id'), 'parameter_set_id', $installer->getTable('axepta_xml_parameter'), 'id', \Magento\Framework\DB\Ddl\Table::ACTION_CASCADE));


        $installer->getConnection()->createTable($installer->getConnection()
                        ->newTable($installer->getTable('axepta_xml_method_allowed_country'))
                        ->addColumn('id', \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER, 11, ['identity' => true, 'nullable' => false, 'primary' => true])
                        ->addColumn('method_id', \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER, 11, ['nullable' => false])
                        ->addColumn('currency', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 255, ['default' => null])
                        ->addColumn('country', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 255, ['default' => null])
                        ->addIndex($installer->getFkName('axepta_xml_method_allowed_country', 'method_id', 'axepta_xml_method', 'id'), 'method_id', $installer->getTable('axepta_xml_method'), 'id', \Magento\Framework\DB\Ddl\Table::ACTION_CASCADE));


        $installer->getConnection()->createTable($installer->getConnection()
                        ->newTable($installer->getTable('axepta_merchant_account'))
                        ->addColumn('id', \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER, 11, ['identity' => true, 'nullable' => false, 'primary' => true])
                        ->addColumn('activation_key', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 255, ['default' => null])
                        ->addColumn('merchant_id', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 255, ['default' => null])
                        ->addColumn('merchant_id_password', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 255, ['default' => null])
                        ->addColumn('hmac_key', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 255, ['default' => null])
                        ->addColumn('store_id', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 10, ['default' => null])
                        ->addColumn('country_id', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 255, ['default' => null])
                        ->addColumn('currency_id', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 10, ['default' => null])
                        ->addColumn('display_method', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 255, ['default' => null])
                        ->addColumn('capture_mode', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 255, ['default' => null])
                        ->addColumn('capture_time', \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER, 11, ['default' => null])
                        ->addColumn('oneclick_payment_active', \Magento\Framework\DB\Ddl\Table::TYPE_BOOLEAN, null, ['default' => false])
                        ->addColumn('3d_secure_active', \Magento\Framework\DB\Ddl\Table::TYPE_BOOLEAN, null, ['default' => false])
                        ->addColumn('recurring_payment_active', \Magento\Framework\DB\Ddl\Table::TYPE_BOOLEAN, null, ['default' => false])
                        ->addColumn('is_active', \Magento\Framework\DB\Ddl\Table::TYPE_BOOLEAN, null, ['default' => null]));


        $installer->getConnection()->createTable($installer->getConnection()
                        ->newTable($installer->getTable('axepta_merchant_account_paygate_configuration'))
                        ->addColumn('id', \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER, 11, ['identity' => true, 'nullable' => false, 'primary' => true])
                        ->addColumn('merchant_account_id', \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER, 11, ['nullable' => false])
                        ->addColumn('method_id', \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER, 11, ['nullable' => false])
                        ->addColumn('is_active', \Magento\Framework\DB\Ddl\Table::TYPE_BOOLEAN, null, ['default' => null])
                        ->addForeignKey($installer->getFkName('axepta_merchant_account_paygate_configuration', 'merchant_account_id', 'axepta_merchant_account', 'id'), 'merchant_account_id', $installer->getTable('axepta_merchant_account'), 'id', \Magento\Framework\DB\Ddl\Table::ACTION_CASCADE)
                        ->addForeignKey($installer->getFkName('axepta_merchant_account_paygate_configuration', 'method_id', 'axepta_xml_method', 'id'), 'method_id', $installer->getTable('axepta_xml_method'), 'id', \Magento\Framework\DB\Ddl\Table::ACTION_CASCADE));


        $installer->getConnection()->createTable($installer->getConnection()
                        ->newTable($installer->getTable('axepta_customer_oneclick_payment_card'))
                        ->addColumn('id', \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER, 11, ['identity' => true, 'nullable' => false, 'primary' => true])
                        ->addColumn('method_id', \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER, 11, ['nullable' => false])
                        ->addColumn('customer_id', \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER, 10, ['unsigned' => true, 'nullable' => false, 'index' => true])
                        ->addColumn('pcnr', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 255, ['default' => null])
                        ->addColumn('ccexpiry', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 255, ['default' => null])
                        ->addColumn('ccbrand', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 255, ['default' => null])
                        ->addForeignKey($installer->getFkName('axepta_customer_oneclick_payment_card_method_id', 'method_id', 'axepta_xml_method', 'id'), 'method_id', $installer->getTable('axepta_xml_method'), 'id', \Magento\Framework\DB\Ddl\Table::ACTION_CASCADE)
                        ->addForeignKey($installer->getFkName('axepta_customer_oneclick_payment_card', 'customer_id', 'customer_entity', 'entity_id'), 'customer_id', $installer->getTable('customer_entity'), 'entity_id', \Magento\Framework\DB\Ddl\Table::ACTION_CASCADE));


        $installer->getConnection()->createTable($installer->getConnection()
                        ->newTable($installer->getTable('axepta_transaction'))
                        ->addColumn('id', \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER, 11, ['identity' => true, 'nullable' => false, 'primary' => true])
                        ->addColumn('order_id', \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER, 10, ['unsigned' => true, 'nullable' => false, 'index' => true])
                        ->addColumn('merchant_id', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 255, ['default' => null])
                        ->addColumn('transaction_reference', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 255, ['default' => null])
                        ->addColumn('transaction_date', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 255, ['default' => null])
                        ->addColumn('pay_id', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 255, ['default' => null])
                        ->addColumn('bid', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 255, ['default' => null])
                        ->addColumn('xid', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 255, ['default' => null])
                        ->addColumn('transaction_type', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 255, ['default' => null])
                        ->addColumn('payment_mean_brand', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 255, ['default' => null])
                        ->addColumn('payment_mean_type', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 255, ['default' => null])
                        ->addColumn('response_code', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 255, ['default' => null])
                        ->addColumn('pcnr', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 255, ['default' => null])
                        ->addColumn('ccexpiry', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 255, ['default' => null])
                        ->addColumn('status', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 255, ['default' => null])
                        ->addColumn('description', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 255, ['default' => null])
                        ->addColumn('amount', \Magento\Framework\DB\Ddl\Table::TYPE_DECIMAL, [10, 4], ['default' => null])
                        ->addColumn('raw_data', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, null, ['default' => null])
                        ->addForeignKey($installer->getFkName('axepta_transaction', 'order_id', 'sales_order', 'entity_id'), 'order_id', $installer->getTable('sales_order'), 'entity_id', \Magento\Framework\DB\Ddl\Table::ACTION_CASCADE));

        
        $installer->getConnection()->createTable($installer->getConnection()
                        ->newTable($installer->getTable('axepta_customer_recurring_payment'))
                        ->addColumn('id', \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER, 11, ['identity' => true, 'nullable' => false, 'primary' => true])
                        ->addColumn('order_id', \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER, 10, ['unsigned' => true, 'nullable' => false, 'index' => true])
                        ->addColumn('customer_id', \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER, 10, ['unsigned' => true, 'nullable' => false, 'index' => true])
                        ->addColumn('product_id', \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER, 10, ['unsigned' => true, 'nullable' => false, 'index' => true])
                        ->addColumn('transaction_id', \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER, 11, ['nullable' => false])
                        ->addColumn('status', \Magento\Framework\DB\Ddl\Table::TYPE_SMALLINT, 1, ['default' => 1])
                        ->addColumn('periodicity', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 10, ['default' => null])
                        ->addColumn('occurence_interval', \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER, 10, ['default' => null])
                        ->addColumn('current_occurence', \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER, 10, ['default' => null])
                        ->addColumn('date_add', \Magento\Framework\DB\Ddl\Table::TYPE_DATETIME, 10, ['default' => null])
                        ->addColumn('last_schedule', \Magento\Framework\DB\Ddl\Table::TYPE_DATETIME, 10, ['default' => null])
                        ->addColumn('next_schedule', \Magento\Framework\DB\Ddl\Table::TYPE_DATETIME, 10, ['default' => null])
                        ->addColumn('current_specific_price', \Magento\Framework\DB\Ddl\Table::TYPE_DECIMAL, [10, 2], ['default' => null])
                        ->addForeignKey($installer->getFkName('axepta_customer_recurring_payment', 'order_id', 'sales_order', 'entity_id'), 'order_id', $installer->getTable('sales_order'), 'entity_id', \Magento\Framework\DB\Ddl\Table::ACTION_CASCADE)
                        ->addForeignKey($installer->getFkName('axepta_customer_recurring_payment', 'customer_id', 'customer_entity', 'entity_id'), 'customer_id', $installer->getTable('customer_entity'), 'entity_id', \Magento\Framework\DB\Ddl\Table::ACTION_CASCADE)
                        ->addForeignKey($installer->getFkName('axepta_customer_recurring_payment', 'product_id', 'catalog_product_entity', 'entity_id'), 'product_id', $installer->getTable('catalog_product_entity'), 'entity_id', \Magento\Framework\DB\Ddl\Table::ACTION_CASCADE)
                        ->addForeignKey($installer->getFkName('axepta_customer_recurring_payment', 'transaction_id', 'axepta_transaction', 'id'), 'transaction_id', $installer->getTable('axepta_transaction'), 'id', \Magento\Framework\DB\Ddl\Table::ACTION_CASCADE));
                        
        
        $installer->getConnection()->createTable($installer->getConnection()
                        ->newTable($installer->getTable('axepta_recurring_payment'))
                        ->addColumn('id', \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER, 11, ['identity' => true, 'nullable' => false, 'primary' => true])
                        ->addColumn('product_id', \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER, 10, ['unsigned' => true, 'nullable' => false, 'index' => true])
                        ->addColumn('periodicity', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 10, ['default' => null])
                        ->addColumn('occurence_interval', \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER, 10, ['default' => null])
                        ->addColumn('recurring_amount', \Magento\Framework\DB\Ddl\Table::TYPE_DECIMAL, [10, 2], ['default' => null])
                        ->addForeignKey($installer->getFkName('axepta_recurring_payment', 'product_id', 'catalog_product_entity', 'entity_id'), 'product_id', $installer->getTable('catalog_product_entity'), 'entity_id', \Magento\Framework\DB\Ddl\Table::ACTION_CASCADE));

        $installer->getConnection()->createTable($installer->getConnection()
                        ->newTable($installer->getTable('axepta_xml_paygate_position'))
                        ->addColumn('id', \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER, 11, ['identity' => true, 'nullable' => false, 'primary' => true, 'auto_increment' => true])
                        ->addColumn('position', \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER, 11, ['nullable' => false])
                        ->addColumn('iso', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 3, ['default' => null])
                        ->addColumn('trigram', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 3, ['nullable' => false]));
                        
        // Update XML
        $this->xml->importXml();
    }
}
