<?php
/**
 * 1997-2019 Quadra Informatique.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0) that is available
 * through the world-wide-web at this URL: http://www.opensource.org/licenses/OSL-3.0
 * If you are unable to obtain it through the world-wide-web, please send an email
 * to modules@quadra-informatique.fr so we can send you a copy immediately.
 *
 * @author    Quadra Informatique <modules@quadra-informatique.fr>
 * @copyright 1997-2019 Quadra Informatique
 * @license   http://www.opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 *
 * @category    Bnp
 */

namespace Bnpparibas\Axepta\Setup;

use Magento\Framework\Setup\ModuleContextInterface;
use Magento\Framework\Setup\SchemaSetupInterface;
use Magento\Framework\Setup\UpgradeSchemaInterface;

class UpgradeSchema implements UpgradeSchemaInterface
{
    protected $xml;

    public function __construct(\Bnpparibas\Axepta\Model\Api\Xml $xml)
    {
        $this->xml = $xml;
    }

    public function upgrade(SchemaSetupInterface $setup, ModuleContextInterface $context)
    {
        $setup->startSetup();

        if (version_compare($context->getVersion(), '0.0.2', '<')) {
            // Pass activation to TEXT
            $setup->getConnection()->changeColumn(
                $setup->getTable('axepta_merchant_account'),
                'activation_key',
                'activation_key',
                [
                    'type' => \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    'length' => null,
                ]
            );

            // add payment type
            $setup->getConnection()->addColumn(
                $setup->getTable('axepta_recurring_payment'),
                'type',
                [
                    'type' => \Magento\Framework\DB\Ddl\Table::TYPE_SMALLINT,
                    'length' => 10,
                    'comment' => 'Payment type',
                ]
            );
        }

        if (version_compare($context->getVersion(), '0.0.4', '<')) {
            // add cetelem category
            $setup->getConnection()->addColumn(
                $setup->getTable('axepta_merchant_account'),
                'cetelem_category',
                [
                    'type' => \Magento\Framework\DB\Ddl\Table::TYPE_SMALLINT,
                    'length' => 10,
                    'comment' => 'Cetelem Category',
                ]
            );
        }

        if (version_compare($context->getVersion(), '1.0.8', '<')) {
            // add cetelem category
            $setup->getConnection()->addColumn(
                $setup->getTable('axepta_merchant_account'),
                'method_name',
                [
                    'type' => \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    'length' => 255,
                    'comment' => 'Method Name',
                ]
            );
        }

        if (version_compare($context->getVersion(), '1.0.20', '<')) {
            $setup->getConnection()->changeColumn(
                $setup->getTable('axepta_transaction'),
                'transaction_date',
                'transaction_date',
                [
                    'type' => \Magento\Framework\DB\Ddl\Table::TYPE_DATETIME,
                    'length' => 10,
                    'comment' => 'Transaction_date',
                ]
            );

            $setup->getConnection()->addColumn(
                $setup->getTable('axepta_transaction'),
                'ref_nr',
                [
                    'type' => \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    'length' => 32,
                    'comment' => 'RefNr based on quote ID',
                    'default' => null,
                ]
            );
        }

        if (version_compare($context->getVersion(), '1.0.21', '<')) {
            $setup->getConnection()->addColumn(
                $setup->getTable('axepta_transaction'),
                'scheme_reference_id',
                [
                    'type' => \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    'length' => 128,
                    'comment' => 'Scheme Reference ID used for the chain (subscription)',
                    'default' => null,
                ]
            );

            $setup->getConnection()->addColumn(
                $setup->getTable('axepta_customer_recurring_payment'),
                'scheme_reference_id',
                [
                    'type' => \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    'length' => 128,
                    'comment' => 'Scheme Reference ID used for the chain (subscription)',
                    'default' => null,
                ]
            );
        }

        if (version_compare($context->getVersion(), '1.0.23', '<')) {
            $setup->getConnection()->addColumn(
                $setup->getTable('axepta_customer_oneclick_payment_card'),
                'holder_name',
                [
                    'type' => \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    'length' => 128,
                    'comment' => 'Holder name',
                    'default' => null,
                ]
            );

            $setup->getConnection()->addColumn(
                $setup->getTable('axepta_transaction'),
                'holder_name',
                [
                    'type' => \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    'length' => 128,
                    'comment' => 'Holder name',
                    'default' => null,
                ]
            );
        }

        if (version_compare($context->getVersion(), '1.0.24', '<')) {
            $setup->getConnection()->addColumn(
                $setup->getTable('axepta_merchant_account'),
                '3d_secure_active_amount',
                [
                    'type' => \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    'length' => 255,
                    'comment' => '3d_secure_active_amount',
                    'default' => null,
                ]
            );
        }

        if (version_compare($context->getVersion(), '1.1.0', '<')) {
            if (!$setup->getConnection()->isTableExists('axepta_xml_paygate_position')) {
                $setup->getConnection()->createTable($setup->getConnection()
                ->newTable($setup->getTable('axepta_xml_paygate_position'))
                ->addColumn('id', \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER, 11, ['identity' => true, 'nullable' => false, 'primary' => true, 'auto_increment' => true])
                ->addColumn('position', \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER, 11, ['nullable' => false])
                ->addColumn('iso', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 3, ['default' => null])
                ->addColumn('trigram', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 3, ['nullable' => false]));

                $this->xml->importXml();
            }
        }

        if (version_compare($context->getVersion(), '1.2.4', '<')) {
            $this->xml->importXml();
        }

        if (version_compare($context->getVersion(), '1.5.5', '<')) {
            $this->xml->importXml();
        }

        if (version_compare($context->getVersion(), '1.5.6', '<')) {
            $this->xml->importXml();
        }

        $setup->endSetup();
    }
}
