<?php

namespace Bnpparibas\Axepta\Ui\DataProvider\Product\Form\Modifier;

use Magento\Catalog\Ui\DataProvider\Product\Form\Modifier\AbstractModifier;
use Magento\Ui\Component\Form\Fieldset;
use Magento\Ui\Component\Form\Field;
use Magento\Ui\Component\Form\Element\Select;
use Magento\Ui\Component\Form\Element\Input;
use Magento\Ui\Component\Form\Element\DataType\Text;
use Magento\Ui\Component\Form\Element\DataType\Number;

class Fields extends AbstractModifier
{

    protected $locator;
    protected $recurringPaymentFactory;

    public function __construct(
        \Magento\Catalog\Model\Locator\LocatorInterface $locator,
        \Bnpparibas\Axepta\Model\RecurringPaymentFactory $recurringPaymentFactory
    ) {
        $this->locator = $locator;
        $this->recurringPaymentFactory = $recurringPaymentFactory;
    }

    public function modifyMeta(array $meta)
    {
        $meta = array_replace_recursive($meta, [
            'axepta_recurring_data' => [
                'arguments' => [
                    'data' => [
                        'config' => [
                            'label' => __('Axepta BNP Paribas recurring data options'),
                            'collapsible' => true,
                            'componentType' => Fieldset::NAME,
                            'dataScope' => 'data.axepta_recurring_data',
                            'sortOrder' => 1
                        ],
                    ],
                ],
                'children' => $this->getFields()
            ],
        ]);

        return $meta;
    }

    public function modifyData(array $data)
    {
        $product = $this->locator->getProduct();
        $productId = $product->getId();

        $recurringPayments = $this->recurringPaymentFactory->create();
        $recurringPayment = $recurringPayments->getCollection()->addFieldToFilter('product_id', ['eq' => $productId])->getFirstItem();
        if (!empty($recurringPayment->getData())) {
            $data[strval($productId)]['axepta_recurring_data']['axepta_type'] = $recurringPayment->getType();
            $data[strval($productId)]['axepta_recurring_data']['periodicity'] = $recurringPayment->getPeriodicity();
            $data[strval($productId)]['axepta_recurring_data']['occurence_interval'] = $recurringPayment->getOccurenceInterval();
            $data[strval($productId)]['axepta_recurring_data']['recurring_amount'] = $recurringPayment->getRecurringAmount();
        }

        return $data;
    }

    protected function getFields()
    {
        return [
            'axepta_type' => [
                'arguments' => [
                    'data' => [
                        'config' => [
                            'label' => __('Type'),
                            'componentType' => Field::NAME,
                            'formElement' => Select::NAME,
                            'dataScope' => 'axepta_type',
                            'dataType' => Text::NAME,
                            'sortOrder' => 5,
                            'options' => [
                                ['value' => '0', 'label' => __('Simple Payment')],
                                ['value' => '1', 'label' => __('Subscription Payment')]
                            ],
                            'switcherConfig' => [
                                'rules' => [
                                    '0' => [
                                        'value' => 0,
                                        'actions' => [
                                            '0' => [
                                                'target' => 'product_form.product_form.axepta_recurring_data.periodicity',
                                                'callback' => 'disable'
                                            ],
                                            '1' => [
                                                'target' => 'product_form.product_form.axepta_recurring_data.occurence_interval',
                                                'callback' => 'disable'
                                            ],
                                            '2' => [
                                                'target' => 'product_form.product_form.axepta_recurring_data.recurring_amount',
                                                'callback' => 'disable'
                                            ]
                                        ]
                                    ],
                                    '1' => [
                                        'value' => 1,
                                        'actions' => [
                                            '0' => [
                                                'target' => 'product_form.product_form.axepta_recurring_data.periodicity',
                                                'callback' => 'enable'
                                            ],
                                            '1' => [
                                                'target' => 'product_form.product_form.axepta_recurring_data.occurence_interval',
                                                'callback' => 'enable'
                                            ],
                                            '2' => [
                                                'target' => 'product_form.product_form.axepta_recurring_data.recurring_amount',
                                                'callback' => 'enable'
                                            ]
                                        ]
                                    ]
                                ],
                                'enabled' => true
                            ]
                        ],
                    ],
                ],
            ],
            'periodicity' => [
                'arguments' => [
                    'data' => [
                        'config' => [
                            'label' => __('Periodicity'),
                            'componentType' => Field::NAME,
                            'formElement' => Select::NAME,
                            'dataScope' => 'periodicity',
                            'dataType' => Text::NAME,
                            'sortOrder' => 10,
                            'options' => [
                                ['value' => 'm', 'label' => __('Month')],
                                ['value' => 'd', 'label' => __('Day')]
                            ],
                        ],
                    ],
                ],
            ],
            'occurence_interval' => [
                'arguments' => [
                    'data' => [
                        'config' => [
                            'label' => __('Interval occurence'),
                            'componentType' => Field::NAME,
                            'formElement' => Input::NAME,
                            'dataScope' => 'occurence_interval',
                            'dataType' => Number::NAME,
                            'sortOrder' => 20
                        ],
                    ],
                ],
            ],
            'recurring_amount' => [
                'arguments' => [
                    'data' => [
                        'config' => [
                            'label' => __('Recurring amount'),
                            'componentType' => Field::NAME,
                            'formElement' => Input::NAME,
                            'dataScope' => 'recurring_amount',
                            'dataType' => Number::NAME,
                            'sortOrder' => 30
                        ],
                    ],
                ],
            ]
        ];
    }
}
