
define([
    'jquery',
    'Magento_Checkout/js/view/payment/default',
    'ko',
    'mage/url',
    'Magento_Checkout/js/model/full-screen-loader',
    'Magento_Customer/js/model/customer',
    'Magento_Checkout/js/model/quote'
], function ($, Component, ko, url, fullScreenLoader, customer, quote) {
    'use strict';

    $(document).on('click', '#axepta-payment-method .more-option-btn', function(){
        var is_selected = $(this).hasClass('selected');
        
        if (is_selected) {
            $(this).removeClass('selected');
            $('#hidden-method').slideUp("500");
            $('#axepta-payment-method .more-option-btn span.more-options').show();
            $('#axepta-payment-method .more-option-btn span.less-options').hide();
        } else {
            $(this).addClass('selected');
            $('#hidden-method').slideDown("500");
            $('#axepta-payment-method .more-option-btn span').html($(this).data('expended-content'));
            $('#axepta-payment-method .more-option-btn span.more-options').hide();
            $('#axepta-payment-method .more-option-btn span.less-options').show();
        }
    });

    return Component.extend({
        displayMethod: ko.observable(),
        methodName: ko.observable(),
        allowedMethods: ko.observableArray([]),
        recurring: ko.observable(),
        oneClick: ko.observable(),
        oneClickUserCards: ko.observableArray([]),
        amountTotal: quote.getTotals(),
        defaults: {
            template: 'Bnpparibas_Axepta/payment/axepta',
            redirectAfterPlaceOrder: false,
            paymentReady: false
        },

        initialize: function () {
            var self = this;

            if(window.checkoutConfig.axepta !== undefined) {
                this.allowedMethods(window.checkoutConfig.axepta.methods);
                this.methodName(window.checkoutConfig.axepta.methodName);
                this.displayMethod(window.checkoutConfig.axepta.displayMethod);
                this.recurring((window.checkoutConfig.axepta !== undefined) ? window.checkoutConfig.axepta.recurringPayment.isActive : false);
                this.oneClick((window.checkoutConfig.axepta !== undefined) ? window.checkoutConfig.axepta.oneClick.isActive : false);
                this.oneClickUserCards((window.checkoutConfig.axepta !== undefined) ? window.checkoutConfig.axepta.oneClick.userCards : false);

                setTimeout(function () {
                    $("#axepta-methods-list").unbind('change');
                    $("#axepta-methods-list").on('change', function () {
                        this.loadPaymentMethods(window.checkoutConfig.axepta);
                    });
                }, 100);
            } else {
                setTimeout(function () {
                    let selectedAddress = quote.billingAddress();
                    console.log(selectedAddress);

                    // Add default country if not set
                    if (!selectedAddress.postcode) {
                        if ($('select[name="country_id"]').find(":selected").val()) {
                            selectedAddress.countryId = $('select[name="country_id"]').find(":selected").val();
                        } else if($('select[name="country_id"]').prop("selectedIndex", 1).val()) {
                            selectedAddress.countryId = $('select[name="country_id"]').prop("selectedIndex", 1).val();
                        } else {
                            selectedAddress.countryId = "FR"; // Default to France if no country is selected or set
                        }
                        
                    }

                    $.ajax({
                        url: url.build('axepta/checkout/ajax/'),
                        type: "POST",
                        dataType: "json",
                        data: {
                            isAjax: true,
                            action: 'getAllowedMethods',
                            countryId: selectedAddress.countryId
                        },
                        success: function (data) {
                            window.checkoutConfig.axepta = data.axepta;
                            self.loadPaymentMethods(data.axepta);
                        }
                    });
                }, 1500);
            }

            this._super();
            return this;
        },
        loadPaymentMethods: function (axepta) {
            this.allowedMethods(axepta.methods);
            this.methodName(axepta.methodName);
            this.displayMethod(axepta.displayMethod);
            this.recurring((axepta !== undefined) ? axepta.recurringPayment.isActive : false);
            this.oneClick((axepta !== undefined) ? axepta.oneClick.isActive : false);
            this.oneClickUserCards((axepta !== undefined) ? axepta.oneClick.userCards : false);
        },
        initObservable: function () {
            this._super().observe('paymentReady');

            return this;
        },

        redirectController: "directlink",
        operation: "payment",
        isOneClick: false,
        parent: null,
        event: null,
        methodData: null,
        getCode: function () {
            return 'axepta';
        },
        
        getTitle: function () {
            return this.methodName();
        },

        isActive: function () {
            if (typeof this.allowedMethods() !== 'undefined' && this.allowedMethods().length > 0) {
                return true;
            }
            return false;
        },

        isLoggedIn: function () {
            return customer.isLoggedIn();
        },
        
        getAllowedMethods: function () {
            if (this.allowedMethods().length > 6) {
                return this.allowedMethods().slice(0, 6);
            }
            return this.allowedMethods();
        },

        getAllowedMethodsHidden: function () {
            return this.allowedMethods().slice(6);
        },

        isOneClickActive: function () {
            return this.oneClick();
        },

        nbCardSaved: function () {
            return this.oneClickUserCards().length;
        },

        getOneclickUserCard: function () {
            return this.oneClickUserCards();
        },

        validate: function () {
            return true;
        },

        getData: function () {
            return {
                'method': this.item.method,
                'po_number': null,
                'additional_data': this.methodData
            };
        },

        count: function () {
            return this.allowedMethods().length;
        },

        saveDataChecked: function (parent, event, methodData) {
            this.parent = parent;
            this.event = event;
            this.methodData = methodData;
            if (methodData.idPcnr) {
                $("#" + methodData.idPcnr).prop("checked", true);
            } else {
                $("#" + methodData.code).prop("checked", true);
            }
            $("#action-checkout").removeClass("disabled");
        },

        placeOrder: function() {
            delete this.methodData.images;

            if ((this.methodData.code === 'BOL' || this.methodData.code === 'PAY') && !this.isLoggedIn()) {
                $("#notice-not-connected-user").show();
                return false;
            }

            if (this.methodData.code === 'BOL' && $('#social-security-number').val() === '') {
                $("#bol-social-security-number").show();
                return false;
            }

            if ($('#social-security-number').val() !== "") {
                this.methodData.socialSecurityNumber = $('#social-security-number').val();
            }

            this.methodData.oneClickRegister = ($('#axepta-oneclick-register').is(':checked')) ? "1" : "0";
            this.methodData.isOneClick = (this.isOneClick) ? "1" : "0";
            this.methodData.operation = this.operation;

            return this._super(this.parent, this.event);
        },

        isIframeDisplay: function () {
            return (this.displayMethod() === "iframe");
        },

        isPaymentReady: function () {
            return this.paymentReady();
        },

        iframeLoaded: function () {
            fullScreenLoader.stopLoader();
        },

        isCustomerGuest: function () {
            return !customer.isLoggedIn();
        },

        afterPlaceOrder: function () {

            if(this.isIframeDisplay) {
                $("#axepta-methods-list-button").hide();
            }

            var redirectUrl = url.build('axepta/payment/' + this.redirectController);
            if (this.isIframeDisplay() && (this.methodData.code !== "SOF" && this.methodData.code !== "FC3" && this.methodData.code !== "FC4" && this.methodData.code !== "PRE" && this.methodData.code !== "PAL" && this.methodData.code !== "KLA")) {
                this.paymentReady(true);
                redirectUrl = redirectUrl + '?CustomField14=iframe'
                $("#" + this.getCode() + '-iframe').attr('src', redirectUrl);
                if (this.iframeIsLoaded) {
                    document.getElementById(this.getCode() + '-iframe').contentWindow.location.reload();
                }
                this.iframeIsLoaded = true;
                this.isPlaceOrderActionAllowed(true);
                fullScreenLoader.stopLoader();
                $('html, body').animate({
                    scrollTop: $("#" + this.getCode() + '-iframe').offset().top
                }, 1000);
            } else {
                window.location.replace(redirectUrl);
            }
        }
    });
});
